@@ -1,36 +1,38 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
+#include "client-config.h"
 
 #include <dnscore/output_stream.h>
 #include <dnscore/counter_output_stream.h>
@@ -40,14 +42,6 @@
 
 //#include message-viewer-dig.h"
 
-#define     VM_WITH_ADDITIONAL                  0x01
-#define     VM_WITH_ANSWER                      0x02
-#define     VM_WITH_AUTHORITY                   0x04
-#define     VM_WITH_QUESTION                    0x08
-#define     VM_WITH_HEADER                      0x10
-#define     VM_WITH_XFR                         0x20
-
-
 static char* message_count_names[4] =
 {
     "QUERY", "ANSWER", "AUTHORITY", "ADDITIONAL"
@@ -76,8 +70,22 @@ message_viewer_dig_start(message_viewer *mv)
 
     /*    ------------------------------------------------------------    */
 
-    osformatln(os, "; ( server found)");         /// @todo 20150709 gve -- still need to find a way to show amount of servers
-    osformatln(os, ";; global options: ");       /// @todo 20150709 gve -- no global options given output_stream                                         *os  = mv->os;
+    if(mv->host != NULL)
+    {
+        // osformatln(os, "<<>> yadifa <<>>"); /: @todo 20200617 edf -- some command line could be printed here
+
+        u32 host_count = host_address_count(mv->host);
+
+        if(host_count == 1)
+        {
+            osformatln(os, "; (1 server found)");
+        }
+        else
+        {
+            osformatln(os, "; (%u servers found)", host_count);
+        }
+    }
+
 }
 
 
@@ -96,15 +104,15 @@ message_viewer_dig_end(message_viewer *mv, long time_duration)
 
     osformat(os,   ";; WHEN: %s", ctime(&timep));
 
-    if(mv->view_mode_with & VM_WITH_XFR)
+    if(mv->view_mode_with & MESSAGE_VIEWER_WITH_XFR)
     {
-        osformatln(os, ";; XFR size: %lu records (messages %lu, bytes %lu)", mv->resource_records_total[1], mv->messages, mv->bytes);
+        osformat(os, ";; XFR size: %lu records (messages %lu, bytes %lu)", mv->resource_records_total[1], mv->messages, mv->bytes);
     }
     else
     {
-        osformatln(os, ";; MSG SIZE rcvd: %ld", mv->bytes);
+        osformat(os, ";; MSG SIZE rcvd: %ld", mv->bytes);
     }
-    osformat(os, "\n");
+    //osformat(os, "\n");
 }
 
 
@@ -114,6 +122,7 @@ message_viewer_dig_header(message_viewer *mv, const u8 *buffer)
     /* 1. get the output stream */
     output_stream *os      = mv->os;
 
+
     /* 2. get values of the different sections: QUESTION, ANSWER, AUTHORITY and ADDITIONAL */
     u16 count[4];
     count[0]               = ntohs(MESSAGE_QD(buffer));
@@ -121,6 +130,7 @@ message_viewer_dig_header(message_viewer *mv, const u8 *buffer)
     count[2]               = ntohs(MESSAGE_NS(buffer));
     count[3]               = ntohs(MESSAGE_AR(buffer));
 
+
     /* 3. add the amount of section resource records into a total */
     message_viewer_resource_record_total_update(mv, count);
 
@@ -137,8 +147,8 @@ message_viewer_dig_header(message_viewer *mv, const u8 *buffer)
 
     u8 rcode               = MESSAGE_RCODE(buffer);
 
-    const char *opcode_txt = get_opcode(opcode);
-    const char *status_txt = get_rcode(rcode);
+    const char *opcode_txt = dns_message_opcode_get_name(opcode);
+    const char *status_txt = dns_message_rcode_get_name(rcode);
 
     mv->section_name       = (opcode != OPCODE_UPDATE)? message_section_names : message_section_update_names;
 
@@ -146,31 +156,30 @@ message_viewer_dig_header(message_viewer *mv, const u8 *buffer)
     /* if no view with header then inmediately return,
      * dig axfr has no header information so --> return
      */
-    if(mv->view_mode_with & VM_WITH_XFR)
+    if(mv->view_mode_with & MESSAGE_VIEWER_WITH_XFR)
     {
         return;
     }
 
-
     /* 6. we have all the information, fill the stream */
     osformat(os, ";; Got answer:\n");
     osformat(os, ";; ->>HEADER<<- opcode: %s, status: %s, id: %hd\n", opcode_txt, status_txt, id);
-    osformat(os, ";; flags: ");
+    osformat(os, ";; flags:");
 
-    if(MESSAGE_QR(buffer) != 0) osprint(os, "qr ");
-    if(MESSAGE_AA(buffer) != 0) osprint(os, "aa ");
-    if(MESSAGE_TC(buffer) != 0) osprint(os, "tc ");
-    if(MESSAGE_RD(buffer) != 0) osprint(os, "rd ");
-    if(MESSAGE_RA(buffer) != 0) osprint(os, "ra ");
-    if(MESSAGE_ZF(buffer) != 0) osprint(os, "zf ");
-    if(MESSAGE_AD(buffer) != 0) osprint(os, "ad ");
-    if(MESSAGE_CD(buffer) != 0) osprint(os, "cd ");
+    if(MESSAGE_QR(buffer) != 0) osprint(os, " qr");
+    if(MESSAGE_AA(buffer) != 0) osprint(os, " aa");
+    if(MESSAGE_TC(buffer) != 0) osprint(os, " tc");
+    if(MESSAGE_RD(buffer) != 0) osprint(os, " rd");
+    if(MESSAGE_RA(buffer) != 0) osprint(os, " ra");
+    if(MESSAGE_ZF(buffer) != 0) osprint(os, " zf");
+    if(MESSAGE_AD(buffer) != 0) osprint(os, " ad");
+    if(MESSAGE_CD(buffer) != 0) osprint(os, " cd");
 
     /* 3. get the names for the presentation */
     char  **count_name;
     count_name             = (opcode != OPCODE_UPDATE)? message_count_names   : message_count_update_names;
 
-    osformat(os, "%s: %hd, %s: %hd, %s: %hd, %s: %hd\n",
+    osformat(os, "; %s: %hd, %s: %hd, %s: %hd, %s: %hd\n",
              count_name[0], count[0],
              count_name[1], count[1],
              count_name[2], count[2],
@@ -182,7 +191,8 @@ message_viewer_dig_header(message_viewer *mv, const u8 *buffer)
 static void
 message_viewer_dig_section_header(message_viewer *mv, u32 section_idx, u16 count)
 {
-    if(mv->view_mode_with & VM_WITH_XFR)
+    (void)count;
+    if(mv->view_mode_with & MESSAGE_VIEWER_WITH_XFR)
     {
         return;
     }
@@ -193,7 +203,7 @@ message_viewer_dig_section_header(message_viewer *mv, u32 section_idx, u16 count
     const char *section_name   = mv->section_name[section_idx];
 
 
-//    if ((view_mode_with & message_print_view_with_mode[section_idx]) && count)
+//    if(message_viewer_requires_section(section_idx, view_mode_with) && count)
     {
         osformat(os, ";; %s:\n", section_name);
     }
@@ -203,7 +213,9 @@ message_viewer_dig_section_header(message_viewer *mv, u32 section_idx, u16 count
 static void
 message_viewer_dig_section_footer(message_viewer *mv, u32 section_idx, u16 count)
 {
-    if(mv->view_mode_with & VM_WITH_XFR)
+    (void)section_idx;
+    (void)count;
+    if(mv->view_mode_with & MESSAGE_VIEWER_WITH_XFR)
     {
         return;
     }
@@ -213,7 +225,7 @@ message_viewer_dig_section_footer(message_viewer *mv, u32 section_idx, u16 count
 
 //    const char *section_name   = mv->section_name[section_idx];
 
-//    if ((view_mode_with & message_print_view_with_mode[section_idx]) && count[section_idx])
+//    if(message_viewer_requires_section(section_idx, view_mode_with))
     {
         osprintln(os, "");
     }
@@ -223,7 +235,7 @@ message_viewer_dig_section_footer(message_viewer *mv, u32 section_idx, u16 count
 static void
 message_viewer_dig_question_record(message_viewer *mv, u8 *record_wire, u16 rclass, u16 rtype)
 {
-    if(mv->view_mode_with & VM_WITH_XFR)
+    if(mv->view_mode_with & MESSAGE_VIEWER_WITH_XFR)
     {
         return;
     }
@@ -271,13 +283,9 @@ message_viewer_dig_question_record(message_viewer *mv, u8 *record_wire, u16 rcla
 
 
 static void
-message_viewer_dig_section_record(message_viewer *mv, u8 *record_wire, u8 view_mode_with)
+message_viewer_dig_section_record(message_viewer *mv, u8 *record_wire, u8 section_idx)
 {
-    if(!(mv->view_mode_with & view_mode_with))
-    {
-        return;
-    }
-
+    (void)section_idx;
 
     /*
      * there is no padding support for formats on complex types (padding is ignored)
@@ -351,11 +359,9 @@ message_viewer_dig_section_record(message_viewer *mv, u8 *record_wire, u8 view_m
     }
     output_stream_write_u8(os, (u8)' ');
 
-
     /* E. write RDATA */
     osprint_rdata(os, rtype, rdata, rdata_size);
 
-
     osprintln(os, "");
     flushout();
 }
