@@ -1,36 +1,37 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
 #ifndef ASYNC_H
 #define	ASYNC_H
 
@@ -48,6 +49,8 @@
 #endif
 #include <dnscore/pace.h>
 
+#define ASYNC_WAIT_TAG 1
+
 #ifdef	__cplusplus
 extern "C" {
 #endif
@@ -61,6 +64,8 @@ extern "C" {
  * This would make the handling synchronous but the processing parallel
  * 
  */
+
+#define FREEBSD12_TEST 0
     
 // used by threads to process a task then push it further in the assembly line
 
@@ -70,10 +75,13 @@ typedef void async_done_callback(struct async_message_s *msg);
 
 struct async_wait_s
 {
-    pthread_mutex_t mutex;
-    pthread_cond_t  cond_wait;
+    mutex_t mutex;
+    cond_t  cond_wait;
     volatile s32 wait_count;
     volatile s32 error_code;
+#if ASYNC_WAIT_TAG
+    u32 tag;
+#endif
 };
 
 typedef struct async_wait_s async_wait_s;
@@ -102,7 +110,7 @@ struct async_message_s
     async_done_callback *handler; // what must be called when the processor has finished working
     void *handler_args;             // complementary arguments for the handler
     
-    volatile u64 start_time;
+    volatile s64 start_time;
 };
 
 typedef struct async_message_s async_message_s;
@@ -119,6 +127,32 @@ typedef struct async_message_s async_message_s;
 
 void async_wait_init(async_wait_s *aw, s32 count);
 
+/**
+ * 
+ * Destroys the synchronisation point
+ * 
+ * @param aw
+ * @return 
+ */
+
+void async_wait_finalize(async_wait_s *aw);
+
+/**
+ * 
+ * Initialises a synchronisation point
+ * count is the number of releases to do before the async_wait call returns
+ * 
+ * @param aw
+ * @param count
+ * @return 
+ */
+
+async_wait_s *async_wait_new_instance(s32 count);
+void async_wait_destroy(async_wait_s *aw);
+
+async_wait_s *async_wait_create_shared(u8 id, s32 count);
+void async_wait_destroy_shared(async_wait_s *aw);
+
 /**
  * Waits until the count has be reduced to 0 (or below if something bad is going on)
  * 
@@ -174,16 +208,6 @@ void async_wait_set_first_error(async_wait_s *aw, s32 error);
 
 s32 async_wait_get_error(async_wait_s *aw);
 
-/**
- * 
- * Destroys the synchronisation point
- * 
- * @param aw
- * @return 
- */
-
-void async_wait_finalize(async_wait_s *aw);
-
 void async_queue_init(async_queue_s *q, u32 size, u64 min_us, u64 max_us, const char* name);
 
 void async_queue_finalize(async_queue_s *q);
