@@ -1,36 +1,36 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
 
 /** @defgroup collections Generic collections functions
  *  @ingroup dnscore
@@ -52,11 +52,6 @@
 #include <dnscore/sys_types.h>
 #include <dnscore/zalloc.h>
 
-/*    ------------------------------------------------------------
- *
- *      STRUCTS
- */
-
 /**
  * Linked list for interface data
  * 
@@ -70,12 +65,13 @@
  * 
  */
 
+#define LISTDL_TAG 0x4c445453494c
+#define LISTDLND_TAG 0x444e4c445453494c
+
 typedef struct list_dl_node_s list_dl_node_s;
 
 // 24 bytes
 
-#define LISTDLND_TAG 0x444e4c445453494c
-
 struct list_dl_node_s
 {
     struct list_dl_node_s *next;
@@ -97,8 +93,6 @@ typedef struct list_dl_s list_dl_s;
 
 // 36 bytes
 
-#define LISTDL_TAG 0x4c445453494c
-
 struct list_dl_s
 {
     list_dl_node_sentiel_s head_sentinel;
@@ -135,8 +129,27 @@ list_dl_iterator_has_next(list_dl_iterator_s *iter)
 static inline void*
 list_dl_iterator_next(list_dl_iterator_s *iter)
 {
-     iter->current_node = iter->current_node->next;
-     return iter->current_node->data;
+    iter->current_node = iter->current_node->next;
+    return iter->current_node->data;
+}
+
+/**
+ * Removes the node from the list, the node is not freed
+ * 
+ * @param list
+ * @param node
+ */
+
+void list_dl_remove_node(list_dl_s *list, list_dl_node_s *node);
+
+static inline void*
+list_dl_iterator_remove(list_dl_iterator_s *iter)
+{
+    list_dl_node_s *current =  iter->current_node;
+    iter->current_node = current->prev;
+    void *data = current->data;
+    list_dl_remove_node(iter->list, current);
+    return data;
 }
 
 /**
@@ -163,14 +176,14 @@ static inline list_dl_node_s *
 list_dl_node_alloc()
 {
     list_dl_node_s *node;
-    ZALLOC_OR_DIE(list_dl_node_s*, node, list_dl_node_s, LISTDLND_TAG);
+    ZALLOC_OBJECT_OR_DIE( node, list_dl_node_s, LISTDLND_TAG);
     return node;
 }
 
 static inline void
 list_dl_node_free(list_dl_node_s *node)
 {
-    ZFREE(node, list_dl_node_s);
+    ZFREE_OBJECT(node);
 }
 
 /**
@@ -187,6 +200,15 @@ list_dl_init(list_dl_s *list)
     list->tail_sentinel.prev = (list_dl_node_s*)&list->head_sentinel;
 }
 
+static inline list_dl_s*
+list_dl_new_instance()
+{
+    list_dl_s *list;
+    ZALLOC_OBJECT_OR_DIE(list, list_dl_s, LISTDL_TAG);
+    list_dl_init(list);
+    return list;
+}
+
 /**
  * Adds an item at the head of the list.
  * 
@@ -198,13 +220,13 @@ static inline void
 list_dl_insert(list_dl_s *list, void *data)
 {
     list_dl_node_s *node;
-    ZALLOC_OR_DIE(list_dl_node_s*, node, list_dl_node_s, LISTDLND_TAG);
+    ZALLOC_OBJECT_OR_DIE( node, list_dl_node_s, LISTDLND_TAG);
     node->next = list->head_sentinel.next;
     node->prev = (list_dl_node_s*)&list->head_sentinel;
     list->head_sentinel.next->prev = node;
     list->head_sentinel.next = node;
     
-#ifdef DEBUG
+#if DEBUG
     assert(list->head_sentinel.next->prev == (list_dl_node_s*)&list->head_sentinel);
 #endif
     
@@ -212,6 +234,8 @@ list_dl_insert(list_dl_s *list, void *data)
     list->size++;
 }
 
+void list_dl_move_to_first_position(list_dl_s *list, void *data);
+
 static inline void
 list_dl_insert_node(list_dl_s *list, list_dl_node_s *node)
 {
@@ -220,7 +244,7 @@ list_dl_insert_node(list_dl_s *list, list_dl_node_s *node)
     list->head_sentinel.next->prev = node;
     list->head_sentinel.next = node;
     
-#ifdef DEBUG
+#if DEBUG
     assert(list->head_sentinel.next->prev == (list_dl_node_s*)&list->head_sentinel);
 #endif
     
@@ -238,13 +262,13 @@ static inline void
 list_dl_append(list_dl_s *list, void *data)
 {
     list_dl_node_s *node;
-    ZALLOC_OR_DIE(list_dl_node_s*, node, list_dl_node_s, LISTDLND_TAG);
+    ZALLOC_OBJECT_OR_DIE( node, list_dl_node_s, LISTDLND_TAG);
     node->next = (list_dl_node_s*)&list->tail_sentinel;
     node->prev = list->tail_sentinel.prev;
     list->tail_sentinel.prev->next = node;
     list->tail_sentinel.prev = node;
     
-#ifdef DEBUG
+#if DEBUG
     assert(list->tail_sentinel.prev->next == (list_dl_node_s*)&list->tail_sentinel);
 #endif
     
@@ -260,7 +284,7 @@ list_dl_append_node(list_dl_s *list, list_dl_node_s *node)
     list->tail_sentinel.prev->next = node;
     list->tail_sentinel.prev = node;
     
-#ifdef DEBUG
+#if DEBUG
     assert(list->tail_sentinel.prev->next == (list_dl_node_s*)&list->tail_sentinel);
 #endif
     list->size++;
@@ -318,24 +342,21 @@ list_dl_remove_first(list_dl_s *list)
 {
     if(list->size > 0)
     {
-#ifdef DEBUG
+#if DEBUG
         assert(list->head_sentinel.next != (list_dl_node_s*)&list->tail_sentinel);
         assert(list->tail_sentinel.prev != (list_dl_node_s*)&list->head_sentinel);
         assert(list->head_sentinel.next != NULL);
         assert(list->tail_sentinel.prev != NULL);
 #endif
-        
         list_dl_node_s *node = list->head_sentinel.next;
         list->head_sentinel.next = node->next;
         node->next->prev = (list_dl_node_s*)&list->head_sentinel;
         list->size--;
         void *data = node->data;
-        ZFREE(node, list_dl_node_s);
-        
-#ifdef DEBUG
+        ZFREE_OBJECT(node);
+#if DEBUG
         assert(list->head_sentinel.next->prev == (list_dl_node_s*)&list->head_sentinel);
 #endif
-        
         return data;
     }
     else
@@ -356,7 +377,7 @@ list_dl_peek_first(list_dl_s *list)
 {
     if(list->size > 0)
     {
-#ifdef DEBUG
+#if DEBUG
         assert(list->head_sentinel.next != (list_dl_node_s*)&list->tail_sentinel);
         assert(list->tail_sentinel.prev != (list_dl_node_s*)&list->head_sentinel);
         assert(list->head_sentinel.next != NULL);
@@ -374,12 +395,32 @@ list_dl_peek_first(list_dl_s *list)
     }
 }
 
+static inline list_dl_node_s*
+list_dl_first_node(list_dl_s *list)
+{
+    if(list->size > 0)
+    {
+#if DEBUG
+        assert(list->head_sentinel.next != (list_dl_node_s*)&list->tail_sentinel);
+        assert(list->tail_sentinel.prev != (list_dl_node_s*)&list->head_sentinel);
+        assert(list->head_sentinel.next != NULL);
+        assert(list->tail_sentinel.prev != NULL);
+#endif
+        
+        return list->head_sentinel.next;
+    }
+    else
+    {
+        return NULL;
+    }
+}
+
 static inline list_dl_node_s*
 list_dl_remove_first_node(list_dl_s *list)
 {
     if(list->size > 0)
     {
-#ifdef DEBUG
+#if DEBUG
         assert(list->head_sentinel.next != (list_dl_node_s*)&list->tail_sentinel);
         assert(list->tail_sentinel.prev != (list_dl_node_s*)&list->head_sentinel);
         assert(list->head_sentinel.next != NULL);
@@ -390,7 +431,7 @@ list_dl_remove_first_node(list_dl_s *list)
         node->next->prev = (list_dl_node_s*)&list->head_sentinel;
         list->size--;
         
-#ifdef DEBUG
+#if DEBUG
         assert(list->head_sentinel.next->prev == (list_dl_node_s*)&list->head_sentinel);
 #endif
         return node;
@@ -415,7 +456,7 @@ list_dl_remove_last(list_dl_s *list)
 {
     if(list->size > 0)
     {
-#ifdef DEBUG
+#if DEBUG
         assert(list->head_sentinel.next != (list_dl_node_s*)&list->tail_sentinel);
         assert(list->tail_sentinel.prev != (list_dl_node_s*)&list->head_sentinel);
         assert(list->head_sentinel.next != NULL);
@@ -427,9 +468,9 @@ list_dl_remove_last(list_dl_s *list)
         node->prev->next = (list_dl_node_s*)&list->tail_sentinel;
         list->size--;
         void *data = node->data;
-        ZFREE(node, list_dl_node_s);
+        ZFREE_OBJECT(node);
         
-#ifdef DEBUG
+#if DEBUG
         assert(list->tail_sentinel.prev->next == (list_dl_node_s*)&list->tail_sentinel);
 #endif
         return data;
@@ -440,6 +481,36 @@ list_dl_remove_last(list_dl_s *list)
     }
 }
 
+static inline void
+list_dl_move_last_to_first(list_dl_s *list)
+{
+    if(list->size > 1)
+    {
+#if DEBUG
+        assert(list->head_sentinel.next != (list_dl_node_s*)&list->tail_sentinel);
+        assert(list->tail_sentinel.prev != (list_dl_node_s*)&list->head_sentinel);
+        assert(list->head_sentinel.next != NULL);
+        assert(list->tail_sentinel.prev != NULL);
+#endif
+        // remove last node
+        
+        list_dl_node_s *node = list->tail_sentinel.prev;
+        list->tail_sentinel.prev = node->prev;
+        node->prev->next = (list_dl_node_s*)&list->tail_sentinel;
+        
+        // insert node
+        
+        node->next = list->head_sentinel.next;
+        node->prev = (list_dl_node_s*)&list->head_sentinel;
+        list->head_sentinel.next->prev = node;
+        list->head_sentinel.next = node;
+        
+#if DEBUG
+        assert(list->tail_sentinel.prev->next == (list_dl_node_s*)&list->tail_sentinel);
+#endif
+    }
+}
+
 /**
  * Returns the data of the last item on the list
  * 
@@ -452,7 +523,7 @@ list_dl_peek_last(list_dl_s *list)
 {
     if(list->size > 0)
     {
-#ifdef DEBUG
+#if DEBUG
         assert(list->head_sentinel.next != (list_dl_node_s*)&list->tail_sentinel);
         assert(list->tail_sentinel.prev != (list_dl_node_s*)&list->head_sentinel);
         assert(list->head_sentinel.next != NULL);
@@ -470,12 +541,32 @@ list_dl_peek_last(list_dl_s *list)
     }
 }
 
+static inline list_dl_node_s*
+list_dl_last_node(list_dl_s *list)
+{
+    if(list->size > 0)
+    {
+#if DEBUG
+        assert(list->head_sentinel.next != (list_dl_node_s*)&list->tail_sentinel);
+        assert(list->tail_sentinel.prev != (list_dl_node_s*)&list->head_sentinel);
+        assert(list->head_sentinel.next != NULL);
+        assert(list->tail_sentinel.prev != NULL);
+#endif
+        
+        return list->tail_sentinel.prev;
+    }
+    else
+    {
+        return NULL;
+    }
+}
+
 static inline list_dl_node_s *
 list_dl_remove_last_node(list_dl_s *list)
 {
     if(list->size > 0)
     {
-#ifdef DEBUG
+#if DEBUG
         assert(list->head_sentinel.next != (list_dl_node_s*)&list->tail_sentinel);
         assert(list->tail_sentinel.prev != (list_dl_node_s*)&list->head_sentinel);
         assert(list->head_sentinel.next != NULL);
@@ -487,7 +578,7 @@ list_dl_remove_last_node(list_dl_s *list)
         node->prev->next = (list_dl_node_s*)&list->tail_sentinel;
         list->size--;
         
-#ifdef DEBUG
+#if DEBUG
         assert(list->tail_sentinel.prev->next == (list_dl_node_s*)&list->tail_sentinel);
 #endif
         return node;
@@ -609,23 +700,7 @@ void *list_dl_get(list_dl_s *list, int index);
 
 ya_result list_dl_foreach(list_dl_s *list, result_callback_function *callback, void *caller_data);
 
-/**
- * Iterates through the items of the function, calling the comparator.
- * 
- * The comparator must return:
- * 
- * COLLECTION_ITEM_SKIP                 : go to next item
- * COLLECTION_ITEM_PROCESS              : delete, then go to next item
- * COLLECTION_ITEM_STOP                 : stop processing
- * COLLECTION_ITEM_PROCESS_THEN_STOP    : delete, then stop processing
- * 
- * @param list
- * @param comparator
- * 
- * @return TRUE if at least one item has been deleted, FALSE otherwise.
- */
-
-bool list_dl_remove_match(list_dl_s *list, result_callback_function *comparator, void *parm);
+bool list_dl_remove_matching_ptr(list_dl_s *list, void *ptr);
 
 /**
  * 
@@ -638,18 +713,8 @@ bool list_dl_remove_match(list_dl_s *list, result_callback_function *comparator,
 static inline u32
 list_dl_size(const list_dl_s *list)
 {
-#ifdef DEBUG
-    if(list->size == 0)
-    {
-        assert(list->head_sentinel.next == (list_dl_node_s*)&list->tail_sentinel);
-        assert(list->tail_sentinel.prev == (list_dl_node_s*)&list->head_sentinel);
-        assert(list->head_sentinel.prev == NULL);
-        assert(list->tail_sentinel.next == NULL);
-    }
-#endif
+
     return list->size;
 }
 
-/*    ------------------------------------------------------------    */
-
 #endif /* LIST_DL_H_ */
