@@ -1,36 +1,36 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
 
 /** @defgroup collections Generic collections functions
  *  @ingroup dnscore
@@ -71,6 +71,7 @@
 
 typedef struct list_sl_node_s list_sl_node_s;
 
+#define LISTSL_TAG   0x4c535453494c
 #define LISTSLND_TAG 0x444e4c535453494c
 
 struct list_sl_node_s
@@ -79,7 +80,7 @@ struct list_sl_node_s
     void *data;
 };
 
-/// The sentiel is a butchered node, meant to avoid the (useless) data field.
+/// The sentinel is a butchered node, meant to avoid the (useless) data field.
 
 typedef struct list_sl_node_sentiel_s list_sl_node_sentiel_s;
 
@@ -115,6 +116,16 @@ list_sl_init(list_sl_s *list)
     list->sentinel.next = NULL;
     list->size = 0;
 }
+
+static inline list_sl_s*
+list_sl_new_instance()
+{
+    list_sl_s *list;
+    ZALLOC_OBJECT_OR_DIE(list, list_sl_s, LISTSL_TAG);
+    list_sl_init(list);
+    return list;
+}
+
 static inline bool
 list_sl_hasnext(const list_sl_s *list, const list_sl_node_s *node)
 {
@@ -132,7 +143,7 @@ static inline void
 list_sl_insert(list_sl_s *list, void *data)
 {
     list_sl_node_s *node;
-    ZALLOC_OR_DIE(list_sl_node_s*, node, list_sl_node_s, LISTSLND_TAG);
+    ZALLOC_OBJECT_OR_DIE( node, list_sl_node_s, LISTSLND_TAG);
     node->next = list->first;
     node->data = data;
     list->first = node;
@@ -178,7 +189,7 @@ list_sl_remove_first(list_sl_s *list)
         void *data = node->data;
         list->first = node->next;
         list->size--;
-        ZFREE(node,list_sl_node_s);
+        ZFREE_OBJECT(node);
         return data;
     }
     else
@@ -237,6 +248,13 @@ bool list_sl_remove(list_sl_s *list, void *data);
 
 void list_sl_clear(list_sl_s *list);
 
+static inline void
+list_sl_delete_instance(list_sl_s *list)
+{
+    list_sl_clear(list);
+    ZFREE_OBJECT(list);
+}
+
 /**
  * Iterates through the items of the function, calling the comparator.
  * 
@@ -285,8 +303,5 @@ list_sl_size(const list_sl_s *list)
 {
     return list->size;
 }
-        
-
-/*    ------------------------------------------------------------    */
 
 #endif /* LIST_SL_H_ */
