@@ -1,36 +1,37 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
 /** @defgroup dnspacket DNS Messages
  *  @ingroup dnscore
  *  @brief 
@@ -45,6 +46,8 @@
 
 #include <dnscore/sys_types.h>
 #include <dnscore/output_stream.h>
+#include <dnscore/message.h>
+#include <dnscore/dns_resource_record.h>
 
 #define PW_MESSAGE_BUFFER_SIZE 0x10500
 
@@ -103,6 +106,47 @@ void packet_writer_create(packet_writer *pw, u8* packet, u32 limit);
 
 ya_result packet_writer_init(packet_writer *pw, u8* packet, u32 packet_offset, u32 size_limit);
 
+static inline ya_result
+packet_writer_init_append_to_message(packet_writer* pw, message_data *mesg)
+{ // do not use the logical max
+    return packet_writer_init(pw, message_get_buffer(mesg), message_get_size(mesg), message_get_buffer_size_max(mesg));
+}
+
+/**
+ *
+ * initialises a writer based on an existing message
+ *
+ * @param pw
+ * @param packet a packet containing a valid message
+ * @param packet_offset
+ * @param size_limit
+ * @return
+ */
+
+
+static inline ya_result
+packet_writer_init_from_message(packet_writer* pw, message_data *mesg)
+{ // do not use the logical max
+    return packet_writer_init(pw, message_get_buffer(mesg), DNS_HEADER_LENGTH, message_get_buffer_size_max(mesg));
+}
+
+/**
+ *
+ * initialises a writer into an existing message, does not parse the message
+ *
+ * @param pw
+ * @param packet a packet containing a valid message
+ * @param packet_offset
+ * @param size_limit
+ * @return
+ */
+
+static inline void
+packet_writer_init_into_message(packet_writer* pw, message_data *mesg)
+{ // do not use the logical max
+    packet_writer_create(pw, message_get_buffer(mesg), message_get_buffer_size_max(mesg));
+}
+
 /**
  * @note uncompressed names will not be compressed, of course *** BUT ***
  *       they will not be used in the compression dictionnary either
@@ -113,22 +157,23 @@ ya_result packet_writer_add_fqdn(packet_writer *pw, const u8* fqdn);
 ya_result packet_writer_add_rdata(packet_writer *pw, u16 rr_type, const u8 *rdata, u16 rdata_len);
 
 ya_result packet_writer_add_record(packet_writer *pw, const u8* fqdn, u16 rr_type, u16 rr_class, u32 ttl, const u8* rdata, u16 rdata_len);
+ya_result packet_writer_add_dnsrr(packet_writer *pw, dns_resource_record* dns_rr);
 
 static inline void packet_writer_forward(packet_writer *pw, u32 bytes)
 {
-    assert(pw->packet_offset + bytes <= PW_MESSAGE_BUFFER_SIZE /*pw->packet_limit*/);
+    assert(pw->packet_offset + bytes <= pw->packet_limit);
     pw->packet_offset += bytes;
 }
 
 static inline void packet_writer_set_u8(packet_writer *pw, u16 value, u32 offset)
 {
-    assert(pw->packet_offset <= PW_MESSAGE_BUFFER_SIZE /*pw->packet_limit*/);
+    assert(pw->packet_offset <= pw->packet_limit);
     pw->packet[offset] = value;
 }
 
 static inline void packet_writer_add_u8(packet_writer *pw, u16 value)
 {
-    assert(pw->packet_offset + 1 <= PW_MESSAGE_BUFFER_SIZE/*pw->packet_limit*/);
+    assert(pw->packet_offset + 1 <= pw->packet_limit);
     pw->packet[pw->packet_offset++] = value;
 }
 
@@ -140,27 +185,27 @@ static inline void packet_writer_set_u16(packet_writer *pw, u16 value, u32 offse
 
 static inline void packet_writer_add_u16(packet_writer *pw, u16 value)
 {
-    assert(pw->packet_offset + 2 <= PW_MESSAGE_BUFFER_SIZE/*pw->packet_limit*/);
+    assert(pw->packet_offset + 2 <= pw->packet_limit);
     SET_U16_AT(pw->packet[pw->packet_offset], value);
     pw->packet_offset += 2;
 }
 
 static inline void packet_writer_add_u32(packet_writer *pw, u32 value)
 {
-    assert(pw->packet_offset + 4 <= PW_MESSAGE_BUFFER_SIZE /*pw->packet_limit*/);
+    assert(pw->packet_offset + 4 <= pw->packet_limit);
     SET_U32_AT(pw->packet[pw->packet_offset], value);
     pw->packet_offset += 4;
 }
 
 static inline void packet_writer_set_u32(packet_writer *pw, u32 value, u32 offset)
 {
-    assert(offset + 4 <= PW_MESSAGE_BUFFER_SIZE /*pw->packet_limit*/);
+    assert(offset + 4 <= pw->packet_limit);
     SET_U32_AT(pw->packet[offset], value);
 }
 
 static inline void packet_writer_add_bytes(packet_writer *pw, const u8 *buffer, u32 len)
 {
-    assert(pw->packet_offset + len <= PW_MESSAGE_BUFFER_SIZE /*pw->packet_limit*/);
+    assert(pw->packet_offset + len <= pw->packet_limit);
     MEMCOPY(&pw->packet[pw->packet_offset], buffer, len);
     pw->packet_offset += len;
 }
@@ -172,7 +217,7 @@ static inline u32 packet_writer_get_offset(const packet_writer *pw)
 
 static inline void packet_writer_set_offset(packet_writer *pw, u32 offset)
 {
-    yassert(offset <= PW_MESSAGE_BUFFER_SIZE /*pw->packet_limit*/);
+    yassert(offset <= pw->packet_limit);
     pw->packet_offset = offset;
 }
 
@@ -201,6 +246,3 @@ ya_result write_tcp_packet(packet_writer *pw, output_stream *tcpos);
 
 #endif	/* _PACKET_WRITER_H */
 /** @} */
-
-/*----------------------------------------------------------------------------*/
-
