@@ -1,36 +1,37 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
 /** @defgroup zalloc very fast, no-overhead specialised memory allocation functions
  *  @ingroup dnscore
  *  @brief no-overhead specialised allocation functions
@@ -56,8 +57,8 @@
 
 #pragma once
 
-#include <pthread.h>
 #include <dnscore/dnscore-config-features.h>
+#include <dnscore/thread.h>
 #include <dnscore/config_settings.h>
 #include <dnscore/sys_types.h>
 #include <dnscore/debug.h>
@@ -91,16 +92,37 @@ void *malloc_string_or_die(size_t len, u64 tag);
 void mfree_string(void *ptr);
 
 /* 8 bytes aligned */
-#define ZALLOC_OR_DIE(cast,label,object,tag) MALLOC_OR_DIE(cast,label,sizeof(object),tag);assert((label) != NULL)
+
+static inline size_t zalloc_memory_block_size(size_t size)
+{
+    return size;
+}
+
+static inline void free_erases(void *ptr, size_t size)
+{
+    if(ptr != NULL)
+    {
+        memset(ptr, 0xfe, size);
+    }
+    free(ptr);
+}
+
+#if DEBUG
+#define ZFREE(label,object) free_erases(label, sizeof(object))
+#else
 #define ZFREE(label,object) free(label)
+#endif
 
-#define ZALLOC_ARRAY_OR_DIE(cast,label,size,tag) MALLOC_OR_DIE(cast,label,size,tag);assert((label) != NULL)
-#define ZFREE_ARRAY(ptr,size_) (void)(size_);free(ptr)
+#define ZALLOC_ARRAY_OR_DIE(cast_,label_,size_,tag_) MALLOC_OR_DIE(cast_,label_,size_,tag_);assert((label_) != NULL)
+#if DEBUG
+#define ZFREE_ARRAY(ptr_,size_) free_erases((ptr_),(size_))
+#else
+#define ZFREE_ARRAY(ptr_,size_) free(ptr_)
+#endif
 
-// 2.4 API, put here to ease back-port
+// preferred way of allocating one instance of a type (struct, ...)
 #define ZALLOC_OBJECT_OR_DIE(label__,object__,tag__) MALLOC_OBJECT_OR_DIE(label__, object__, tag__);assert((label__) != NULL)
 #define ZALLOC_OBJECT_ARRAY_OR_DIE(label__,object__, count__,tag__) MALLOC_OBJECT_ARRAY_OR_DIE(label__, object__, count__, tag__);assert((label__) != NULL)
-#define ZFREE_OBJECT(label__) free((label__))
 
 #define ZALLOC_ARRAY_RESIZE(type_,array_,count_,newcount_)		    \
     {									    \
@@ -109,6 +131,9 @@ void mfree_string(void *ptr);
 	(count_) = zalloc_new_count;					    \
     }
 
+#define ZFREE_OBJECT(label__) free((label__))
+#define ZFREE_OBJECT_OF_TYPE(label__,type__) free(label__)
+
 #else
 
 /**
@@ -134,7 +159,7 @@ int zalloc_init();
 
 // actually does nothing, just there for symmetry
 
-void zalloc_finalise();
+void zalloc_finalize();
     
 /**
  * @brief Allocates one slot in a memory set
@@ -176,7 +201,7 @@ u64 zheap_line_avail(u32 page);
  * Now this does nothing
  */
 
-static inline void zalloc_set_owner_thread(pthread_t owner) {(void)owner;}
+static inline void zalloc_set_owner_thread(thread_t owner) {(void)owner;}
 
 static inline void* zalloc(s32 size)
 {
@@ -245,16 +270,24 @@ void zfree_unaligned(void* ptr);
  * THIS SHOULD BE OPTIMIZED BY THE COMPILER AS ONE AND ONLY ONE CALL
  */
 
-#define ZALLOC(object) ((((sizeof(object) + 7) >> 3)-1) < ZALLOC_PG_SIZE_COUNT)?zalloc_line(((sizeof(object) + 7) >> 3)-1):malloc(sizeof(object))
-#define ZFREE(ptr,object) ((((sizeof(object) + 7) >> 3)-1) < ZALLOC_PG_SIZE_COUNT)?zfree_line(ptr,(((sizeof(object) + 7) >> 3)-1)):free(ptr)
-//
-#define ZALLOC_OR_DIE(cast,label,object,tag) if((label=(cast)ZALLOC(object))==NULL) {DIE(ZALLOC_ERROR_OUTOFMEMORY); } assert((label) != NULL)
+static inline size_t zalloc_memory_block_size(size_t size)
+{
+    return (size + 7) & ~7;
+}
+
+void* zalloc_line(u32 page_index);
+
+#define ZALLOC_OBJECT(object__) ((((sizeof(object__) + 7) >> 3)-1) < ZALLOC_PG_SIZE_COUNT)?zalloc_line(((sizeof(object__) + 7) >> 3)-1):malloc(sizeof(object__))
+#define ZFREE(ptr,object__) ((((sizeof(object__) + 7) >> 3)-1) < ZALLOC_PG_SIZE_COUNT)?zfree_line(ptr,(((sizeof(object__) + 7) >> 3)-1)):free(ptr)
+
+#define ZALLOC_BYTES(size__) ((((((size__) + 7) >> 3)-1) < ZALLOC_PG_SIZE_COUNT)?zalloc_line((((size__) + 7) >> 3)-1):malloc((size__)))
 #define ZALLOC_ARRAY_OR_DIE(cast,label,size_,tag) if((label = (cast)zalloc(size_)) == NULL) {DIE(ZALLOC_ERROR_OUTOFMEMORY); } assert((label) != NULL)
 #define ZFREE_ARRAY(ptr,size_) zfree(ptr,size_)
-// 2.4 API, put here to ease back-port
-#define ZALLOC_OBJECT_OR_DIE(label__,object__,tag__) if((label__=(object__*)ZALLOC(object__))==NULL) {DIE(ZALLOC_ERROR_OUTOFMEMORY); } assert((label__) != NULL)
-#define ZALLOC_OBJECT_ARRAY_OR_DIE(label__,object__,count__,tag__) if((label__=(object__*)ZALLOC(object__)*(count__))==NULL) {DIE(ZALLOC_ERROR_OUTOFMEMORY); } assert((label__) != NULL)
+// preferred way of allocating one instance of a type (struct, ...)
+#define ZALLOC_OBJECT_OR_DIE(label__,object__,tag__) if((label__=(object__*)ZALLOC_OBJECT(object__))==NULL) {DIE(ZALLOC_ERROR_OUTOFMEMORY); } assert((label__) != NULL)
+#define ZALLOC_OBJECT_ARRAY_OR_DIE(label__,object__,count__,tag__) if((label__=(object__*)ZALLOC_BYTES(sizeof(object__)*(count__)))==NULL) {DIE(ZALLOC_ERROR_OUTOFMEMORY); } assert((label__) != NULL)
 #define ZFREE_OBJECT(label__) zfree((label__), sizeof(*(label__)))
+#define ZFREE_OBJECT_OF_TYPE(label__,type__) zfree((label__), sizeof(type__))
 /**
  * (Z)Allocates a new array of count type elements so it can hold
  * newcount type elements. (It takes granularity into account to avoid
