@@ -1,36 +1,37 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
 /** @defgroup streaming Streams
  *  @ingroup dnscore
  *  @brief
@@ -125,6 +126,8 @@ bytearray_output_stream_write(output_stream* stream, const u8* buffer, u32 len)
 static ya_result
 bytearray_output_stream_flush(output_stream* stream)
 {
+    (void)stream;
+
     return SUCCESS;
 }
 
@@ -190,7 +193,7 @@ bytearray_output_stream_init_ex(output_stream* out_stream, u8* array, u32 size,
 {
     bytearray_output_stream_data* data;
 
-    ZALLOC_OR_DIE(bytearray_output_stream_data*, data, bytearray_output_stream_data, BYTE_ARRAY_OUTPUT_STREAM_DATA_TAG);
+    ZALLOC_OBJECT_OR_DIE( data, bytearray_output_stream_data, BYTE_ARRAY_OUTPUT_STREAM_DATA_TAG);
 
     flags |= BYTEARRAY_ZALLOC_CONTEXT;
     
@@ -210,6 +213,43 @@ bytearray_output_stream_reset(output_stream* stream)
     data->buffer_offset = 0;
 }
 
+ya_result
+bytearray_output_stream_ensure(output_stream* stream, u32 size)
+{
+    bytearray_output_stream_data* data = (bytearray_output_stream_data*)stream->data;
+    
+    if(data->buffer_size < size)
+    {
+        /* Either we can resize, either we have to trunk */
+        
+        size = (size + 7) & ~7;
+
+        if((data->flags & BYTEARRAY_DYNAMIC) != 0)
+        {
+            u8* newbuffer;
+
+            MALLOC_OR_DIE(u8*, newbuffer, size, BYTE_ARRAY_OUTPUT_STREAM_BUFF_TAG);
+            MEMCOPY(newbuffer, data->buffer, data->buffer_offset);
+
+            if((data->flags & BYTEARRAY_OWNED) != 0)
+            {
+                free(data->buffer);
+            }
+
+            data->buffer = newbuffer;
+            data->buffer_size = size;
+
+            data->flags |= BYTEARRAY_OWNED;
+        }
+        else
+        {
+            return ERROR; // not dynamic
+        }
+    }
+    
+    return SUCCESS;
+}
+
 u32
 bytearray_output_stream_size(output_stream* stream)
 {
@@ -279,7 +319,7 @@ bytearray_output_stream_zdup(output_stream* out_stream)
     bytearray_output_stream_data* data = (bytearray_output_stream_data*)out_stream->data;
     u8 *ret;
     u32 n = MAX(data->buffer_offset, 1); // because allocating 0 bytes can be an hassle
-    ZALLOC_ARRAY_OR_DIE(u8*, ret, n, BAOSZDUP_TAG);
+    ZALLOC_OBJECT_ARRAY_OR_DIE(ret, u8, n, BAOSZDUP_TAG);
     memcpy(ret, data->buffer, n);
     
     return ret;
@@ -298,6 +338,3 @@ bytearray_output_stream_dup(output_stream* out_stream)
 }
 
 /** @} */
-
-/*----------------------------------------------------------------------------*/
-
