@@ -1,36 +1,37 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
 /** @defgroup dnscoretools Generic Tools
  *  @ingroup dnscore
  *  @brief
@@ -39,6 +40,7 @@
  */
 
 #include "dnscore/dnscore-config.h"
+#include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -46,7 +48,11 @@
 #include <dirent.h>
 #include <sys/socket.h>
 
+#define FDTOOLS_C_ 1
+
 #include "dnscore/fdtools.h"
+#include "dnscore/zalloc.h"
+#include "dnscore/ptr_set.h"
 #include "dnscore/timems.h"
 #include "dnscore/logger.h"
 #include "dnscore/mutex.h"
@@ -58,12 +64,33 @@ extern logger_handle *g_system_logger;
 
 #define DEBUG_FD_OPEN_CLOSE_MONITOR 0
 
-#ifdef DEBUG
+#define FDTRACK_TAG 0x4b434152544446
+
+#if DEBUG
 // avoids logging these operations in the logger.
 // this prevents a self-deadlock if the logger limit is reached (2^20 lines)
 bool logger_is_self();
 #endif
 
+#if DEBUG
+static void
+admin_warn(const char *pathname)
+{
+    uid_t uid = getuid();
+    uid_t euid = getuid();
+    gid_t gid = getgid();
+    gid_t egid = getegid();
+    
+    bool is_admin = (uid==0)||(euid==0)||(gid==0)||(egid==0);
+    
+    if(is_admin)
+    {
+        printf("It is unlikely file or directory creation should be made as an admin '%s' (DEBUG)\n", pathname);
+        fflush(NULL);
+    }
+}
+#endif
+
 #if DEBUG_FD_OPEN_CLOSE_MONITOR
 
 // file descriptor track enabled
@@ -92,11 +119,11 @@ static void fd_set_name(int fd, const char *name)
     
     log_debug6("file descriptor: %i is '%s'", fd, name);
     
-    u32_node *node = u32_set_avl_insert(&fd_to_name, fd);
+    u32_node *node = u32_set_insert(&fd_to_name, fd);
                 
     if(node->value == NULL)
     {    
-        ZALLOC_OR_DIE(fd_track*, track, fd_track, GENERIC_TAG);
+        ZALLOC_OBJECT_OR_DIE(track, fd_track, FDTRACK_TAG);
         node->value = track;
         track->opener = 0;
         track->closer = 0;
@@ -111,7 +138,7 @@ static void fd_set_name(int fd, const char *name)
         }
     }
     
-    strncpy(track->name, name, sizeof(track->name));
+    strcpy_ex(track->name, name, sizeof(track->name));
     track->opener = debug_stacktrace_get();
     track->name[sizeof(track->name) - 1] = '\0';
     
@@ -122,7 +149,7 @@ static void fd_clear_name(int fd)
 {
     group_mutex_lock(&fd_mtx, GROUP_MUTEX_WRITE);
     
-    u32_node *node = u32_set_avl_find(&fd_to_name, fd);
+    u32_node *node = u32_set_find(&fd_to_name, fd);
     if(node != NULL)
     {
         yassert(node->value != NULL);
@@ -181,8 +208,9 @@ writefully(int fd, const void *buf, size_t count)
             {
                 break;
             }
-
+            
             int err = errno;
+            
             if(err == EINTR)
             {
                 continue;
@@ -190,7 +218,12 @@ writefully(int fd, const void *buf, size_t count)
 
             if(err == EAGAIN) /** @note It is nonsense to call writefully with a non-blocking fd */
             {
-                break;
+                if(current - start > 0)
+                {
+                    break;
+                }
+
+                return MAKE_ERRNO_ERROR(ETIMEDOUT);
             }
             
             if(err == ENOSPC)
@@ -205,7 +238,7 @@ writefully(int fd, const void *buf, size_t count)
                 break;
             }
 
-            return -1;
+            return MAKE_ERRNO_ERROR(err);
         }
 
         current += n;
@@ -222,17 +255,17 @@ writefully(int fd, const void *buf, size_t count)
  */
 
 ssize_t
-readfully(int fd, void *buf, size_t count)
+readfully(int fd, void *buf, size_t length)
 {
     u8* start = (u8*)buf;
     u8* current = start;
     ssize_t n;
 
-    while(count > 0)
+    while(length > 0)
     {
-        if((n = read(fd, current, count)) <= 0)
+        if((n = read(fd, current, length)) <= 0)
         {
-            if(n == 0)
+            if(n == 0) // end of file
             {
                 break;
             }
@@ -248,7 +281,7 @@ readfully(int fd, void *buf, size_t count)
             {
                 break;
             }
-            
+
             if(current - start > 0)
             {
                 break;
@@ -258,7 +291,7 @@ readfully(int fd, void *buf, size_t count)
         }
 
         current += n;
-        count -= n;
+        length -= n;
     }
 
     return current - start;
@@ -271,7 +304,7 @@ readfully(int fd, void *buf, size_t count)
  */
 
 ssize_t
-writefully_limited(int fd, const void *buf, size_t count, double minimum_rate)
+writefully_limited(int fd, const void *buf, size_t count, double minimum_rate_us)
 {
     const u8* start = (const u8*)buf;
     const u8* current = start;
@@ -310,12 +343,25 @@ writefully_limited(int fd, const void *buf, size_t count, double minimum_rate)
                 
                 u64 now = timeus();
 
-                double t = (now - tstart);
-                double b = (current - (u8*)buf)  * 1000000.;
+                u64 time_elapsed_u64 = now - tstart;
 
-                if(b < minimum_rate * t)  /* b/t < minimum_rate */
-                {                           
-                    return TCP_RATE_TOO_SLOW;
+                if(time_elapsed_u64 >= ONE_SECOND_US)
+                {
+                    double time_elapsed_us = time_elapsed_u64;
+
+                    double bytes_written = (current - (u8*)buf)  * ONE_SECOND_US_F;
+
+                    double expected_bytes_written = minimum_rate_us * time_elapsed_us;
+
+                    if(bytes_written < expected_bytes_written)  /* bytes/time < minimum_rate */
+                    {
+#if DEBUG
+                        log_warn("writefully_limited: rate of %fBps < %fBps (%fµs)", bytes_written, expected_bytes_written, time_elapsed_us);
+#else
+                        log_debug("writefully_limited: rate of %fBps < %fBps (%fµs)", bytes_written, expected_bytes_written, time_elapsed_us);
+#endif
+                        return TCP_RATE_TOO_SLOW;
+                    }
                 }
 
                 continue;
@@ -350,7 +396,7 @@ writefully_limited(int fd, const void *buf, size_t count, double minimum_rate)
  */
 
 ssize_t
-readfully_limited(int fd, void *buf, size_t count, double minimum_rate)
+readfully_limited(int fd, void *buf, size_t count, double minimum_rate_us)
 {
     u8* start = (u8*)buf;
     u8* current = start;
@@ -388,17 +434,120 @@ readfully_limited(int fd, void *buf, size_t count, double minimum_rate)
                 u64 now = timeus();
 
                 /* t is in us */
-                
-                double t = (now - tstart);
-                double b = (current - (u8*)buf) * 1000000.;
 
-                double expected_rate = minimum_rate * t;
-                
-                if(b < expected_rate)  /* b/t < minimum_rate */
+                u64 time_elapsed_u64 = now - tstart;
+
+                double time_elapsed_us = time_elapsed_u64;
+
+                if(time_elapsed_u64 >= ONE_SECOND_US)
                 {
-                    log_debug("readfully_limited: rate of %f < %f (%fµs)", b, expected_rate, t);
+                    double bytes_read = (current - (u8*)buf) * ONE_SECOND_US_F;
 
-                    return TCP_RATE_TOO_SLOW;
+                    double expected_bytes_read = minimum_rate_us * time_elapsed_us;
+
+                    if(bytes_read < expected_bytes_read)  // bytes/time < minimum_rate
+                    {
+                        time_elapsed_us /= 1000000.0;
+#if DEBUG
+                        log_warn("readfully_limited: rate of %fBps < %fBps (%fµs) (DEBUG)", bytes_read, expected_bytes_read, time_elapsed_us);
+#else
+                        log_debug("readfully_limited: rate of %fBps < %fBps (%fµs)", bytes_read, expected_bytes_read, time_elapsed_us);
+#endif
+                        return TCP_RATE_TOO_SLOW;
+                    }
+                }
+
+                continue;
+            }
+
+            if(current - start > 0)
+            {
+                break;
+            }
+
+            return -1;  /* EOF */
+        }
+
+        current += n;
+        count -= n;
+    }
+
+    return current - start;
+}
+
+/**
+ * Reads fully the buffer from the fd
+ * It will only return a short count for system errors.
+ * ie: fs full, non-block would block, fd invalid/closed, ...
+ */
+
+ssize_t
+readfully_limited_ex(int fd, void *buf, size_t count, s64 timeout_us, double minimum_rate_us)
+{
+    u8* start = (u8*)buf;
+    u8* current = start;
+    ssize_t n;
+
+    if(timeout_us <= 0)
+    {
+        timeout_us = 1;
+    }
+
+    s64 tstart = timeus();
+
+    // ASSUME : timeout set on fd for read & write
+
+    while(count > 0)
+    {
+        if((n = read(fd, current, count)) <= 0)
+        {
+            if(n == 0)
+            {
+                break;
+            }
+
+            int err = errno;
+
+            if(err == EINTR)
+            {
+                continue;
+            }
+
+            if(err == EAGAIN) /** @note It is nonsense to call readfully with a non-blocking fd */
+            {
+                /*
+                 * Measure the current elapsed time
+                 * Measure the current bytes
+                 * compare with the minimum rate
+                 * act on it
+                 */
+
+                s64 now = timeus();
+
+                /* t is in us */
+
+                s64 time_elapsed_u64 = now - tstart;
+
+                double time_elapsed_us = time_elapsed_u64;
+
+                if(time_elapsed_u64 >= timeout_us)
+                {
+                    double bytes_read = (current - (u8*)buf);
+                    bytes_read *= ONE_SECOND_US_F;
+                    bytes_read /= time_elapsed_u64;
+
+                    double expected_bytes_read = minimum_rate_us * time_elapsed_us;
+
+                    if(bytes_read < expected_bytes_read)  // bytes/time < minimum_rate
+                    {
+                        time_elapsed_us /= 1000000.0;
+#if DEBUG
+                        log_warn("readfully_limited: rate of %fBps < %fBps (%fµs) (DEBUG)", bytes_read, expected_bytes_read, time_elapsed_us);
+#else
+                        log_debug("readfully_limited: rate of %fBps < %fBps (%fµs)", bytes_read, expected_bytes_read, time_elapsed_us);
+#endif
+                        return TCP_RATE_TOO_SLOW;
+                    }
                 }
 
                 continue;
@@ -491,8 +640,8 @@ unlink_ex(const char *folder, const char *filename)
 {
     char fullpath[PATH_MAX];
     
-    int l0 = strlen(folder);
-    int l1 = strlen(filename);
+    size_t l0 = strlen(folder);
+    size_t l1 = strlen(filename);
     if(l0 + l1 < sizeof(fullpath))
     {    
         memcpy(fullpath, folder, l0);
@@ -504,10 +653,64 @@ unlink_ex(const char *folder, const char *filename)
     }
     else
     {
+        errno = ENOMEM;
         return -1;
     }
 }
 
+/**
+ * Copies the absolute path of a file into a buffer.
+ * 
+ * @param filename the file name
+ * @param buffer the output buffer
+ * @param buffer_size the size of the output buffer
+ * @return the string length (without the terminator)
+ */
+
+ya_result
+file_get_absolute_path(const char *filename, char *buffer, size_t buffer_size)
+{
+    ya_result ret;
+    
+    if(filename[0] == '/')
+    {
+        strcpy_ex(buffer, filename, buffer_size);
+        ret = strlen(buffer);
+        return ret;
+    }
+    else
+    {
+        if(getcwd(buffer, buffer_size) == NULL)
+        {
+            return ERRNO_ERROR;
+        }
+        
+        size_t n = strlen(buffer);
+        
+        if(n < buffer_size)
+        {
+            ret = n + 1;
+
+            buffer += n;
+            buffer_size -= n;
+            
+            *buffer++ = '/';
+            --buffer_size;
+            
+            n = strlen(filename);
+            if(n < buffer_size)
+            {
+                memcpy(buffer, filename, n);
+                buffer[n] = '\0';
+                
+                return ret + n;
+            }
+        }
+
+        return ERROR; // not enough room
+    }
+}
+
 #if DEBUG_BENCH_FD
 static debug_bench_s debug_open;
 static debug_bench_s debug_open_create;
@@ -541,7 +744,7 @@ open_ex(const char *pathname, int flags)
 {
     int fd;
     
-#ifdef DEBUG
+#if DEBUG
     if(!logger_is_self() && logger_is_running())
     {
         log_debug6("open_ex(%s,%o)", STRNULL(pathname), flags);
@@ -555,6 +758,11 @@ open_ex(const char *pathname, int flags)
     fdtools_debug_bench_register();
     u64 bench = debug_bench_start(&debug_open);
 #endif
+
+#if DNSCORE_FDTOOLS_CLOEXEC
+    bool cloexec = (flags & O_CLOEXEC) != 0;
+    flags &= ~O_CLOEXEC;
+#endif
     
     while((fd = open(pathname, flags)) < 0)
     {
@@ -565,12 +773,24 @@ open_ex(const char *pathname, int flags)
             break;
         }
     }
+
+#if DNSCORE_FDTOOLS_CLOEXEC
+    if((fd >= 0) && cloexec)
+    {
+        ya_result ret;
+
+        if(FAIL(ret = fd_setcloseonexec(fd)))
+        {
+            log_warn("open_ex(%s,%o): failed to set CLOEXEC: %r", STRNULL(pathname), flags|O_CLOEXEC, ret);
+        }
+    }
+#endif
     
 #if DEBUG_BENCH_FD
     debug_bench_stop(&debug_open, bench);
 #endif
     
-#ifdef DEBUG
+#if DEBUG
     if(!logger_is_self() && logger_is_running())
     {
         log_debug6("open_ex(%s,%o): %r (%i)", STRNULL(pathname), flags, ERRNO_ERROR, fd);
@@ -601,7 +821,7 @@ open_create_ex(const char *pathname, int flags, mode_t mode)
 {
     int fd;
     
-#ifdef DEBUG
+#if DEBUG
     if(!logger_is_self() && logger_is_running())
     {
         log_debug6("open_create_ex(%s,%o,%o)", STRNULL(pathname), flags, mode);
@@ -609,6 +829,10 @@ open_create_ex(const char *pathname, int flags, mode_t mode)
     }
 #endif
     
+#if DEBUG
+    admin_warn(pathname);
+#endif
+    
     yassert(pathname != NULL);
     
 #if DEBUG_BENCH_FD
@@ -632,7 +856,7 @@ open_create_ex(const char *pathname, int flags, mode_t mode)
     debug_bench_stop(&debug_open_create, bench);
 #endif
     
-#ifdef DEBUG
+#if DEBUG
     if(!logger_is_self() && logger_is_running())
     {
         log_debug6("open_create_ex(%s,%o,%o): %r (%i)", STRNULL(pathname), flags, mode, ERRNO_ERROR, fd);
@@ -648,21 +872,25 @@ open_create_ex(const char *pathname, int flags, mode_t mode)
     return fd;
 }
 
-int mkstemp_ex(char *template)
+int mkstemp_ex(char * tmp_name_template)
 {
-    int fd = mkstemp(template);
-#ifdef DEBUG
+#ifndef WIN32
+    int fd = mkstemp(tmp_name_template);
+#if DEBUG
     if(!logger_is_self() && logger_is_running())
     {
 #if DEBUG_FD_OPEN_CLOSE_MONITOR
         if(fd > 0)
         {
-            fd_set_name(fd, template);
+            fd_set_name(fd, tmp_name_template);
         }
 #endif
     }
 #endif
     return fd;
+#else
+    return -1;
+#endif
 }
 
 /**
@@ -678,6 +906,9 @@ ya_result
 open_create_ex_nolog(const char *pathname, int flags, mode_t mode)
 {
     int fd;
+#if DEBUG
+    admin_warn(pathname);
+#endif
     while((fd = open(pathname, flags, mode)) < 0)
     {
         int err = errno;
@@ -703,7 +934,7 @@ open_create_ex_nolog(const char *pathname, int flags, mode_t mode)
  */
 
 ya_result
-#ifndef DEBUG
+#if !DNSCORE_HAS_CLOSE_EX_REF
 close_ex(int fd)
 #else
 close_ex_ref(int* fdp)
@@ -711,7 +942,7 @@ close_ex_ref(int* fdp)
 {
     ya_result return_value = SUCCESS;
 
-#ifdef DEBUG
+#if DNSCORE_HAS_CLOSE_EX_REF
     int fd = *fdp;
     
     if(fd == -2)
@@ -722,7 +953,7 @@ close_ex_ref(int* fdp)
     *fdp = -2;
 #endif
     
-#ifdef DEBUG
+#if DEBUG
     if(!logger_is_self() && logger_is_running())
     {
 #if DEBUG_FD_OPEN_CLOSE_MONITOR
@@ -756,7 +987,7 @@ close_ex_ref(int* fdp)
     debug_bench_stop(&debug_close, bench);
 #endif
     
-#ifdef DEBUG
+#if DEBUG
     if(!logger_is_self() && logger_is_running())
     {
         log_debug6("close_ex(%i): %r", fd, return_value);
@@ -780,7 +1011,7 @@ close_ex_ref(int* fdp)
  */
 
 ya_result
-#ifndef DEBUG
+#if !DNSCORE_HAS_CLOSE_EX_REF
 close_ex_nolog(int fd)
 #else
 close_ex_nolog_ref(int* fdp)
@@ -788,7 +1019,7 @@ close_ex_nolog_ref(int* fdp)
 {
     ya_result return_value = SUCCESS;
     
-#ifdef DEBUG
+#if DNSCORE_HAS_CLOSE_EX_REF
     int fd = *fdp;
     
     if(fd == -2)
@@ -816,6 +1047,7 @@ close_ex_nolog_ref(int* fdp)
 int
 fsync_ex(int fd)
 {
+#ifndef WIN32
     while(fsync(fd) < 0)
     {
         int err = errno;
@@ -824,14 +1056,17 @@ fsync_ex(int fd)
             return ERRNO_ERROR;
         }
     }
-    
+#else
+    FlushFileBuffers(fd);
+#endif
     return SUCCESS;
 }
 
 int
 fdatasync_ex(int fd)
 {
-#if IS_LINUX_FAMILY
+#ifndef WIN32
+#if defined(__linux__)
     while(fdatasync(fd) < 0)
 #else
     while(fsync(fd) < 0)
@@ -843,7 +1078,9 @@ fdatasync_ex(int fd)
             return ERRNO_ERROR;
         }
     }
-    
+#else
+    FlushFileBuffers(fd);
+#endif
     return SUCCESS;
 }
 
@@ -877,6 +1114,36 @@ int dup2_ex(int old_fd, int new_fd)
     return ret;
 }
 
+int truncate_ex(const char *path, off_t len)
+{
+    int ret;
+    while((ret = truncate(path, len)) < 0)
+    {
+        int err = errno;
+        if(err != EINTR)
+        {
+            return ERRNO_ERROR;
+        }
+    }
+    
+    return ret;
+}
+
+int ftruncate_ex(int fd, off_t len)
+{
+    int ret;
+    while((ret = ftruncate(fd, len)) < 0)
+    {
+        int err = errno;
+        if(err != EINTR)
+        {
+            return ERRNO_ERROR;
+        }
+    }
+    
+    return ret;
+}
+
 /**
  * Returns the type of socket.
  * 
@@ -917,8 +1184,7 @@ filesize(const char *name)
 
 /**
  * Checks for existence of a file/dir/link
- * 
- * 
+ *
  * @param name the file name
  * 
  * @return 1 if the file exists, 0 if the file does not exists, an error otherwise
@@ -928,10 +1194,17 @@ ya_result
 file_exists(const char *name)
 {
     struct stat s;
+#ifndef WIN32
     if(lstat(name, &s) >= 0)    // MUST be lstat
     {
         return 1;
     }
+#else
+    if(stat(name, &s) >= 0)    // MUST be lstat
+    {
+        return 1;
+    }
+#endif
     
     int err = errno;
     
@@ -943,16 +1216,58 @@ file_exists(const char *name)
     return MAKE_ERRNO_ERROR(err);
 }
 
+/**
+ *
+ * Checks if a file exists and is a link
+ *
+ * @param name the file name
+ *
+ * @return  0 : not a link
+ *          1 : a link
+ *        < 0 : error
+ */
+
 ya_result
 file_is_link(const char *name)
 {
+#ifndef WIN32
     struct stat s;
     if(lstat(name, &s) >= 0)    // MUST be lstat
     {
-        return S_ISLNK(s.st_mode);
+        return S_ISLNK(s.st_mode)?SUCCESS:ERROR;
     }
     
     return ERRNO_ERROR;
+#else
+    return 0;
+#endif
+}
+
+/**
+ *
+ * Checks if a file exists and is a directory
+ *
+ * @param name the file name
+ *
+ * @return  0 : not a link
+ *          1 : a link
+ *        < 0 : error
+ */
+
+ya_result
+file_is_directory(const char *name)
+{
+#ifndef WIN32
+    struct stat s;
+    if(lstat(name, &s) >= 0)    // MUST be lstat
+    {
+        return S_ISDIR(s.st_mode)?SUCCESS:ERROR;
+    }
+
+    return ERRNO_ERROR;
+#else
+    return 0;
+#endif
 }
 
 /**
@@ -975,9 +1290,13 @@ file_is_link(const char *name)
 int
 mkdir_ex(const char *pathname, mode_t mode, u32 flags)
 {
-#ifdef DEBUG
+#if DEBUG
     log_debug("mkdir_ex(%s,%o)", pathname, mode);
 #endif
+    
+#if DEBUG
+    admin_warn(pathname);
+#endif
                 
     const char *s;
     char *t;
@@ -1021,7 +1340,7 @@ mkdir_ex(const char *pathname, mode_t mode, u32 flags)
             
             if(err != ENOENT)
             {
-#ifdef DEBUG
+#if DEBUG
                 log_debug("mkdir_ex(%s,%o): stat returned %r", pathname, mode, MAKE_ERRNO_ERROR(err));
 #endif
                 
@@ -1030,7 +1349,7 @@ mkdir_ex(const char *pathname, mode_t mode, u32 flags)
             
             if(mkdir(dir_path, mode) < 0)
             {
-#ifdef DEBUG
+#if DEBUG
                 log_debug("mkdir_ex(%s,%o): mkdir(%s, %o) returned %r", pathname, mode, dir_path, mode, MAKE_ERRNO_ERROR(err));
 #endif
     
@@ -1069,10 +1388,12 @@ file_mtime(const char *name, s64 *timestamp)
     yassert(timestamp != NULL);
     if(stat(name, &st) >= 0)
     {
-#if !__APPLE__
-        s64 ts = (1000000LL * st.st_mtim.tv_sec) + (st.st_mtim.tv_nsec / 1000LL);
+#ifdef WIN32
+        s64 ts = ONE_SECOND_US * st.st_mtime;
+#elif !__APPLE__
+        s64 ts = (ONE_SECOND_US * st.st_mtim.tv_sec) + (st.st_mtim.tv_nsec / 1000LL);
 #else
-        s64 ts = (1000000LL * st.st_mtimespec.tv_sec) + (st.st_mtimespec.tv_nsec / 1000LL);
+        s64 ts = (ONE_SECOND_US * st.st_mtimespec.tv_sec) + (st.st_mtimespec.tv_nsec / 1000LL);
 #endif
         *timestamp = ts;
         return SUCCESS;
@@ -1101,10 +1422,12 @@ fd_mtime(int fd, s64 *timestamp)
     yassert(timestamp != NULL);
     if(fstat(fd, &st) >= 0)
     {
-#if !__APPLE__
-        s64 ts = (1000000LL * st.st_mtim.tv_sec) + (st.st_mtim.tv_nsec / 1000LL);
+#ifdef WIN32
+        s64 ts = ONE_SECOND_US * st.st_mtime;
+#elif !__APPLE__
+        s64 ts = (ONE_SECOND_US * st.st_mtim.tv_sec) + (st.st_mtim.tv_nsec / 1000LL);
 #else
-        s64 ts = (1000000LL * st.st_mtimespec.tv_sec) + (st.st_mtimespec.tv_nsec / 1000LL);
+        s64 ts = (ONE_SECOND_US * st.st_mtimespec.tv_sec) + (st.st_mtimespec.tv_nsec / 1000LL);
 #endif
         *timestamp = ts;
         return SUCCESS;
@@ -1116,6 +1439,41 @@ fd_mtime(int fd, s64 *timestamp)
     }
 }
 
+ya_result
+fd_setcloseonexec(int fd)
+{
+#ifndef WIN32
+    int ret = fcntl(fd, F_SETFD, FD_CLOEXEC);
+    if(FAIL(ret))
+    {
+        ret = ERRNO_ERROR;
+    }
+    return ret;
+#else
+    return SUCCESS;
+#endif
+}
+
+ya_result
+fd_setnonblocking(int fd)
+{
+#ifndef WIN32
+    int ret;
+    if(ISOK(ret = fcntl(fd, F_GETFL, 0)))
+    {
+        fcntl(fd, F_SETFL, ret | O_NONBLOCK);
+    }
+    else
+    {
+        ret = ERRNO_ERROR;
+    }
+
+    return ret;
+#else
+    return SUCCESS;
+#endif
+}
+
 /**
  * Fixes an issue with the dirent not always set as expected.
  *
@@ -1163,7 +1521,7 @@ dirent_get_file_type(const char *folder, const char *name)
     d_type = DT_UNKNOWN;
     
     /*
-     * If the FS OR the OS does not supports d_type :
+     * If the FS OR the OS does not support d_type :
      */
 
     if(ISOK(snprintf(fullpath, sizeof(fullpath), "%s/%s", folder, name)))
@@ -1212,12 +1570,18 @@ readdir_forall(const char *basedir, readdir_callback *func, void *args)
         }
         
         // ignore names "." and ".."
+
+#ifndef WIN32
+        const char* tmp_name = tmp->d_name;
+#else
+        const char* tmp_name = tmp->name;
+#endif
         
-        if(tmp->d_name[0] == '.')
+        if(tmp_name[0] == '.')
         {
             if(
-                ((tmp->d_name[1] == '.') && (tmp->d_name[2] == '\0')) ||
-                (tmp->d_name[1] == '\0')
+                ((tmp_name[1] == '.') && (tmp_name[2] == '\0')) ||
+                (tmp_name[1] == '\0')
                 )
             {
                 group_mutex_unlock(&readdir_mutex, GROUP_MUTEX_WRITE);
@@ -1225,16 +1589,16 @@ readdir_forall(const char *basedir, readdir_callback *func, void *args)
             }
         }
         
-        size_t name_len = strlen(tmp->d_name);
+        size_t name_len = strlen(tmp_name);
         
         if(name_len + basedir_len + 1 + 1 > sizeof(path))
         {
             group_mutex_unlock(&readdir_mutex, GROUP_MUTEX_WRITE);
-            log_err("readdir_forall: '%s/%s' is bigger than expected (%i): skipping", basedir, tmp->d_name, sizeof(path));
+            log_err("readdir_forall: '%s/%s' is bigger than expected (%i): skipping", basedir, tmp_name, sizeof(path));
             continue;
         }
         
-        memcpy(name, tmp->d_name, name_len + 1);
+        memcpy(name, tmp_name, name_len + 1);
         
         group_mutex_unlock(&readdir_mutex, GROUP_MUTEX_WRITE);
 
@@ -1284,4 +1648,115 @@ readdir_forall(const char *basedir, readdir_callback *func, void *args)
     return ret;
 }
 
+struct file_mtime_set_s
+{
+    ptr_set files_mtime;
+    char *name;
+    bool is_new;
+};
+
+typedef struct file_mtime_set_s file_mtime_set_t;
+
+static ptr_set file_mtime_sets = { NULL, ptr_set_asciizp_node_compare};
+static mutex_t file_mtime_sets_mtx;
+
+file_mtime_set_t*
+file_mtime_set_get_for_file(const char *filename)
+{
+    file_mtime_set_t *ret;
+    mutex_lock(&file_mtime_sets_mtx);
+    ptr_node *sets_node = ptr_set_insert(&file_mtime_sets, (char*)filename);
+    if(sets_node->value != NULL)
+    {
+        ret = (file_mtime_set_t*)sets_node->value;
+    }
+    else
+    {
+        sets_node->key = strdup(filename);
+        ZALLOC_OBJECT_OR_DIE(ret, file_mtime_set_t, GENERIC_TAG);
+        ret->files_mtime.root = NULL;
+        ret->files_mtime.compare = ptr_set_asciizp_node_compare;
+        ret->name = strdup(filename);
+        ret->is_new = TRUE;
+        sets_node->value = ret;
+        file_mtime_set_add_file(ret, filename);
+    }
+    mutex_unlock(&file_mtime_sets_mtx);
+    return ret;
+}
+
+void
+file_mtime_set_add_file(file_mtime_set_t *ctx, const char *filename)
+{
+    s64 mtime;
+
+    if(FAIL(file_mtime(filename, &mtime)))
+    {
+        mtime = MIN_S64;
+    }
+
+    ptr_node *node = ptr_set_insert(&ctx->files_mtime, (char*)filename);
+    if(node->value == NULL)
+    {
+        node->key = strdup(filename);
+        node->value_s64 = mtime;
+    }
+}
+
+bool
+file_mtime_set_modified(file_mtime_set_t *ctx)
+{
+    if(ctx->is_new)
+    {
+        ctx->is_new = FALSE;
+        return TRUE;
+    }
+
+    ptr_set_iterator iter;
+    ptr_set_iterator_init(&ctx->files_mtime, &iter);
+    while(ptr_set_iterator_hasnext(&iter))
+    {
+        ptr_node *node = ptr_set_iterator_next_node(&iter);
+        const char *filename = (const char*)node->key;
+        s64 mtime;
+        if(ISOK(file_mtime(filename, &mtime)))
+        {
+            if(node->value_s64 < mtime)
+            {
+                return TRUE;
+            }
+        }
+        else
+        {
+            return TRUE;
+        }
+    }
+    return FALSE;
+}
+
+void
+file_mtime_set_clear(file_mtime_set_t *ctx)
+{
+    ptr_set_iterator iter;
+    ptr_set_iterator_init(&ctx->files_mtime, &iter);
+    while(ptr_set_iterator_hasnext(&iter))
+    {
+        ptr_node *node = ptr_set_iterator_next_node(&iter);
+        free(node->key);
+    }
+    ptr_set_destroy(&ctx->files_mtime);
+    file_mtime_set_add_file(ctx, ctx->name);
+}
+
+void
+file_mtime_set_delete(file_mtime_set_t *ctx)
+{
+    mutex_lock(&file_mtime_sets_mtx);
+    ptr_set_delete(&file_mtime_sets, ctx->name);
+    mutex_unlock(&file_mtime_sets_mtx);
+    free(ctx->name);
+    file_mtime_set_clear(ctx);
+    ZFREE_OBJECT(ctx);
+}
+
 /** @} */
