@@ -1,36 +1,37 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
 /** @defgroup streaming Streams
  *  @ingroup dnscore
  *  @brief
@@ -90,13 +91,15 @@ static inline void file_input_stream_debug_bench_register()
  */
 
 static ya_result
-file_input_stream_read(input_stream* stream_, u8* buffer, u32 len)
+file_input_stream_read(input_stream* stream_, void* buffer_, u32 len)
 {
 #if DEBUG_BENCH_FD
     file_input_stream_debug_bench_register();
     u64 bench = debug_bench_start(&debug_read);
 #endif
     
+    u8 *buffer = (u8*)buffer_;
+    
     file_input_stream* stream = (file_input_stream*)stream_;
 
     u8* start = buffer;
@@ -117,7 +120,7 @@ file_input_stream_read(input_stream* stream_, u8* buffer, u32 len)
                 continue;
             }
             
-#ifdef DEBUG
+#if DEBUG
             if(err == EBADF)
             {
                 fprintf(stderr, "bad file descriptor %i", stream->data.fd);
@@ -189,14 +192,16 @@ file_input_stream_skip(input_stream* stream_, u32 len)
     return ERRNO_ERROR;
 }
 
-static const input_stream_vtbl file_input_stream_vtbl ={
+static const input_stream_vtbl file_input_stream_vtbl =
+{
     file_input_stream_read,
     file_input_stream_skip,
     file_input_stream_close,
     "file_input_stream"
 };
 
-static const input_stream_vtbl file_input_stream_noclose_vtbl ={
+static const input_stream_vtbl file_input_stream_noclose_vtbl =
+{
     file_input_stream_read,
     file_input_stream_skip,
     file_input_stream_noclose,
@@ -246,7 +251,7 @@ fd_input_stream_detach(input_stream *stream_)
 ya_result
 file_input_stream_open(input_stream *stream_, const char *filename)
 {
-    int fd = open_ex(filename, O_RDONLY);
+    int fd = open_ex(filename, O_RDONLY|O_CLOEXEC);
     
 #if (_XOPEN_SOURCE >= 600 || _POSIX_C_SOURCE >= 200112L) && !defined(__gnu__hurd__)
     if(fd >= 0)
@@ -261,7 +266,7 @@ file_input_stream_open(input_stream *stream_, const char *filename)
 ya_result
 file_input_stream_open_ex(input_stream *stream_, const char *filename, int flags)
 {
-    int fd = open_ex(filename, O_RDONLY | flags);
+    int fd = open_ex(filename, O_RDONLY|O_CLOEXEC| flags);
     
 #if (_XOPEN_SOURCE >= 600 || _POSIX_C_SOURCE >= 200112L) && !defined(__gnu__hurd__)
     if(fd >= 0)
@@ -273,14 +278,16 @@ file_input_stream_open_ex(input_stream *stream_, const char *filename, int flags
     return fd_input_stream_attach(stream_, fd);
 }
 
-ya_result fd_input_stream_get_filedescriptor(input_stream* stream_)
+ya_result
+fd_input_stream_get_filedescriptor(input_stream* stream_)
 {
     file_input_stream* stream = (file_input_stream*)stream_;
 
     return stream->data.fd ;
 }
 
-ya_result fd_input_stream_seek(input_stream* stream_, u64 offset)
+ya_result
+fd_input_stream_seek(input_stream* stream_, u64 offset)
 {
     if(is_fd_input_stream(stream_))
     {
@@ -315,7 +322,8 @@ is_fd_input_stream(input_stream* stream_)
     return (stream != NULL) && (stream->vtbl->read == file_input_stream_read);
 }
 
-void file_input_steam_advise_sequential(input_stream* stream_)
+void
+file_input_steam_advise_sequential(input_stream* stream_)
 {
     file_input_stream* stream = (file_input_stream*)stream_;
     
@@ -327,7 +335,4 @@ void file_input_steam_advise_sequential(input_stream* stream_)
 #endif
 }
 
-
 /** @} */
-
-/*----------------------------------------------------------------------------*/
