@@ -1,36 +1,37 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
 /** @defgroup logger Logging functions
  *  @ingroup dnscore
  *  @brief
@@ -48,13 +49,13 @@
 #define LOGGER_CHANNEL_SYSLOG_MULTITHREADED 0
 #define SYSLOG_IS_THREADSAFE 0
 
-#if LOGGER_CHANNEL_SYSLOG_MULTITHREADED == 0
+#if !LOGGER_CHANNEL_SYSLOG_MULTITHREADED
 #undef SYSLOG_IS_THREADSAFE
 #define SYSLOG_IS_THREADSAFE 1
 #endif
 
-#if SYSLOG_IS_THREADSAFE == 0
-#include <pthread.h>
+#if !SYSLOG_IS_THREADSAFE
+#include <dnscore/thread.h>
 #endif
 
 #include "dnscore/logger_channel_syslog.h"
@@ -77,6 +78,7 @@ typedef struct logger_syslog_data logger_syslog_data;
 static ya_result
 logger_channel_syslog_constmsg(logger_channel* chan, int level, char* text, u32 text_len, u32 date_offset)
 {
+    (void)chan;
     (void)text_len;
     if(level > LOG_DEBUG)
     {
@@ -90,6 +92,8 @@ logger_channel_syslog_constmsg(logger_channel* chan, int level, char* text, u32
 static ya_result
 logger_channel_syslog_msg(logger_channel* chan, int level, char* text, ...)
 {
+    (void)chan;
+
     char tmp[SYSLOG_MAX_LINE_SIZE];
 
     va_list args;
@@ -136,6 +140,8 @@ static void
 logger_channel_syslog_flush(logger_channel* chan)
 {
     /* NOP */
+
+    (void)chan;
 }
 
 static void
@@ -183,8 +189,14 @@ static const logger_channel_vtbl syslog_vtbl = {
 void
 logger_channel_syslog_open(const char* ident, int options, int facility, logger_channel* chan)
 {
+    if(chan == NULL)
+    {
+        osformatln(termerr, "tried to open syslog (%s) on uninitialised channel", ident);
+        return;
+    }
+    
     logger_syslog_data *sd;
-    MALLOC_OR_DIE(logger_syslog_data*, sd, sizeof(logger_syslog_data), 0x4d5254534e414843); /* CHANSTRM */
+    MALLOC_OBJECT_OR_DIE(sd, logger_syslog_data, 0x4d5254534e414843); /* CHANSTRM */
     sd->ident = strdup(ident);
     sd->options = options;
     sd->facility = facility;
