@@ -1,36 +1,37 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
 /** @defgroup dnscoretools Generic Tools
  *  @ingroup dnscore
  *  @brief
@@ -39,7 +40,6 @@
  */
 
 #include "dnscore/dnscore-config.h"
-#include "dnscore/dnscore-config.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -48,10 +48,12 @@
 
 #include <arpa/inet.h>
 #include <sys/socket.h>
+#include <dnscore/timems.h>
+#include <dnscore/config_settings.h>
 
 #include "dnscore/parsing.h"
 
-#if !HAS_TIMEGM
+#if !HAVE_TIMEGM && !HAS_TIMEGM
 static inline time_t timegm(struct tm *tv)
 {
     return timegm_internal(tv);
@@ -106,6 +108,8 @@ parse_u32_check_range(const char *src, u32 *dst, u32 min, u32 max, u8 base)
     return OK;
 }
 
+
+
 ya_result
 parse_u32_check_range_len_base10(const char *src, u32 src_len, u32 *dst, u32 min, u32 max)
 {
@@ -219,9 +223,6 @@ parse_s32_check_range_len_base10(const char *src, u32 src_len, s32 *dst, s32 min
     return SUCCESS;
 }
 
-#if 0 /* fix */
-#else // THX
-
 ya_result
 parse_u64_check_range_len_base10(const char *src, u32 src_len, u64 *dst, u64 min, u64 max)
 {
@@ -257,9 +258,7 @@ parse_u64_check_range_len_base10(const char *src, u32 src_len, u64 *dst, u64 min
                 return PARSEINT_ERROR;
             }
 
-            output_value *= base_multiplier;
-
-            output_value += value;
+            output_value += value * base_multiplier;
 
             base_multiplier *= 10;
         }
@@ -277,9 +276,7 @@ parse_u64_check_range_len_base10(const char *src, u32 src_len, u64 *dst, u64 min
                 return PARSEINT_ERROR;
             }
 
-            output_value *= base_multiplier;
-
-            output_value += value;
+            output_value += value * base_multiplier;
 
             base_multiplier *= 10;
         }
@@ -300,7 +297,7 @@ parse_u64_check_range_len_base10(const char *src, u32 src_len, u64 *dst, u64 min
                 return PARSEINT_ERROR;
             }
 
-            output_value *= base_multiplier;
+            value *= base_multiplier;
 
             if(output_value > max - value)  // check before addition there will be no 64 bits overflow
             {
@@ -316,11 +313,10 @@ parse_u64_check_range_len_base10(const char *src, u32 src_len, u64 *dst, u64 min
         return PARSEINT_ERROR;
     }
     
-    *dst = (u32)output_value;
+    *dst = output_value;
 
     return SUCCESS;
 }
-#endif // THX
 
 
 
@@ -344,13 +340,13 @@ parse_yyyymmddhhmmss_check_range_len(const char *src, u32 src_len, time_t *dst)
         return PARSEDATE_ERROR;
     }
 
-#ifdef DEBUG
+#if DEBUG
     memset(&thetime, 0xff, sizeof(thetime));
 #endif
     
     u32 tmp_u32;
     
-    if(FAIL(parse_u32_check_range_len_base10(src, 4, &tmp_u32, 1970, 2038)))
+    if(FAIL(parse_u32_check_range_len_base10(src, 4, &tmp_u32, 1970, 2106/*2038*/)))
     {
         return PARSEDATE_ERROR;
     }
@@ -773,7 +769,7 @@ parse_ip_address(const char *src, u32 src_len_, u8 *dst, u32 dst_len)
     }
     
     char tmp[64];
-    src_len = MIN(src_len, sizeof(tmp)-1);
+    src_len = MIN((size_t)src_len, sizeof(tmp)-1);
     memcpy(tmp, src, src_len);
     tmp[src_len] = '\0';
 
