@@ -1,36 +1,36 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
 
 #include "dnscore/dnscore-config.h"
 #include <unistd.h>
@@ -49,7 +49,6 @@
  *----------------------------------------------------------------------------*/
 
 #define MODULE_MSG_HANDLE		g_system_logger
-extern logger_handle *g_system_logger;
 
 #define DEBUG_PIPE_OUTPUT_STREAM 0
 
@@ -131,6 +130,7 @@ pipe_stream_output_write(output_stream* stream, const u8* buffer, u32 len)
 static ya_result
 pipe_stream_output_flush(output_stream* stream)
 {
+    (void)stream;
     return SUCCESS;
 }
 
@@ -159,14 +159,16 @@ static const output_stream_vtbl pipe_stream_output_vtbl = {
 
 
 static ya_result
-pipe_stream_input_read(input_stream* stream, u8 *buffer, u32 len)
+pipe_stream_input_read(input_stream* stream, void *buffer_, u32 len)
 {
     if(len == 0)
     {
         return 0;
     }
     
-#ifdef DEBUG
+    u8 *buffer = (u8*)buffer_;
+    
+#if DEBUG
     memset(buffer, 0xff, len);
 #endif
 
@@ -272,11 +274,11 @@ void
 pipe_stream_init(output_stream *output, input_stream *input, u32 buffer_size)
 {
     pipe_stream_data *data;
-    MALLOC_OR_DIE(pipe_stream_data*, data, sizeof(pipe_stream_data), PIPESDTA_TAG);   
+    MALLOC_OBJECT_OR_DIE(data, pipe_stream_data, PIPESDTA_TAG);   
     ZEROMEMORY(data, sizeof(pipe_stream_data));
     MALLOC_OR_DIE(u8*, data->buffer, buffer_size, PIPESBFR_TAG);
     
-#ifdef DEBUG
+#if DEBUG
     memset(data->buffer, 0xff, buffer_size);
 #endif
     
@@ -319,6 +321,4 @@ pipe_stream_write_available(output_stream *input)
     return data->write_avail;
 }
 
-/*    ------------------------------------------------------------    */
-
 /** @} */
