@@ -1,36 +1,37 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
 /** @defgroup
  *  @ingroup dnscore
  *  @brief
@@ -46,19 +47,28 @@
 #include <stdlib.h>
 #include <arpa/inet.h>
 #include <ctype.h>
+#include <netdb.h>
+#include <strings.h>
 
 #include "dnscore/rfc.h"
 #include "dnscore/ctrl-rfc.h"
 #include "dnscore/string_set.h"
+#include "dnscore/ptr_set.h"
+#include "dnscore/u32_set.h"
+#include "dnscore/format.h"
+#include "dnscore/mutex.h"
 
 #define DNSCORE_RFC_C
 
 /*------------------------------------------------------------------------------
  * GLOBAL VARIABLES */
 
+static mutex_t rfc_init_mtx = MUTEX_INITIALIZER;
 static string_node* class_set = NULL;
 static string_node* type_set = NULL;
 static string_node* dnssec_algo_set = NULL;
+static string_set word_set = NULL;
+static bool rfc_init_done = FALSE;
 
 const class_table qclass[] = {
     { CLASS_IN,   CLASS_IN_NAME   },
@@ -132,12 +142,16 @@ const type_table qtype[] = {
     { TYPE_TALINK,     TYPE_TALINK_NAME     }, // 58
     { TYPE_CDS,        TYPE_CDS_NAME        }, // 59
 
-
+    { TYPE_CDNSKEY,    TYPE_CDNSKEY_NAME    }, // 60
+    { TYPE_OPENPGPKEY, TYPE_OPENPGPKEY_NAME }, // 61
+    { TYPE_CSYNC,      TYPE_CSYNC_NAME      }, // 62
 
     { TYPE_SPF,        TYPE_SPF_NAME        }, // 99
     { TYPE_UINFO,      TYPE_UINFO_NAME      }, // 100
 
-
+    { TYPE_UID,        TYPE_UID_NAME        }, // 101
+    { TYPE_GID,        TYPE_GID_NAME        }, // 102
+    { TYPE_UNSPEC,     TYPE_UNSPEC_NAME     }, // 103
 
     { TYPE_NID,        TYPE_NID_NAME        }, // 104
     { TYPE_L32,        TYPE_L32_NAME        }, // 105
@@ -157,24 +171,11 @@ const type_table qtype[] = {
     { TYPE_ANY,        TYPE_ANY_NAME        }, // 255
     { TYPE_URI,        TYPE_URI_NAME        }, // 256
     { TYPE_CAA,        TYPE_CAA_NAME        }, // 257
+    { TYPE_AVC,        TYPE_AVC_NAME        }, // 258
     { TYPE_TA,         TYPE_TA_NAME         }, // 32768
     { TYPE_DLV,        TYPE_DLV_NAME        }, // 32769
     
-#if HAS_DYNAMIC_PROVISIONING
-    { TYPE_ZONE_TYPE,           TYPE_ZONE_TYPE_NAME         },
-    { TYPE_ZONE_FILE,           TYPE_ZONE_FILE_NAME         },
-    { TYPE_ZONE_NOTIFY,         TYPE_ZONE_NOTIFY_NAME       },
-    { TYPE_ZONE_MASTER,         TYPE_ZONE_MASTER_NAME       },
-    { TYPE_ZONE_DNSSEC,         TYPE_ZONE_DNSSEC_NAME       },
-    { TYPE_ZONE_SLAVES,         TYPE_ZONE_SLAVES_NAME       },
-    { TYPE_SIGINTV,             TYPE_SIGINTV_NAME           },
-    { TYPE_SIGREGN,             TYPE_SIGREGN_NAME           },
-    { TYPE_SIGJITR,             TYPE_SIGJITR_NAME           },
-    { TYPE_NTFRC,               TYPE_NTFRC_NAME             },
-    { TYPE_NTFRP,               TYPE_NTFRP_NAME             },
-    { TYPE_NTFRPI,              TYPE_NTFRPI_NAME            },
-    { TYPE_NTFAUTO,             TYPE_NTFAUTO_NAME           },
-#endif
+
     
 #if 1//HAS_CTRL
     { TYPE_CTRL_SRVCFGRELOAD,     TYPE_CTRL_SRVCFGRELOAD_NAME     },
@@ -190,6 +191,7 @@ const type_table qtype[] = {
     { TYPE_CTRL_ZONEUNFREEZE,     TYPE_CTRL_ZONEUNFREEZE_NAME     },
     { TYPE_CTRL_ZONEUNFREEZEALL,  TYPE_CTRL_ZONEUNFREEZEALL_NAME  },
     { TYPE_CTRL_ZONESYNC,         TYPE_CTRL_ZONESYNC_NAME         },
+    { TYPE_CTRL_ZONENOTIFY,       TYPE_CTRL_ZONENOTIFY_NAME       },
 
 #endif  
     { 0,               NULL                 }
@@ -209,31 +211,34 @@ const dnssec_algo_table dnssec_algo[] = {
 //  { DNSKEY_ALGORITHM_GOST,            DNSKEY_ALGORITHM_GOST_NAME            },     // 12
     { DNSKEY_ALGORITHM_ECDSAP256SHA256, DNSKEY_ALGORITHM_ECDSAP256SHA256_NAME },     // 13
     { DNSKEY_ALGORITHM_ECDSAP384SHA384, DNSKEY_ALGORITHM_ECDSAP384SHA384_NAME },     // 14
+    { DNSKEY_ALGORITHM_ED25519,         DNSKEY_ALGORITHM_ED25519_NAME         },     // 15
+    { DNSKEY_ALGORITHM_ED448,           DNSKEY_ALGORITHM_ED448_NAME           },     // 16
+#ifdef DNSKEY_ALGORITHM_DUMMY
+    { DNSKEY_ALGORITHM_DUMMY,           DNSKEY_ALGORITHM_DUMMY_NAME           },     // 254
+#endif
     { 0,                                NULL                                  }
 };
 
 
 static char *opcode[16] =
 {
-    "QUERY",
-    "IQUERY",
+    "QUERY",    // 0
+    "IQUERY",   // 1 obsolete
     "STATUS",
+    "OPCODE3",
     "NOTIFY",
-
     "UPDATE",
-    "?",
-    "?",
-    "?",
-
-    "?",
+    "OPCODE6",
+    "OPCODE7",
+    
+    "OPCODE8",        // 8
     "CTRL", /* special for yadifa client for view the result in verbose mode */
-    "?",
-    "?",
-
-    "?",
-    "?",
-    "?",
-    "?"
+    "OPCODE9",
+    "OPCODE10",
+    "OPCODE11",
+    "OPCODE12",
+    "OPCODE13",
+    "OPCODE14"
 };
 
 static char *rcode[32] =
@@ -252,66 +257,88 @@ static char *rcode[32] =
                               //   9       /* Not Authorized                     rfc 2845 */
     "NOTZONE",                //   10      /* Name not contained in zone         rfc 2136 */
 
-    "?",
-    "?",
-    "?",
-    "?",
-    "?",
+    "RCODE11",
+    "RCODE12",
+    "RCODE13",
+    "RCODE14",
+    "RCODE15",
 
     "BADVERS",                //   16      /* Bad OPT Version         rfc 2671 / rfc 6891 */
 
-#if 0 /* fix */
-#else // THX
+    "RCODE17",
+    "RCODE18",
+    "RCODE19",
 
-    "-",
-    "-",
-    "-",
+    "RCODE20",
+    "RCODE21",
+    "RCODE22",
 
-    "-",
-    "-",
-    "-",
-
-#endif // THX
-    "-",
+    "RCODE23",
     
-    "-",
-    "-",
-    "-",
-    "-",
+    "RCODE24",
+    "RCODE25",
+    "RCODE26",
+    "RCODE27",
     
-    "-",
-    "-",
-    "-",
-    "-"
+    "RCODE28",
+    "RCODE29",
+    "RCODE30",
+    "RCODE31"
 };
 
+
+static char*
+rfc_word_get(const char *w)
+{
+    string_node *node = string_set_insert(&word_set, w); // assumes new value set to 0
+    if(node->value == 0)
+    {
+        node->key = strdup(w);
+    }
+    
+    if(node->value < 0xffff)
+    {
+        ++node->value;
+    }
+    
+    return (char*)node->key;
+}
+
+static void rfc_word_destroy_cb(string_node *node)
+{
+    free((char*)node->key);node->key = NULL;
+}
+
+static void rfc_word_destroy()
+{
+    string_set_callback_and_destroy(&word_set, rfc_word_destroy_cb);
+}
+
 const char*
-get_opcode(u16 o)
+dns_message_opcode_get_name(u16 o)
 {
     return opcode[o & 0x0f];
 }
-
-
 const char*
-get_rcode(u16 r)
+dns_message_rcode_get_name(u16 r)
 {
     return rcode[r & 0x1f];
 }
-
-
 const char*
-get_name_from_class(u16 c)
+dns_class_get_name(u16 c)
 {
     switch(c)
     {
         case CLASS_IN:
             return CLASS_IN_NAME;
-        case CLASS_HS:
-            return CLASS_HS_NAME;
         case CLASS_CH:
             return CLASS_CH_NAME;
+        case CLASS_HS:
+            return CLASS_HS_NAME;
         case CLASS_CTRL:
             return CLASS_CTRL_NAME;
+        case CLASS_NONE:
+            return CLASS_NONE_NAME;
         case CLASS_ANY:
             return CLASS_ANY_NAME;
         default:
@@ -319,9 +346,8 @@ get_name_from_class(u16 c)
     }
 }
 
-
 const char*
-get_name_from_type(u16 t)
+dns_type_get_name(u16 t)
 {
     switch(t)
     {
@@ -437,13 +463,30 @@ get_name_from_type(u16 t)
             return TYPE_RKEY_NAME;
         case TYPE_TALINK:
             return TYPE_TALINK_NAME;
+        case TYPE_CDS:
+            return TYPE_CDS_NAME;
+        case TYPE_CDNSKEY:
+            return TYPE_CDNSKEY_NAME;
+        case TYPE_OPENPGPKEY:
+            return TYPE_OPENPGPKEY_NAME;
 
         case TYPE_SPF:
             return TYPE_SPF_NAME;
         case TYPE_UINFO:
             return TYPE_UINFO_NAME;
 
-
+        case TYPE_NID:
+            return TYPE_NID_NAME;
+        case TYPE_L32:
+            return TYPE_L32_NAME;
+        case TYPE_L64:
+            return TYPE_L64_NAME;
+        case TYPE_LP:
+            return TYPE_LP_NAME;
+        case TYPE_EUI48:
+            return TYPE_EUI48_NAME;
+        case TYPE_EUI64:
+            return TYPE_EUI64_NAME;
         case TYPE_TKEY:
             return TYPE_TKEY_NAME;
         case TYPE_TSIG:
@@ -467,14 +510,43 @@ get_name_from_type(u16 t)
         case TYPE_DLV:
             return TYPE_DLV_NAME;
             
-
+#if HAS_CTRL
+        case TYPE_CTRL_SRVSHUTDOWN:
+            return TYPE_CTRL_SHUTDOWN_NAME;
+        case TYPE_CTRL_ZONEFREEZE:
+            return TYPE_CTRL_ZONEFREEZE_NAME;
+        case TYPE_CTRL_ZONEUNFREEZE:
+            return TYPE_CTRL_ZONEUNFREEZE_NAME;
+        case TYPE_CTRL_ZONERELOAD:
+            return TYPE_CTRL_ZONERELOAD_NAME;
+        case TYPE_CTRL_SRVLOGREOPEN:
+            return TYPE_CTRL_LOGREOPEN_NAME;
+        case TYPE_CTRL_SRVCFGRELOAD:
+            return TYPE_CTRL_SRVCFGRELOAD_NAME;
+        case TYPE_CTRL_ZONECFGRELOAD:
+            return TYPE_CTRL_ZONECFGRELOAD_NAME;
+        case TYPE_CTRL_ZONESYNC:
+            return TYPE_CTRL_ZONESYNC_NAME;
+        case TYPE_CTRL_SRVQUERYLOG:
+            return TYPE_CTRL_SRVQUERYLOG_NAME;
+        case TYPE_CTRL_SRVLOGLEVEL:
+            return TYPE_CTRL_SRVLOGLEVEL_NAME;
+        case TYPE_CTRL_ZONENOTIFY:
+            return TYPE_CTRL_ZONENOTIFY_NAME;
+        //case TYPE_CTRL_%:
+        //    return TYPE_CTRL_SCFGMERGE_NAME;
+        //case TYPE_CTRL_CFGSAVE:
+        //    return TYPE_CTRL_CFGSAVE_NAME;
+        //case TYPE_CTRL_CFGLOAD:
+        //    return TYPE_CTRL_CFGLOAD_NAME;
+#endif
         default:
             return NULL;
     }
 }
 
 const char*
-get_name_from_dnssec_algo(u16 d)
+dns_encryption_algorithm_get_name(u16 d)
 {
     switch(d)
     {
@@ -500,7 +572,14 @@ get_name_from_dnssec_algo(u16 d)
             return DNSKEY_ALGORITHM_ECDSAP256SHA256_NAME;
         case DNSKEY_ALGORITHM_ECDSAP384SHA384:
             return DNSKEY_ALGORITHM_ECDSAP384SHA384_NAME;
-
+        case DNSKEY_ALGORITHM_ED25519:
+            return DNSKEY_ALGORITHM_ED25519_NAME;
+        case DNSKEY_ALGORITHM_ED448:
+            return DNSKEY_ALGORITHM_ED448_NAME;
+#ifdef DNSKEY_ALGORITHM_DUMMY
+        case DNSKEY_ALGORITHM_DUMMY:
+            return DNSKEY_ALGORITHM_DUMMY_NAME;
+#endif
         default:
             return NULL;
     }
@@ -516,9 +595,9 @@ get_name_from_dnssec_algo(u16 d)
  *  @retval NOK
  */
 int
-get_class_from_name(const char *src, u16 *dst)
+dns_class_from_name(const char *src, u16 *dst)
 {
-    const string_node *node = string_set_avl_find(&class_set, (const char *)src);
+    const string_node *node = string_set_find(&class_set, (const char *)src);
 
     if(node != NULL)
     {
@@ -564,9 +643,9 @@ get_class_from_name(const char *src, u16 *dst)
  *  @retval NOK
  */
 int
-get_type_from_name(const char *src, u16 *dst)
+dns_type_from_name(const char *src, u16 *dst)
 {
-    string_node *node = string_set_avl_find(&type_set, (const char *)src);
+    string_node *node = string_set_find(&type_set, (const char *)src);
 
     if(node != NULL)
     {
@@ -613,9 +692,9 @@ get_type_from_name(const char *src, u16 *dst)
  *  @retval NOK
  */
 int
-get_dnssec_algo_from_name(const char *src, u8 *dst)
+dns_encryption_algorithm_from_name(const char *src, u8 *dst)
 {
-    const string_node *node = string_set_avl_find(&dnssec_algo_set, (const char *)src);
+    const string_node *node = string_set_find(&dnssec_algo_set, (const char *)src);
 
     if(node != NULL)
     {
@@ -630,147 +709,396 @@ get_dnssec_algo_from_name(const char *src, u8 *dst)
 
 
 int
-get_class_from_case_name(const char *src, u16 *dst)
+dns_class_from_case_name(const char *src, u16 *dst)
 {
     char txt[16];
-    s32 n = strlen(src);
-    if(n > sizeof(txt))
+    size_t n = strlen(src);
+    if(n >= sizeof(txt))
     {
         return UNKNOWN_DNS_CLASS;
     }
     
-    for(s32 i = 0; i < n; i++)
+    for(size_t i = 0; i < n; i++)
     {
         txt[i] = toupper(src[i]);
     }
     
     txt[n] = '\0';
     
-    return get_class_from_name(txt, dst);
+    return dns_class_from_name(txt, dst);
 }
 
 
 int
-get_type_from_case_name(const char *src, u16 *dst)
+dns_type_from_case_name(const char *src, u16 *dst)
 {
     char txt[16];
-    s32 n = strlen(src);
-    if(n > sizeof(txt))
+    size_t n = strlen(src);
+    if(n >= sizeof(txt))
     {
         return UNKNOWN_DNS_TYPE;
     }
     
-    for(s32 i = 0; i < n; i++)
+    for(size_t i = 0; i < n; i++)
     {
         txt[i] = toupper(src[i]);
     }
     
     txt[n] = '\0';
     
-    ya_result ret = get_type_from_name(txt, dst);
+    ya_result ret = dns_type_from_name(txt, dst);
     
     return ret;
 }
 
 
 int
-get_dnssec_algo_from_case_name(const char *src, u8 *dst)
+dns_encryption_algorithm_from_case_name(const char *src, u8 *dst)
 {
-    char txt[16];
-    s32 n = strlen(src);
-    if(n > sizeof(txt))
+    char txt[32];
+    size_t n = strlen(src);
+    if(n >= sizeof(txt))
     {
         return UNKNOWN_DNSSEC_ALGO;
     }
 
-    for(s32 i = 0; i < n; i++)
+    for(size_t i = 0; i < n; i++)
     {
         txt[i] = toupper(src[i]);
     }
 
     txt[n] = '\0';
 
-    return get_dnssec_algo_from_name(txt, dst);
+    return dns_encryption_algorithm_from_name(txt, dst);
 }
 
 
 int
-get_type_from_case_name_len(const char *src, int src_len, u16 *dst)
+dns_type_from_case_name_length(const char *src, int src_len, u16 *dst)
 {
     char txt[16];
 
-    if(src_len > sizeof(txt))
+    if(src_len >= (int)sizeof(txt))
     {
         return UNKNOWN_DNS_TYPE;
     }
     
-    for(s32 i = 0; i < src_len; i++)
+    for(int i = 0; i < src_len; i++)
     {
         txt[i] = toupper(src[i]);
     }
     
     txt[src_len] = '\0';
     
-    ya_result ret = get_type_from_name(txt, dst);
+    ya_result ret = dns_type_from_name(txt, dst);
     
     return ret;
 }
 
+static ptr_set protocol_name_to_id_set = PTR_SET_ASCIIZCASE_EMPTY;
+static u32_set protocol_id_to_name_set = U32_SET_EMPTY;
+static ptr_set server_name_to_port_set = PTR_SET_ASCIIZCASE_EMPTY;
+static u32_set server_port_to_name_set = U32_SET_EMPTY;
+
+static void
+protocol_name_to_id_init()
+{
+#ifndef WIN32
+    if(ptr_set_isempty(&protocol_name_to_id_set))
+    {
+        struct protoent *ent;
+        
+        for(;;)
+        {
+            if((ent = getprotoent()) == NULL)
+            {
+                break;
+            }
+            
+            ptr_node *node = ptr_set_insert(&protocol_name_to_id_set, ent->p_name);
+            node->key = rfc_word_get(ent->p_name);
+            
+            if(node->value != NULL)
+            {
+                continue;
+            }
+            
+            node->value = (void*)(intptr)ent->p_proto;
+            
+            u32_node *id_node = u32_set_insert(&protocol_id_to_name_set, ent->p_proto);
+            id_node->value = node->key;
+            
+            char **a = ent->p_aliases;
+            if(a != NULL)
+            {
+                while(*a != NULL)
+                {
+                    node = ptr_set_insert(&protocol_name_to_id_set, *a);
+                    
+                    if(node->value == NULL)
+                    {
+                        node->key = rfc_word_get(*a);
+                        node->value = (void*)(intptr)ent->p_proto;
+                    }
+                    ++a;    
+                }
+            }
+        }
+        
+        endprotoent();
+    }
+#endif
+}
+
+static void
+protocol_name_to_id_finalize()
+{
+    ptr_set_destroy(&protocol_name_to_id_set);
+    u32_set_destroy(&protocol_id_to_name_set);
+}
+
+ya_result
+protocol_name_to_id(const char* name, int *out_proto)
+{
+    ya_result ret = PARSEINT_ERROR;
+    
+    if(sscanf(name, "%d", &ret) <= 0)
+    {
+        ptr_node *node = ptr_set_find(&protocol_name_to_id_set, name);
+        
+        if(node != NULL)
+        {
+            ret = (int)(intptr)node->value;
+        }
+    }
+    else
+    {
+        if(ret > 255)
+        {
+            ret = INVALID_STATE_ERROR;
+        }
+    }
+    
+    if(ISOK(ret) && (out_proto != NULL))
+    {
+        *out_proto = ret;
+    }
+    
+    return ret;
+}
+
+ya_result
+protocol_id_to_name(int proto, char *name, size_t name_len)
+{
+    u32_node *proto_node = u32_set_find(&protocol_id_to_name_set, proto);
+    if(proto_node != NULL)
+    {
+        return snformat(name, name_len, "%s", (const char*)proto_node->value);
+    }
+    else
+    {
+        return snformat(name, name_len, "%i", proto);
+    }
+}
+
+static void
+server_name_to_port_init()
+{
+#ifndef WIN32
+    if(ptr_set_isempty(&server_name_to_port_set))
+    {
+        struct servent *ent;
+        
+        for(;;)
+        {
+            if((ent = getservent()) == NULL)
+            {
+                break;
+            }
+            
+            ptr_node *node = ptr_set_insert(&server_name_to_port_set, ent->s_name);
+            
+            if(node->value != NULL)
+            {
+                continue;
+            }
+            
+            node->key = rfc_word_get(ent->s_name);
+            u16 hport = ntohs(ent->s_port);
+            node->value = (void*)(intptr)hport;
+            
+            u32_node *port_node = u32_set_insert(&server_port_to_name_set, hport);
+            port_node->value = node->key;
+            
+            char **a = ent->s_aliases;
+            if(a != NULL)
+            {
+                while(*a != NULL)
+                {
+                    node = ptr_set_insert(&server_name_to_port_set, *a);
+                    if(node->value == NULL)
+                    {
+                        node->key = rfc_word_get(*a);
+                        node->value = (void*)(intptr)ntohs(ent->s_port);
+                    }
+                    
+                    ++a;
+                }
+            }
+        }
+        endservent();
+    }
+#endif
+}
+
+static void
+server_name_to_port_finalize()
+{
+    if(!ptr_set_isempty(&server_name_to_port_set))
+    {
+        ptr_set_destroy(&server_name_to_port_set);
+        u32_set_destroy(&server_port_to_name_set);
+    }
+}
+
+ya_result
+server_name_to_port(const char* name, int *out_port)
+{
+    ya_result ret = PARSEINT_ERROR;
+    
+    if(sscanf(name, "%d", &ret) <= 0)
+    {
+        ptr_node *node = ptr_set_find(&server_name_to_port_set, name);
+        
+        if(node != NULL)
+        {
+            ret = (int)(intptr)node->value;
+        }
+    }
+    else
+    {
+        if(ret > 65535)
+        {
+            ret = INVALID_STATE_ERROR;
+        }
+    }
+    
+    if(ISOK(ret) && (out_port != NULL))
+    {
+        *out_port = ret;
+    }
+    
+    return ret;
+}
+
+ya_result
+server_port_to_name(int port, char *name, size_t name_len)
+{
+    u32_node *port_node = u32_set_find(&server_port_to_name_set, port);
+    if(port_node != NULL)
+    {
+        return snformat(name, name_len, "%s", (const char*)port_node->value);
+    }
+    else
+    {
+        return snformat(name, name_len, "%i", port);
+    }
+}
+
+static void
+rfc_dnssec_algo_init()
+{
+    int i;
+
+    string_set_init(&dnssec_algo_set);
+
+    for(i = 0; dnssec_algo[i].id != 0; i++)
+    {
+        string_node* node = string_set_insert(&dnssec_algo_set, dnssec_algo[i].data);
+        node->value       = dnssec_algo[i].id;
+    }
+
+    // alias
+    {
+        string_node* node = string_set_insert(&dnssec_algo_set, DNSKEY_ALGORITHM_DSASHA1_NSEC3_NAME2);
+        node->value       = DNSKEY_ALGORITHM_DSASHA1_NSEC3;
+    }
+
+    // alias
+    {
+        string_node* node = string_set_insert(&dnssec_algo_set, DNSKEY_ALGORITHM_RSASHA1_NSEC3_NAME2);
+        node->value       = DNSKEY_ALGORITHM_RSASHA1_NSEC3;
+    }
+}
+
+static void
+rfc_dnssec_algo_finalize()
+{
+    string_set_destroy(&dnssec_algo_set);
+}
 
 void
 rfc_init()
 {
+    mutex_lock(&rfc_init_mtx);
+    
+    if(rfc_init_done)
+    {
+        mutex_unlock(&rfc_init_mtx);
+        return;
+    }
+    
+    rfc_init_done = TRUE;
+    
     int i;
 
-    string_set_avl_init(&class_set);
+    string_set_init(&class_set);
 
     for(i = 0; qclass[i].id != 0; i++)
     {
-        string_node* node = string_set_avl_insert(&class_set, qclass[i].data);
+        string_node* node = string_set_insert(&class_set, qclass[i].data);
         node->value       = qclass[i].id;
     }
 
-    string_set_avl_init(&type_set);
+    string_set_init(&type_set);
 
     for(i = 0; qtype[i].id != 0; i++)
     {
-        string_node* node = string_set_avl_insert(&type_set, qtype[i].data);
+        string_node* node = string_set_insert(&type_set, qtype[i].data);
         node->value       = qtype[i].id;
     }
+    
+    protocol_name_to_id_init();
+    server_name_to_port_init();
+    rfc_dnssec_algo_init();
+    
+    mutex_unlock(&rfc_init_mtx);
 }
 
 void
 rfc_finalize()
 {
-    string_set_avl_destroy(&class_set);
-    string_set_avl_destroy(&type_set);
-}
-
-
-void
-rfc_dnssec_algo_init()
-{
-    int i;
-
-    string_set_avl_init(&dnssec_algo_set);
-
-    for(i = 0; dnssec_algo[i].id != 0; i++)
+    mutex_lock(&rfc_init_mtx);
+    if(!rfc_init_done)
     {
-        string_node* node = string_set_avl_insert(&dnssec_algo_set, dnssec_algo[i].data);
-        node->value       = dnssec_algo[i].id;
+        mutex_unlock(&rfc_init_mtx);
+        return;
     }
+    
+    rfc_init_done = FALSE;
+    
+    rfc_dnssec_algo_finalize();
+    server_name_to_port_finalize();
+    protocol_name_to_id_finalize();
+    string_set_destroy(&type_set);
+    string_set_destroy(&class_set);
+    rfc_word_destroy();
+    
+    mutex_unlock(&rfc_init_mtx);
 }
 
-void
-rfc_dnssec_algo_finalize()
-{
-    string_set_avl_destroy(&dnssec_algo_set);
-}
-
-
-
 ya_result
-get_value_from_casename(const value_name_table *table, const char *name, u32 *out_value)
+value_name_table_get_value_from_casename(const value_name_table *table, const char *name, u32 *out_value)
 {
     while(table->data != NULL)
     {
@@ -787,7 +1115,160 @@ get_value_from_casename(const value_name_table *table, const char *name, u32 *ou
     return UNKNOWN_NAME;
 }
 
+ya_result
+value_name_table_get_name_from_value(const value_name_table *table, u32 value, const char** out_name)
+{
+    while(table->data != NULL)
+    {
+        if(table->id == value)
+        {
+            *out_name = table->data;
+            return SUCCESS;
+        }
+
+        table++;
+    }
+
+    return INVALID_ARGUMENT_ERROR;
+}
+
+/*
+ * SOA
+ */
+
+ya_result
+rr_soa_get_serial(const u8* rdata, u16 rdata_size, u32* out_serial)
+{
+    s32 soa_size = rdata_size;
+
+    const u8* soa_start = rdata;
+
+    u32 len = dnsname_len(soa_start);
+
+    soa_size -= len;
+
+    if(soa_size <= 0)
+    {
+        return INCORRECT_RDATA;
+    }
+
+    soa_start += len;
+
+    len = dnsname_len(soa_start);
+    soa_size -= len;
+
+    if(soa_size != 5 * 4) /* Only the 5 32 bits (should) remain */
+    {
+        return INCORRECT_RDATA;
+    }
+
+    soa_start += len;
+
+    if(out_serial != NULL)
+    {
+        *out_serial = ntohl(GET_U32_AT(*soa_start));
+    }
+
+    return SUCCESS;
+}
+
+ya_result
+rr_soa_increase_serial(u8* rdata, u16 rdata_size, u32 increment)
+{
+    s32 soa_size = rdata_size;
+
+    u8* soa_start = rdata;
+
+    u32 len = dnsname_len(soa_start);
+
+    soa_size -= len;
+
+    if(soa_size <= 0)
+    {
+        return INCORRECT_RDATA;
+    }
+
+    soa_start += len;
+
+    len = dnsname_len(soa_start);
+    soa_size -= len;
+
+    if(soa_size != 5 * 4) /* Only the 5 32 bits (should) remain */
+    {
+        return INCORRECT_RDATA;
+    }
+
+    soa_start += len;
+
+    SET_U32_AT(*soa_start, htonl(ntohl(GET_U32_AT(*soa_start)) + increment));
+
+    return SUCCESS;
+}
+
+ya_result
+rr_soa_set_serial(u8* rdata, u16 rdata_size, u32 serial)
+{
+    s32 soa_size = rdata_size;
+
+    u8* soa_start = rdata;
+
+    u32 len = dnsname_len(soa_start);
+
+    soa_size -= len;
+
+    if(soa_size <= 0)
+    {
+        return INCORRECT_RDATA;
+    }
+
+    soa_start += len;
+
+    len = dnsname_len(soa_start);
+    soa_size -= len;
+
+    if(soa_size != 5 * 4) /* Only the 5 32 bits (should) remain */
+    {
+        return INCORRECT_RDATA;
+    }
+
+    soa_start += len;
+
+    SET_U32_AT(*soa_start, htonl(serial));
+
+    return SUCCESS;
+}
+
+ya_result
+rr_soa_get_minimumttl(const u8* rdata, u16 rdata_size, s32 *out_minimum_ttl)
+{
+    s32 soa_size = rdata_size;
+
+    const u8* soa_start = rdata;
+
+    u32 len = dnsname_len(soa_start);
+
+    soa_size -= len;
+
+    if(soa_size <= 0)
+    {
+        return INCORRECT_RDATA;
+    }
+
+    soa_start += len;
+
+    len = dnsname_len(soa_start);
+    soa_size -= len;
+
+    if(soa_size != 5 * 4) /* Only the 5 32 bits (should) remain */
+    {
+        return INCORRECT_RDATA;
+    }
+
+    soa_start += len + 16;
+
+    *out_minimum_ttl = ntohl(GET_U32_AT(*soa_start));
+
+    return SUCCESS;
+}
+
 /** @} */
-
-/*----------------------------------------------------------------------------*/
-
