@@ -1,36 +1,37 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
 /** @defgroup 
  *  @ingroup 
  *  @brief 
@@ -54,15 +55,22 @@
 #include "dnscore/identity.h"
 
 #include "dnscore/server-setup.h"
+#include "dnscore/mt_output_stream.h"
+
+#include "dnscore/process.h"
+#include "dnscore/signals.h"
 
 #include <fcntl.h>
-#include <signal.h>
+#ifndef WIN32
 #include <sys/resource.h>
+#endif
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
-void stdtream_detach_fd_and_close();
+#if DNSCORE_HAS_LOG_THREAD_TAG
+void thread_tag_log_tags();
+#endif
 
 /*------------------------------------------------------------------------------
  * GLOBAL VARIABLES */
@@ -105,12 +113,12 @@ ttylog_err(const char *format, ...)
  *  @return OK
  *  @return Otherwise log_quit will stop the program
  */
-void
-server_setup_daemon_go()
+int
+server_setup_daemon_go_ex(bool parent_exit, bool child_logger_ownership)
 {
+#ifndef WIN32
     mode_t                                                         mask = 0;
     pid_t                                                               pid;
-
     struct sigaction                                                     sa;
 
     /*    ------------------------------------------------------------    */
@@ -142,38 +150,68 @@ server_setup_daemon_go()
     {
         osformatln(termerr, "daemonize: unable to stop all thread pools: %r", error_code);
         flusherr();
-        
+
         exit(EXIT_FAILURE);
     }
+
+#if DNSCORE_HAS_LOG_THREAD_TAG
+    thread_tag_log_tags();
+#endif
     
     logger_flush();
     logger_stop();
 
     /* Clear file creation mask */
     umask(mask);
+
+#if DEBUG
+    debug_println("releasing ownership of the logger");
+    flushout();
+#endif
+
+    logger_release_ownership();
     
     /* Become a session leader to lose controlling TTYs */
     if((pid = fork()) < 0)
     {
-        osformatln(termerr, "cannot fork: %r", ERRNO_ERROR);
+        osformatln(termerr, "[%i] cannot fork: %r", getpid(), ERRNO_ERROR);
         flusherr();
         
         exit(EXIT_FAILURE);
     }
 
+    // if the logger was running, disconnect ownership from the parent and give it to the child
+
     if(pid != 0) /* parent */
     {
-#ifdef DEBUG
-        formatln("first level parent done");
+#if DEBUG
+        formatln("[%i] first level parent done", getpid());
         flushout();
 #endif
-        
-        exit(EXIT_SUCCESS);
+        if(parent_exit)
+        {
+            exit(EXIT_SUCCESS);
+        }
+
+        // else re-enable everything and return
+
+        logger_start();
+
+#if DNSCORE_HAS_LOG_THREAD_TAG
+        thread_tag_log_tags();
+#endif
+        dnscore_reset_timer();
+        thread_pool_start_all();
+        service_start_all();
+        logger_flush();
+
+        return pid;
     }
 
     /* Set program in new session */
     setsid();
 
+#ifndef WIN32
     /* Ensure future opens won't allocate controlling TTYs */
     sa.sa_handler = SIG_IGN;
     sigemptyset(&sa.sa_mask);
@@ -186,6 +224,7 @@ server_setup_daemon_go()
         
         exit(EXIT_FAILURE);
     }
+#endif
 
     /* Stevens way of detaching the program from the parent process,
      * forking twice
@@ -200,38 +239,36 @@ server_setup_daemon_go()
 
     if(pid != 0) /* parent */
     {
-#ifdef DEBUG
-        println("second level parent done");
+#if DEBUG
+        formatln("[%i] second level parent done", getpid());
         flushout();
 #endif
         exit(EXIT_SUCCESS);
     }
-    
-#ifdef DEBUG
-    println("detaching from console");
+
+#if DEBUG
+    debug_osformatln(termout, "taking ownership of the logger (%i)", getpid());
     flushout();
 #endif
-    
+
+    g_pid = getpid();
+
+    if(child_logger_ownership)
+    {
+        logger_take_ownership(getpid());
+    }
+
     logger_start();
-    
+
+#if DNSCORE_HAS_LOG_THREAD_TAG
+    thread_tag_log_tags();
+#endif
+
     log_debug("daemonize: start timer");
     
     dnscore_reset_timer();
-    
-    log_debug("daemonize: start thread pools");
-    
-    thread_pool_start_all();
-    
-    log_debug("daemonize: start services");
-    
-    service_start_all();
-    
-    /* Change the current working directory to the root so
-     * we won't prevent file systems from being unmounted.
-     */
-    
-#ifdef DEBUG
-    
+
+#if DEBUG
     // It has been asked to fix the "tmpfile vulnerability" on DEBUG builds.
     // Although this mode is meant for us (Gery & I), we can imagine that
     // someone else could have some interest in it too.
@@ -240,8 +277,8 @@ server_setup_daemon_go()
     // Later, the "server" part will also be modifiable by the caller.
 
     char output_file[PATH_MAX];
-    snformat(output_file, sizeof(output_file), "/tmp/server-%013x-%05x.std", timeus(), getpid());
-    formatln("redirecting all to '%s'\n", output_file);
+    snformat(output_file, sizeof(output_file), "/tmp/server-%013x-%05x.std.txt", timeus(), getpid());
+    debug_osformatln(termout,"redirecting all to '%s'\n", output_file);
     int file_flags = O_RDWR|O_CREAT|O_EXCL; // ensure no overwrite
 #else
     const char *output_file  = "/dev/null";
@@ -249,18 +286,46 @@ server_setup_daemon_go()
 #endif
 
     /* Attach file descriptors 0, 1, and 2 to /dev/null */
-    
+
     flushout();
     flusherr();
 
-    int tmpfd; 
+    int tmpfd;
     if((tmpfd = open_create_ex(output_file, file_flags, 0660)) < 0)
     {
-        log_err("stdin: %s '%s'", strerror(errno), output_file);
+        fprintf(stderr, "couldn't get file for redirection: %s '%s'\n", strerror(errno), output_file);
+        fflush(NULL);
         exit(EXIT_FAILURE);
     }
 
-    for(int i = 0; i <= 2; i++)
+    int tmpfdin;
+
+#if DEBUG
+    if((tmpfdin = open_create_ex("/dev/null", O_WRONLY, 0440)) < 0)
+    {
+        fprintf(stderr, "couldn't get file for redirection: %s '%s'\n", strerror(errno), output_file);
+        fflush(NULL);
+        exit(EXIT_FAILURE);
+    }
+#else
+    tmpfdin = tmpfd;
+#endif
+
+    {
+        int fd;
+        if((fd = dup2_ex(tmpfdin, 0)) < 0)
+        {
+            log_err("dup2 failed from %i to %i: %s", tmpfdin, 0, strerror(errno));
+            exit(EXIT_FAILURE);
+        }
+        if(fd != 0)
+        {
+            log_err("expected fd %d instead of %d", 0, fd);
+            exit(EXIT_FAILURE);
+        }
+    }
+
+    for(int i = 1; i <= 2; i++)
     {
         int fd;
         if((fd = dup2_ex(tmpfd, i)) < 0)
@@ -274,22 +339,60 @@ server_setup_daemon_go()
             exit(EXIT_FAILURE);
         }
     }
+
+#if DEBUG
+    debug_osformatln(termout, "detaching from console");
+    flushout();
+#endif
+
+
+
+    log_debug("daemonize: start thread pools");
     
-    stdtream_detach_fd_and_close();
+    thread_pool_start_all();
     
-    logger_output_stream_open(&__termout__, g_system_logger, LOG_INFO, 512);
-    logger_output_stream_open(&__termerr__, g_system_logger, LOG_ERR, 512);
+    log_debug("daemonize: start services");
     
+    service_start_all();
+    
+    /* Change the current working directory to the root so
+     * we won't prevent file systems from being unmounted.
+     */
+
+#if DEBUG
+    debug_println("daemonized");
+    flushout();
+#endif
+
     log_info("daemonized");
     
     logger_flush();
+
+    return pid;
+#else
+    return 0;
+#endif
 }
 
+void
+server_setup_daemon_go()
+{
+    server_setup_daemon_go_ex(TRUE, TRUE);
+}
+
+/// @NOTE 20201105 edf -- pid_file_pathp must be a char** because of managed paths, it cannot be a const char** either
+
 ya_result
-server_setup_env(pid_t *pid, char **pid_file_pathp, uid_t uid, gid_t gid, u32 setup_flags)
+server_setup_env(pid_t *pid, char ** pid_file_pathp, uid_t uid, gid_t gid, u32 setup_flags)
 {
     ya_result                                    return_code;
 
+    if((pid == NULL) || (pid_file_pathp == NULL) || (*pid_file_pathp == NULL))
+    {
+        return UNEXPECTED_NULL_ARGUMENT_ERROR;
+    }
+
+#ifndef WIN32
     if(setup_flags & SETUP_CORE_LIMITS)
     {
         struct rlimit core_limits = {RLIM_INFINITY, RLIM_INFINITY};
@@ -298,14 +401,14 @@ server_setup_env(pid_t *pid, char **pid_file_pathp, uid_t uid, gid_t gid, u32 se
         {
             ttylog_err("unable to set core dump limit: %r", ERRNO_ERROR);
         }
-#ifdef DEBUG
+#if DEBUG
         else
         {
             log_debug("core no-limit set");
         }
 #endif
     }
-        
+
     if(setup_flags & SETUP_ROOT_CHANGE)
     {
         log_info("going to jail");
@@ -333,15 +436,17 @@ server_setup_env(pid_t *pid, char **pid_file_pathp, uid_t uid, gid_t gid, u32 se
     if(setup_flags & SETUP_CREATE_PID_FILE)
     {
         /* Setup environment */
+
         if(FAIL(return_code = pid_file_create(pid, *pid_file_pathp, uid, gid)))
         {
-            ttylog_err("unable to create pid file '%s': %r", *pid_file_pathp, return_code);
+            ttylog_err("unable to create pid file '%s': %r", pid_file_pathp, return_code);
 
             return return_code;
         }
     }
-
+#endif
     return SUCCESS;
 }
 
 /** @} */
+
