@@ -1,36 +1,37 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
 /** @defgroup dnscoreerror Error
  *  @ingroup dnscore
  *  @brief
@@ -52,6 +53,8 @@
 #include "dnscore/parser.h"
 #include "dnscore/config_settings.h"
 #include "dnscore/cmdline.h"
+#include "dnscore/zone_reader.h"
+#include "dnscore/zone_reader_text.h"
 
 #define ERRORTBL_TAG 0x4c4254524f525245
 
@@ -76,22 +79,6 @@ dief(ya_result error_code, const char* format, ...)
 
 static u32_node *error_set = NULL;
 
-void
-error_unregister_all()
-{
-    u32_set_avl_iterator iter;
-    
-    u32_set_avl_iterator_init(&error_set, &iter);
-    while(u32_set_avl_iterator_hasnext(&iter))
-    {
-        u32_node *error_node = u32_set_avl_iterator_next_node(&iter);
-        free(error_node->value);
-        error_node->value = NULL;
-    }
-    
-    u32_set_avl_destroy(&error_set);
-}
-
 void
 error_register(ya_result code, const char* text)
 {
@@ -107,7 +94,7 @@ error_register(ya_result code, const char* text)
         return;
     }
     
-    u32_node *error_node = u32_set_avl_insert(&error_set, code);
+    u32_node *error_node = u32_set_insert(&error_set, code);
     
     if(error_node->value == 0)
     {
@@ -115,6 +102,18 @@ error_register(ya_result code, const char* text)
     }
 }
 
+static void
+error_unregister_all_cb(u32_node *node)
+{
+    free(node->value);
+}
+
+void
+error_unregister_all()
+{
+    u32_set_callback_and_destroy(&error_set, error_unregister_all_cb);
+}
+
 static char error_gettext_tmp[64];
 
 /**
@@ -145,7 +144,7 @@ error_gettext(ya_result code)
     
     u32_node *error_node;
     
-    error_node = u32_set_avl_find(&error_set, code);
+    error_node = u32_set_find(&error_set, code);
     if(error_node != NULL)
     {
         return (const char*)error_node->value;
@@ -153,7 +152,7 @@ error_gettext(ya_result code)
     
     u32 error_base = code & 0xffff0000;
 
-    error_node = u32_set_avl_find(&error_set, error_base);
+    error_node = u32_set_find(&error_set, error_base);
     if(error_node != NULL)
     {
         return (const char*)error_node->value;
@@ -186,7 +185,7 @@ error_writetext(output_stream *os, ya_result code)
     if((code & 0xffff0000) == ERRNO_ERROR_BASE)
     {
         code &= 0xffff;
-#ifdef DEBUG
+#if DEBUG
         if(code == EINTR)
         {
             osprint(os, "<EINTR> "); // whoopsie
@@ -200,7 +199,7 @@ error_writetext(output_stream *os, ya_result code)
     
     u32_node *error_node;
     
-    error_node = u32_set_avl_find(&error_set, code);
+    error_node = u32_set_find(&error_set, code);
     if(error_node != NULL)
     {
         osprint(os, (const char*)error_node->value);
@@ -209,7 +208,7 @@ error_writetext(output_stream *os, ya_result code)
     
     u32 error_base = code & 0xffff0000;
 
-    error_node = u32_set_avl_find(&error_set, error_base);
+    error_node = u32_set_find(&error_set, error_base);
     if(error_node != NULL)
     {
         osformatln(os, "%s(%08x)", (const char*)error_node->value, code);
@@ -232,7 +231,7 @@ dnscore_register_errors()
     dnscore_register_errors_done = TRUE;
 
     error_register(SUCCESS, "SUCCESS");
-    error_register(SERVER_ERROR_BASE, "SERVER_ERROR_BASE");
+
     error_register(PARSEB16_ERROR, "PARSEB16_ERROR");
     error_register(PARSEB32_ERROR, "PARSEB32_ERROR");
     error_register(PARSEB32H_ERROR, "PARSEB32H_ERROR");
@@ -240,6 +239,15 @@ dnscore_register_errors()
     error_register(PARSEINT_ERROR, "PARSEINT_ERROR");
     error_register(PARSEDATE_ERROR, "PARSEDATE_ERROR");
     error_register(PARSEIP_ERROR, "PARSEIP_ERROR");
+
+    error_register(CIRCULAR_FILE_FULL, "CIRCULAR_FILE_FULL");
+    error_register(CIRCULAR_FILE_SHORT, "CIRCULAR_FILE_SHORT");
+    error_register(CIRCULAR_FILE_END, "CIRCULAR_FILE_FULL");
+    error_register(CIRCULAR_FILE_LIMIT_EXCEEDED, "CIRCULAR_FILE_LIMIT_EXCEEDED");
+
+    error_register(DATA_FORMAT_ERROR, "DATA_FORMAT_ERROR");
+
+    error_register(LOCK_FAILED, "LOCK_FAILED");
     
     error_register(TCP_RATE_TOO_SLOW, "TCP_RATE_TOO_SLOW");
 
@@ -265,6 +273,8 @@ dnscore_register_errors()
     error_register(CONFIG_KEY_UNKNOWN, "CONFIG_KEY_UNKNOWN");
     error_register(CONFIG_KEY_PARSE_ERROR, "CONFIG_KEY_PARSE_ERROR");
     error_register(CONFIG_SECTION_ERROR, "CONFIG_SECTION_ERROR");
+    error_register(CONFIG_IS_BUSY, "CONFIG_IS_BUSY");
+    error_register(CONFIG_FILE_NOT_FOUND, "CONFIG_FILE_NOT_FOUND");
 
     error_register(LOGGER_INITIALISATION_ERROR, "LOGGER_INITIALISATION_ERROR");
     error_register(COMMAND_ARGUMENT_EXPECTED, "COMMAND_ARGUMENT_EXPECTED");
@@ -279,10 +289,13 @@ dnscore_register_errors()
     error_register(INVALID_PATH, "INVALID_PATH");
     error_register(PID_LOCKED, "PID_LOCKED");
 
-
     error_register(UNABLE_TO_COMPLETE_FULL_READ, "UNABLE_TO_COMPLETE_FULL_READ");
     error_register(UNEXPECTED_EOF, "UNEXPECTED_EOF");
     error_register(UNSUPPORTED_TYPE, "UNSUPPORTED_TYPE");
+    error_register(UNSUPPORTED_CLASS, "UNSUPPORTED_CLASS");
+
+    error_register(CANNOT_OPEN_FILE, "CANNOT_OPEN_FILE");
+
     error_register(UNKNOWN_NAME, "UNKNOWN_NAME");
     error_register(BIGGER_THAN_PATH_MAX, "BIGGER_THAN_PATH_MAX");
     error_register(UNABLE_TO_COMPLETE_FULL_WRITE, "UNABLE_TO_COMPLETE_FULL_WRITE");
@@ -311,9 +324,14 @@ dnscore_register_errors()
     error_register(CHARON_ERROR_UNKNOWN_ID, "CHARON_ERROR_UNKNOWN_ID");
     error_register(CHARON_ERROR_EXPECTED_MAGIC_HEAD, "CHARON_ERROR_EXPECTED_MAGIC_HEAD");
     error_register(CHARON_ERROR_INVALID_HEAD, "CHARON_ERROR_INVALID_HEAD");
+    error_register(CHARON_ERROR_INVALID_TAIL, "CHARON_ERROR_INVALID_TAIL");
     error_register(CHARON_ERROR_INVALID_COMMAND, "CHARON_ERROR_INVALID_COMMAND");
     error_register(CHARON_ERROR_COMMAND_SEQ_MISMATCHED, "CHARON_ERROR_COMMAND_SEQ_MISMATCHED");
 
+    error_register(CHARON_ERROR_UNKNOWN_MAGIC, "CHARON_ERROR_UNKNOWN_MAGIC");
+    error_register(CHARON_ERROR_ALREADY_RUNNING, "CHARON_ERROR_ALREADY_RUNNING");
+    error_register(CHARON_ERROR_ALREADY_STOPPED, "CHARON_ERROR_ALREADY_STOPPED");
+
     error_register(LOGGER_CHANNEL_ALREADY_REGISTERED, "LOGGER_CHANNEL_ALREADY_REGISTERED");
     error_register(LOGGER_CHANNEL_NOT_REGISTERED, "LOGGER_CHANNEL_NOT_REGISTERED");
     error_register(LOGGER_CHANNEL_HAS_LINKS, "LOGGER_CHANNEL_HAS_LINKS");
@@ -324,7 +342,6 @@ dnscore_register_errors()
     error_register(DOMAIN_TOO_LONG, "DOMAIN_TOO_LONG");
     error_register(INCORRECT_IPADDRESS, "INCORRECT_IPADDRESS");
     error_register(INCORRECT_RDATA, "INCORRECT_RDATA");
-    error_register(ZONEFILE_UNSUPPORTED_TYPE, "ZONEFILE_UNSUPPORTED_TYPE");
     error_register(LABEL_TOO_LONG, "LABEL_TOO_LONG");
     error_register(INVALID_CHARSET, "INVALID_CHARSET");
     error_register(ZONEFILE_INVALID_TYPE, "ZONEFILE_INVALID_TYPE");
@@ -341,32 +358,44 @@ dnscore_register_errors()
     error_register(ZONE_ALREADY_UP_TO_DATE, "ZONE_ALREADY_UP_TO_DATE");
     error_register(UNKNOWN_DNS_TYPE, "UNKNOWN_DNS_TYPE");
     error_register(UNKNOWN_DNS_CLASS, "UNKNOWN_DNS_CLASS");
-    error_register(INVALID_MESSAGE, "INVALID_MESSAGE");
+    //error_register(INVALID_MESSAGE, "INVALID_MESSAGE");
     error_register(MESSAGE_HAS_WRONG_ID, "MESSAGE_HAS_WRONG_ID");
     error_register(MESSAGE_IS_NOT_AN_ANSWER, "MESSAGE_IS_NOT_AN_ANSWER");
     error_register(MESSAGE_UNEXPECTED_ANSWER_DOMAIN, "MESSAGE_UNEXPECTED_ANSWER_DOMAIN");
     error_register(MESSAGE_UNEXPECTED_ANSWER_TYPE_CLASS, "MESSAGE_UNEXPECTED_ANSWER_TYPE_CLASS");
     error_register(MESSAGE_CONTENT_OVERFLOW, "MESSAGE_CONTENT_OVERFLOW");
+    error_register(MESSAGE_TRUNCATED, "MESSAGE_TRUNCATED");
     
     error_register(RRSIG_COVERED_TYPE_DIFFERS, "RRSIG_COVERED_TYPE_DIFFERS");
     error_register(RRSIG_OUTPUT_DIGEST_SIZE_TOO_BIG, "RRSIG_OUTPUT_DIGEST_SIZE_TOO_BIG");
     error_register(RRSIG_UNSUPPORTED_COVERED_TYPE, "RRSIG_UNSUPPORTED_COVERED_TYPE");
     error_register(RRSIG_VERIFICATION_FAILED, "RRSIG_VERIFICATION_FAILED");
+
+    error_register(UNKNOWN_DNSSEC_ALGO, "UNKNOWN_DNSSEC_ALGO");
     
     /* DNS */
 
-    error_register(MAKE_DNSMSG_ERROR(RCODE_NOERROR), "NOERROR");
-    error_register(MAKE_DNSMSG_ERROR(RCODE_FORMERR), "FORMERR");
-    error_register(MAKE_DNSMSG_ERROR(RCODE_SERVFAIL), "SERVFAIL");
-    error_register(MAKE_DNSMSG_ERROR(RCODE_NXDOMAIN), "NXDOMAIN");
-    error_register(MAKE_DNSMSG_ERROR(RCODE_NOTIMP), "NOTIMP");
-    error_register(MAKE_DNSMSG_ERROR(RCODE_REFUSED), "REFUSED");
-    error_register(MAKE_DNSMSG_ERROR(RCODE_YXDOMAIN), "YXDOMAIN");
-    error_register(MAKE_DNSMSG_ERROR(RCODE_YXRRSET), "YXRRSET");
-    error_register(MAKE_DNSMSG_ERROR(RCODE_NXRRSET), "NXRRSET");
-    error_register(MAKE_DNSMSG_ERROR(RCODE_NOTAUTH), "NOTAUTH");
-    error_register(MAKE_DNSMSG_ERROR(RCODE_NOTZONE), "NOTZONE");
-    
+    error_register(RCODE_ERROR_CODE(RCODE_NOERROR), "NOERROR");
+    error_register(RCODE_ERROR_CODE(RCODE_FORMERR), "FORMERR");
+    error_register(RCODE_ERROR_CODE(RCODE_SERVFAIL), "SERVFAIL");
+    error_register(RCODE_ERROR_CODE(RCODE_NXDOMAIN), "NXDOMAIN");
+    error_register(RCODE_ERROR_CODE(RCODE_NOTIMP), "NOTIMP");
+    error_register(RCODE_ERROR_CODE(RCODE_REFUSED), "REFUSED");
+    error_register(RCODE_ERROR_CODE(RCODE_YXDOMAIN), "YXDOMAIN");
+    error_register(RCODE_ERROR_CODE(RCODE_YXRRSET), "YXRRSET");
+    error_register(RCODE_ERROR_CODE(RCODE_NXRRSET), "NXRRSET");
+    error_register(RCODE_ERROR_CODE(RCODE_NOTAUTH), "NOTAUTH");
+    error_register(RCODE_ERROR_CODE(RCODE_NOTZONE), "NOTZONE");
+
+    error_register(RCODE_ERROR_CODE(RCODE_BADVERS), "BADVERS");
+    error_register(RCODE_ERROR_CODE(RCODE_BADSIG), "BADSIG");
+    error_register(RCODE_ERROR_CODE(RCODE_BADKEY), "BADKEY");
+    error_register(RCODE_ERROR_CODE(RCODE_BADTIME), "BADTIME");
+    error_register(RCODE_ERROR_CODE(RCODE_BADMODE), "BADMODE");
+    error_register(RCODE_ERROR_CODE(RCODE_BADNAME), "BADNAME");
+    error_register(RCODE_ERROR_CODE(RCODE_BADALG), "BADALG");
+    error_register(RCODE_ERROR_CODE(RCODE_BADTRUNC), "BADTRUNC");
+
     error_register(DNSSEC_ERROR_BASE, "DNSSEC_ERROR_BASE");
 
     error_register(DNSSEC_ERROR_NOENGINE, "DNSSEC_ERROR_NOENGINE");
@@ -375,6 +404,9 @@ dnscore_register_errors()
 
     error_register(DNSSEC_ERROR_UNSUPPORTEDKEYALGORITHM, "DNSSEC_ERROR_UNSUPPORTEDKEYALGORITHM");
     error_register(DNSSEC_ERROR_UNSUPPORTEDDIGESTALGORITHM, "DNSSEC_ERROR_UNSUPPORTEDDIGESTALGORITHM");
+    error_register(DNSSEC_ERROR_FILE_FORMAT_VERSION, "DNSSEC_ERROR_FILE_FORMAT_VERSION");
+    error_register(DNSSEC_ERROR_EXPECTED_CLASS_IN, "DNSSEC_ERROR_EXPECTED_CLASS_IN");
+    error_register(DNSSEC_ERROR_EXPECTED_TYPE_DNSKEY, "DNSSEC_ERROR_EXPECTED_TYPE_DNSKEY");
 
     error_register(DNSSEC_ERROR_DUPLICATEKEY, "DNSSEC_ERROR_DUPLICATEKEY");
     error_register(DNSSEC_ERROR_INCOMPLETEKEY, "DNSSEC_ERROR_INCOMPLETEKEY");
@@ -387,12 +419,22 @@ dnscore_register_errors()
     error_register(DNSSEC_ERROR_KEYISTOOBIG, "DNSSEC_ERROR_KEYISTOOBIG");
     error_register(DNSSEC_ERROR_KEYRING_ALGOTAG_COLLISION, "DNSSEC_ERROR_KEYRING_ALGOTAG_COLLISION");
 
+    error_register(DNSSEC_ERROR_KEYRING_KEY_IS_NOT_PRIVATE,"DNSSEC_ERROR_KEYRING_KEY_IS_NOT_PRIVATE");
+
+    error_register(DNSSEC_ERROR_KEY_GENERATION_FAILED, "DNSSEC_ERROR_KEY_GENERATION_FAILED");
+    error_register(DNSSEC_ERROR_NO_KEY_FOR_DOMAIN, "DNSSEC_ERROR_NO_KEY_FOR_DOMAIN");
+
+    error_register(DNSSEC_ERROR_CANNOT_WRITE_NEW_FILE, "DNSSEC_ERROR_CANNOT_WRITE_NEW_FILE");
+    error_register(DNSSEC_ERROR_FIELD_NOT_HANDLED, "DNSSEC_ERROR_FIELD_NOT_HANDLED");
+    error_register(DNSSEC_ERROR_CANNOT_READ_KEY_FROM_RDATA, "DNSSEC_ERROR_CANNOT_READ_KEY_FROM_RDATA");
+
     error_register(DNSSEC_ERROR_RSASIGNATUREFAILED, "DNSSEC_ERROR_RSASIGNATUREFAILED");
     error_register(DNSSEC_ERROR_DSASIGNATUREFAILED, "DNSSEC_ERROR_DSASIGNATUREFAILED");
 
     error_register(DNSSEC_ERROR_NSEC3_INVALIDZONESTATE, "DNSSEC_ERROR_NSEC3_INVALIDZONESTATE");
     error_register(DNSSEC_ERROR_NSEC3_LABELTODIGESTFAILED, "DNSSEC_ERROR_NSEC3_LABELTODIGESTFAILED");
     error_register(DNSSEC_ERROR_NSEC3_DIGESTORIGINOVERFLOW, "DNSSEC_ERROR_NSEC3_DIGESTORIGINOVERFLOW");
+    error_register(DNSSEC_ERROR_NSEC3_LABELNOTFOUND, "DNSSEC_ERROR_NSEC3_LABELNOTFOUND");
     
     error_register(DNSSEC_ERROR_NSEC_INVALIDZONESTATE, "DNSSEC_ERROR_NSEC_INVALIDZONESTATE");
 
@@ -406,12 +448,15 @@ dnscore_register_errors()
     error_register(ZALLOC_ERROR_MMAPFAILED, "ZALLOC_ERROR_MMAPFAILED");
     error_register(ZALLOC_ERROR_OUTOFMEMORY, "ZALLOC_ERROR_OUTOFMEMORY");
     
+    error_register(ZONEREAD_ERROR_BASE,"ZONEREAD_ERROR_BASE");
+    error_register(ZRE_AXFR_FILE_NOT_FOUND,"ZRE_AXFR_FILE_NOT_FOUND");
+    error_register(ZRE_NO_VALID_FILE_FOUND,"ZRE_NO_VALID_FILE_FOUND");
+        
+    zone_reader_text_init_error_codes();
+    
     parser_init_error_codes();
     config_init_error_codes();
     cmdline_init_error_codes();
 }
 
 /** @} */
-
-/*----------------------------------------------------------------------------*/
-
