@@ -1,36 +1,37 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
 /** @defgroup format C-string formatting
  *  @ingroup dnscore
  *  @brief
@@ -69,7 +70,7 @@ static const char dateseparator[1] = {'-'};
 static const char timeseparator[1] = {':'};
 static const char fracseparator[1] = {'.'};
 static const char datetimeseparator[1] = {' '};
-static const char utcsuffix[1] = {'U'};
+static const char utcsuffix[1] = {'Z'};
 
 /** 
  *  dtus
@@ -227,6 +228,46 @@ static format_handler_descriptor datetime_format_handler_descriptor =
     datetime_format_handler_method
 };
 
+/**
+ *  dts
+ *
+ *  @note 64 bits epoch written up to the s
+ */
+
+void
+localdatetime_format_handler_method(const void *restrict val, output_stream *stream, s32 padding, char pad_char, bool left_justified, void * restrict reserved_for_method_parameters)
+{
+    (void)padding;
+    (void)pad_char;
+    (void)left_justified;
+    (void)reserved_for_method_parameters;
+    u64 epoch_us = (u64)(intptr)val;
+    time_t epoch = (time_t)(epoch_us / 1000000);
+
+    struct tm t;
+
+    gmtime_r(&epoch, &t);
+
+    format_dec_u64(t.tm_year + 1900, stream, 4, '0', FALSE);
+    output_stream_write(stream, dateseparator, 1);
+    format_dec_u64(t.tm_mon + 1    , stream, 2, '0', FALSE);
+    output_stream_write(stream, dateseparator, 1);
+    format_dec_u64(t.tm_mday       , stream, 2, '0', FALSE);
+    output_stream_write(stream, datetimeseparator, 1);
+    format_dec_u64(t.tm_hour       , stream, 2, '0', FALSE);
+    output_stream_write(stream, timeseparator, 1);
+    format_dec_u64(t.tm_min        , stream, 2, '0', FALSE);
+    output_stream_write(stream, timeseparator, 1);
+    format_dec_u64(t.tm_sec        , stream, 2, '0', FALSE);
+}
+
+static format_handler_descriptor localdatetime_format_handler_descriptor =
+{
+    "ldts",
+    4,
+    localdatetime_format_handler_method
+};
+
 /** 
  *  date
  *
@@ -431,6 +472,7 @@ packedepoch_format_handler_method(const void *restrict val, output_stream *strea
     format_dec_u64(t.tm_hour       , stream, 2, '0', FALSE);
     format_dec_u64(t.tm_min        , stream, 2, '0', FALSE);
     format_dec_u64(t.tm_sec        , stream, 2, '0', FALSE);
+    // DO NOT ADD A SUFFIX HERE
 }
 
 static format_handler_descriptor packedepoch_format_handler_descriptor =
@@ -461,7 +503,8 @@ timeformat_class_init()
     format_registerclass(&datetime_format_handler_descriptor);
     format_registerclass(&date_format_handler_descriptor);
     format_registerclass(&time_format_handler_descriptor);
-    
+    format_registerclass(&localdatetime_format_handler_descriptor);
+
     // 32 bits
     
     format_registerclass(&epoch_format_handler_descriptor);
@@ -470,6 +513,3 @@ timeformat_class_init()
 }
 
 /** @} */
-
-/*----------------------------------------------------------------------------*/
-
