@@ -1,36 +1,37 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
 /** @defgroup ### #######
  *  @ingroup dnscore
  *  @brief
@@ -58,6 +59,10 @@
 #include "dnscore/fdtools.h"
 #include "dnscore/pipe_stream.h"
 #include "dnscore/message.h"
+#include "dnscore/pool.h"
+#include "dnscore/random.h"
+#include "dnscore/thread_pool.h"
+#include "dnscore/tcp_io_stream.h"
 
 /* it depends if host is DARWIN or LINUX */
 #ifdef HAVE_SYS_SYSLIMITS_H
@@ -76,16 +81,19 @@ typedef struct xfr_input_stream_data xfr_input_stream_data;
 
 #define XFRISDTA_TAG 0x4154445349524658
 #define XFRISSOA_TAG 0x414f535349524658
+#define XFRPOOL_TAG 0x4c4f4f50524658
 
 struct xfr_input_stream_data
 {
     output_stream pipe_stream_output;
     input_stream pipe_stream_input;
     input_stream source_stream;
+    output_stream source_output_stream;
     packet_unpack_reader_data reader;
     
     message_data *message;
     const u8 *origin;
+    u8 *pool;   // 64KB
         
     u8 *first_soa_record;
     u32 first_soa_record_size;
@@ -93,16 +101,48 @@ struct xfr_input_stream_data
     u16 ancount;
     u16 xfr_mode;
     u32 record_index;                   // index of the record in the stream
+
     u32 last_serial;
+    u32 last_refresh;
+    u32 last_retry;
+    u32 last_expire;
+    u32 last_nttl;
+
+    u64 mesg_hdr_mask;
+    u64 mesg_hdr_result;
+    u64 next_hdr_mask;
+    u64 next_hdr_result;
+
     ya_result last_error;
     bool eos;
     bool ixfr_mark;
+    bool owns_message;
+    bool owns_input_stream;
 #if DNSCORE_HAS_TSIG_SUPPORT
     bool last_message_had_tsig;
     bool need_cleanup_tsig;
 #endif
 };
 
+static pool_s xfr_pool;
+static mutex_t xfr_pool_init_mtx = MUTEX_INITIALIZER;
+static bool xfr_pool_initialised = FALSE;
+
+static void *xfr_pool_alloc(void *args)
+{
+    (void)args;
+    void *p;
+    MALLOC_OBJECT_ARRAY(p, u8, 0x1010a, XFRPOOL_TAG);
+    // void *p = malloc(0x1010a);
+    return p;
+}
+
+static void xfr_pool_free(void *ptr, void* args)
+{
+    (void)args;
+    free(ptr);
+}
+
 /**
  * Reads from the (tcp) input stream for an xfr
  * Detects the xfr type
@@ -111,7 +151,7 @@ struct xfr_input_stream_data
  * @return error code
  */
 
-#if HAS_NON_AA_AXFR_SUPPORT
+//#if HAS_NON_AA_AXFR_SUPPORT
 
 /*
  * Non-RFC-compliant masks (allows AA bit not set)
@@ -124,40 +164,40 @@ struct xfr_input_stream_data
  */
 
 #ifdef WORDS_BIGENDIAN
-#define AXFR_MESSAGE_HEADER_MASK    (( (u64) 0 )                                    | \
+#define AXFR_MESSAGE_LENIENT_HEADER_MASK    (( (u64) 0 )                                    | \
                                      (((u64) (QR_BITS  | TC_BITS )) << 40 )| \
                                      (((u64) ( RA_BITS | RCODE_BITS )) << 32 )      | \
                                      ( (u64) 1LL << 16 ))
 
-#define AXFR_MESSAGE_HEADER_RESULT  (( (u64) (QR_BITS ) << 40 )            | \
+#define AXFR_MESSAGE_LENIENT_HEADER_RESULT  (( (u64) (QR_BITS ) << 40 )            | \
                                      ( ((u64) 1LL) << 16 ))
 
-#define AXFR_NEXT_MESSAGE_HEADER_MASK (( (u64) 0LL )                                   | \
+#define AXFR_NEXT_MESSAGE_LENIENT_HEADER_MASK (( (u64) 0LL )                                   | \
                                       (((u64) ( QR_BITS  | TC_BITS )) << 40 )| \
                                       (((u64) ( RCODE_BITS )) << 32 ))
 
 
-#define AXFR_NEXT_MESSAGE_HEADER_RESULT   (((u64) ( QR_BITS  )) << 40 )
+#define AXFR_NEXT_MESSAGE_LENIENT_HEADER_RESULT   (((u64) ( QR_BITS  )) << 40 )
 
 #else
-#define AXFR_MESSAGE_HEADER_MASK     (( (u64) 0LL )                                   | \
+#define AXFR_MESSAGE_LENIENT_HEADER_MASK     (( (u64) 0LL )                                   | \
                                       (((u64) ( QR_BITS  | TC_BITS )) << 16 )| \
                                       (((u64) ( RCODE_BITS )) << 24 )       | \
                                       (((u64) 1LL) << 40 ))
 
-#define AXFR_MESSAGE_HEADER_RESULT   ((((u64) ( QR_BITS  )) << 16 )| \
+#define AXFR_MESSAGE_LENIENT_HEADER_RESULT   ((((u64) ( QR_BITS  )) << 16 )| \
                                       (((u64) 1LL) << 40 ))
 
-#define AXFR_NEXT_MESSAGE_HEADER_MASK     (( (u64) 0LL )                                   | \
+#define AXFR_NEXT_MESSAGE_LENIENT_HEADER_MASK     (( (u64) 0LL )                                   | \
                                       (((u64) ( QR_BITS  | TC_BITS )) << 16 )| \
                                       (((u64) ( RCODE_BITS )) << 24 ))
 
 
-#define AXFR_NEXT_MESSAGE_HEADER_RESULT   (((u64) ( QR_BITS  )) << 16 )
+#define AXFR_NEXT_MESSAGE_LENIENT_HEADER_RESULT   (((u64) ( QR_BITS  )) << 16 )
 
 #endif
 
-#else
+//#else
 
 /*
  * RFC compliant masks (AA bit must be set) 
@@ -197,7 +237,7 @@ struct xfr_input_stream_data
 
 #endif
 
-#endif
+//#endif
 
 /*
  * Reads the content of a message from the reader field in data (packet reader)
@@ -208,10 +248,10 @@ struct xfr_input_stream_data
 static ya_result
 xfr_input_stream_read_packet(xfr_input_stream_data *data)
 {
-    message_data *message = data->message;
+    //message_data *message = data->message;
     packet_unpack_reader_data *reader = &data->reader;
-    u8 *record = &message->pool_buffer[0];
-    u32 record_len;
+    u8 *record = data->pool; // no persistence of content needed
+    s32 record_len;
     ya_result return_value = SUCCESS;
         
 #if DEBUG_XFR_INPUT_STREAM
@@ -240,10 +280,10 @@ xfr_input_stream_read_packet(xfr_input_stream_data *data)
         }
 
 #if DEBUG_XFR_INPUT_STREAM
-        log_debug("xfr_input_stream: #%u %{recordwire}", data->record_index, record);
+        log_debug("xfr_input_stream: <%u %{recordwire}", data->record_index, record);
 #endif
         
-        u8 *ptr = record + dnsname_len(record);
+        const u8 *ptr = record + dnsname_len(record);
 
         u16 rtype = GET_U16_AT(*ptr);
 
@@ -257,7 +297,7 @@ xfr_input_stream_read_packet(xfr_input_stream_data *data)
                 {
                     data->eos = TRUE;
 
-                    return_value = ERROR; // OWNER OF SOA RECORD SHOULD BE ORIGIN (protocol error)
+                    return_value = INVALID_STATE_ERROR; // OWNER OF SOA RECORD SHOULD BE ORIGIN (protocol error)
 
                     return return_value;
                 }
@@ -267,18 +307,26 @@ xfr_input_stream_read_packet(xfr_input_stream_data *data)
                 ptr += dnsname_len(ptr);
                 u32 soa_serial = ntohl(GET_U32_AT(*ptr));
 
-                if(data->xfr_mode == TYPE_ANY)
+                if(data->xfr_mode == TYPE_ANY) // the type of stream has not been decided yet
                 {
                     if(data->record_index == 1)
                     {
-                        /*
-                         * This is an IXFR, the first record is not sent up
-                         */
-
+                        // second record is an SOA: this is an IXFR, the first record is not sent up
+                        
+#if DEBUG_XFR_INPUT_STREAM
+                        log_debug("xfr_input_stream: #%u %{recordwire} ; (IXFR START)", data->record_index, data->first_soa_record);
+#endif
+                         
                         data->xfr_mode = TYPE_IXFR;
                     }
                     else
                     {
+                        // second record is not an SOA: this is an AXFR, the first record is sent up
+                        
+#if DEBUG_XFR_INPUT_STREAM
+                        log_debug("xfr_input_stream: #%u %{recordwire} ; (AXFR START)", data->record_index, record);
+#endif
+                        
                         output_stream_write(&data->pipe_stream_output, data->first_soa_record, data->first_soa_record_size);
                         data->xfr_mode = TYPE_AXFR;
                     }
@@ -337,6 +385,7 @@ xfr_input_stream_read_packet(xfr_input_stream_data *data)
             case TYPE_ANY:
                 return INVALID_PROTOCOL;
             default:
+            {
                 if(data->record_index == 1)
                 {
                     // special case to detect an AXFR returned by an IXFR query
@@ -352,11 +401,13 @@ xfr_input_stream_read_packet(xfr_input_stream_data *data)
                     }
                     else
                     {
-                        return_value = ERROR;
+                        return_value = INVALID_STATE_ERROR; // XFR mode should be "ANY"
                         return return_value;    // invalid status
                     }
                 }
+
                 break;
+            }
         }
         
 #if DEBUG_XFR_INPUT_STREAM
@@ -370,7 +421,7 @@ xfr_input_stream_read_packet(xfr_input_stream_data *data)
             break;
         }
         
-        if(return_value != record_len)
+        if(return_value != (s32)record_len)
         {
             return UNEXPECTED_EOF;
         }
@@ -382,30 +433,28 @@ xfr_input_stream_read_packet(xfr_input_stream_data *data)
 }
 
 static ya_result
-xfr_input_stream_read(input_stream *is, u8 *buffer, u32 len)
+xfr_input_stream_fill(input_stream *is, u32 len)
 {
     xfr_input_stream_data *data = (xfr_input_stream_data*)is->data;
     input_stream *source_stream = &data->source_stream;
-    message_data *message = data->message;
+    message_data *mesg = data->message;
 #if DNSCORE_HAS_TSIG_SUPPORT
-    const tsig_item *tsig = message->tsig.tsig;
+    const tsig_item *tsig = message_tsig_get_key(mesg);
 #endif
-    packet_unpack_reader_data *reader = &data->reader;
+    packet_unpack_reader_data *pr = &data->reader;
     
     if(FAIL(data->last_error))
     {
         return data->last_error;
     }
     
-    ya_result return_value = SUCCESS;
+    ya_result ret = SUCCESS;
     
-    /* while there is not enough bytes on the input */
-    
-    while(pipe_stream_read_available(&data->pipe_stream_input) < len)
+    while(pipe_stream_read_available(&data->pipe_stream_input) < (s32)len)
     {
         /* read the packet and write on the output (so it can be read back on the input) */
         
-        if(FAIL(return_value = xfr_input_stream_read_packet(data)))
+        if(FAIL(ret = xfr_input_stream_read_packet(data)))
         {
             break;
         }
@@ -422,62 +471,62 @@ xfr_input_stream_read(input_stream *is, u8 *buffer, u32 len)
 
         /* next TCP chunk */
         
-#ifdef DEBUG
-        memset(&message->buffer[0], 0xee, sizeof(message->buffer));
+#if DEBUG
+        message_debug_trash_buffer(mesg);
 #endif
         
         u16 tcplen;
         
-        return_value = input_stream_read_nu16(source_stream, &tcplen); /* this is wrong ... */
+        ret = input_stream_read_nu16(source_stream, &tcplen); /* this is wrong ... */
 
-        if(return_value != 2)
+        if(ret != 2)
         {
-#ifdef DEBUG
-            log_debug("xfr_input_stream_read: next message is %ld bytes long", return_value);
+#if DEBUG
+            log_debug("xfr_input_stream_read: next message is %ld bytes long", ret);
 #endif
             break;
         }
 
         if(tcplen == 0)
         {
-            return_value = UNEXPECTED_EOF;
+            ret = UNEXPECTED_EOF;
             break;
         }
 
-        if(FAIL(return_value = input_stream_read_fully(source_stream, message->buffer, tcplen)))
+        if(FAIL(ret = input_stream_read_fully(source_stream, message_get_buffer(mesg), tcplen)))
         {
             break;
         }
-        
+               
+        message_set_size(mesg, ret);
+
 #if DEBUG_XFR_INPUT_STREAM
-        log_memdump(g_system_logger, MSG_DEBUG1, &message->buffer[0], tcplen, 32);
+        LOGGER_EARLY_CULL_PREFIX(MSG_INFO) message_log(MODULE_MSG_HANDLE, MSG_INFO, mesg);
 #endif
-        
-        message->received = return_value;
 
 
-#ifdef DEBUG
-        memset(&message->buffer[tcplen], 0xdd, DNSPACKET_MAX_LENGTH + 1 - tcplen);
+#if DEBUG
+        memset(&message_get_buffer(mesg)[tcplen], 0xdd, DNSPACKET_MAX_LENGTH + 1 - tcplen);
 #endif
         /*
          * Check the headers
          */
 
-        const u64 *h64 = (u64*)message->buffer;
-        const u64 m64 = AXFR_NEXT_MESSAGE_HEADER_MASK;
-        const u64 r64 = AXFR_NEXT_MESSAGE_HEADER_RESULT;
+        const u64 *h64 = (u64*)message_get_buffer(mesg);
+        const u64 m64 = data->next_hdr_mask; // AXFR_NEXT_MESSAGE_HEADER_MASK;
+        const u64 r64 = data->next_hdr_result; // AXFR_NEXT_MESSAGE_HEADER_RESULT;
 
-        if(((*h64&m64) != r64) || (MESSAGE_NS(message->buffer) != 0))
+        if(((*h64&m64) != r64) || (message_get_authority_count_ne(mesg) != 0))
         {
-            u8 code = MESSAGE_RCODE(message->buffer);
+            u8 code = message_get_rcode(mesg);
 
             if(code != 0)
             {
-                return_value = MAKE_DNSMSG_ERROR(code);
+                ret = MAKE_DNSMSG_ERROR(code);
             }
             else
             {
-                return_value = UNPROCESSABLE_MESSAGE;
+                ret = UNPROCESSABLE_MESSAGE;
             }
             
             break;
@@ -495,51 +544,73 @@ xfr_input_stream_read(input_stream *is, u8 *buffer, u32 len)
              *
              */
 
-            message->tsig.tsig = NULL;
+            message_tsig_clear_key(mesg);
 
-            if(FAIL(return_value = tsig_message_extract(message)))
+            if(FAIL(ret = tsig_message_extract(mesg)))
             {
                 break;
             }
             
-            if((return_value == 1) && (message->tsig.tsig != tsig))
+            if((ret == 1) && (message_tsig_get_key(mesg) != tsig))
             {
                 /* This is not the one we started with */
 
                 log_debug("xfr_input_stream: signature key does not match");
 
-                return_value = TSIG_BADSIG;
+                ret = TSIG_BADSIG;
                 break;
             }
 
-            if(FAIL(return_value = tsig_verify_tcp_next_message(message)))
+            if(FAIL(ret = tsig_verify_tcp_next_message(mesg)))
             {
                 break;
             }
         }
 #endif
-        message_header *header = (message_header*)message->buffer;
+        message_header *header = message_get_header(mesg);
         
         data->ancount = ntohs(header->ancount);
 
-        packet_reader_init(reader, message->buffer, message->received);
-        reader->offset = DNS_HEADER_LENGTH;
+        packet_reader_init_from_message_at(pr, mesg, DNS_HEADER_LENGTH);
 
         u16 n = ntohs(header->qdcount);
         
         while(n > 0)
         {
-            if(FAIL(return_value = packet_reader_skip_fqdn(reader)))
+            if(FAIL(ret = packet_reader_skip_fqdn(pr)))
             {
                 break;
             }
 
-            packet_reader_skip(reader, 4);
+            packet_reader_skip(pr, 4);
 
             n--;
         }
     } // for(;;) /* process all TCP chunks */
     
+    return ret;
+}
+
+static ya_result
+xfr_input_stream_read(input_stream *is, void *buffer_, u32 len)
+{
+    xfr_input_stream_data *data = (xfr_input_stream_data*)is->data;
+    message_data *mesg = data->message;
+#if DNSCORE_HAS_TSIG_SUPPORT
+    const tsig_item *tsig = message_tsig_get_key(mesg);
+#endif
+
+    if(FAIL(data->last_error))
+    {
+        return data->last_error;
+    }
+    
+    u8 *buffer = (u8*)buffer_;
+    
+    ya_result return_value = xfr_input_stream_fill(is, len);
+    
+    /* while there is not enough bytes on the input */
+    
     if(ISOK(return_value))
     {
         if((return_value = pipe_stream_read_available(&data->pipe_stream_input)) > 0) // never fails
@@ -549,7 +620,7 @@ xfr_input_stream_read(input_stream *is, u8 *buffer, u32 len)
 #if DNSCORE_HAS_TSIG_SUPPORT
                 if(data->need_cleanup_tsig)
                 {
-                    tsig_verify_tcp_last_message(message);
+                    tsig_verify_tcp_last_message(mesg);
                     data->need_cleanup_tsig = FALSE;
                 }
 #endif
@@ -561,7 +632,7 @@ xfr_input_stream_read(input_stream *is, u8 *buffer, u32 len)
 #if DNSCORE_HAS_TSIG_SUPPORT
             if(tsig != NULL)
             {
-                tsig_verify_tcp_last_message(message);
+                tsig_verify_tcp_last_message(mesg);
                 data->need_cleanup_tsig = FALSE;
 
                 if(!data->last_message_had_tsig)
@@ -584,7 +655,7 @@ xfr_input_stream_read(input_stream *is, u8 *buffer, u32 len)
     {
 #if DNSCORE_HAS_TSIG_SUPPORT
         // cleanup
-        tsig_verify_tcp_last_message(message);
+        tsig_verify_tcp_last_message(mesg);
         data->need_cleanup_tsig = FALSE;
 #endif
     }
@@ -632,18 +703,46 @@ static void
 xfr_input_stream_close(input_stream *is)
 {
     xfr_input_stream_data *data = (xfr_input_stream_data*)is->data;
-    
+
+#if DNSCORE_HAS_TSIG_SUPPORT
     if(data->need_cleanup_tsig)
     {
-        log_err("TSIG has not been cleared");
+        message_clear_hmac(data->message);
+
+        if(ISOK(data->last_error))
+        {
+#if DEBUG
+            log_warn("xfr: %{dnsname}: TSIG has not been cleared (DEBUG)", data->origin);
+#else
+            log_debug("xfr: %{dnsname}: TSIG has not been cleared (%r)", data->origin, data->last_error);
+#endif
+        }
         data->need_cleanup_tsig = FALSE;
     }
+#endif
+    
+#if DEBUG_XFR_INPUT_STREAM
+    log_debug("xfr_input_stream: %{dnsname}: close, last serial is %i //////////////////////////////", data->origin, data->last_serial);
+#endif
+    
+    pool_release(&xfr_pool, data->pool);
     
     output_stream_close(&data->pipe_stream_output);
     input_stream_close(&data->pipe_stream_input);
     free(data->first_soa_record);
     
-#ifdef DEBUG
+    if(data->owns_message)
+    {
+        message_free(data->message);
+    }
+    
+    if(data->owns_input_stream)
+    {
+        input_stream_close(&data->source_stream);
+        output_stream_close(&data->source_output_stream);
+    }
+        
+#if DEBUG
     memset(data, 0xfe, sizeof(xfr_input_stream_data));
 #endif
     
@@ -674,16 +773,25 @@ static const input_stream_vtbl xfr_input_stream_vtbl =
  */
 
 ya_result
-xfr_input_stream_init(input_stream* filtering_stream, const u8 *origin, input_stream *xfr_source_stream, message_data *message, u32 current_serial, xfr_copy_flags flags)
+xfr_input_stream_init(input_stream* filtering_stream, const u8 *origin, input_stream *xfr_source_stream, message_data *mesg, u32 current_serial, xfr_copy_flags flags)
 {
-    yassert(filtering_stream != NULL && origin != NULL && xfr_source_stream != NULL && message != NULL);
+    yassert(filtering_stream != NULL && origin != NULL && xfr_source_stream != NULL && mesg != NULL);
+    
+    mutex_lock(&xfr_pool_init_mtx);
+    if(!xfr_pool_initialised)
+    {
+        xfr_pool_initialised = TRUE;
+        pool_init(&xfr_pool, xfr_pool_alloc, xfr_pool_free, NULL, "xfr stream data pool");
+    }
+    mutex_unlock(&xfr_pool_init_mtx);
     
     input_stream *is = xfr_source_stream;
     
-    packet_unpack_reader_data reader;
+    packet_unpack_reader_data pr;
     u8 *buffer;
     u8 *record;
     u8 *ptr;
+    u8 *pool = NULL;   // 128KB
 #if DNSCORE_HAS_TSIG_SUPPORT
     const tsig_item *tsig;
 #endif
@@ -705,11 +813,15 @@ xfr_input_stream_init(input_stream* filtering_stream, const u8 *origin, input_st
     u8 old_mac[64];
 #endif
     
+#if DEBUG_XFR_INPUT_STREAM
+    log_debug("xfr_input_stream: %{dnsname}: init, current serial is %i //////////////////////////////", origin, current_serial);
+#endif
+    
     /*
      * ensure the stream will be unusable if the initialisation fails
      */
     
-    input_stream_set_void(filtering_stream);
+    input_stream_set_sink(filtering_stream);
     
     /*
      * Start by reading the first packet, and determine if it's an AXFR or an IXFR (for the name)
@@ -746,42 +858,48 @@ xfr_input_stream_init(input_stream* filtering_stream, const u8 *origin, input_st
         
         if(tcplen >= DNS_HEADER_LENGTH)
         {
-            if(ISOK(return_value = input_stream_read_fully(is, message->pool_buffer, DNS_HEADER_LENGTH)))
+            u8 tmp_hdr[DNS_HEADER_LENGTH];
+            
+            if(ISOK(return_value = input_stream_read_fully(is, tmp_hdr, DNS_HEADER_LENGTH)))
             {
-                return_value = MAKE_DNSMSG_ERROR(MESSAGE_RCODE(message->pool_buffer));
+                return_value = MAKE_DNSMSG_ERROR(MESSAGE_RCODE(tmp_hdr));
             }
         }
 
         return return_value;
     }
     
+    pool = pool_alloc(&xfr_pool);
+    
     /* read the whole message */
 
-    buffer = &message->buffer[0];
-    record = &message->pool_buffer[0];
-    
-    assert(sizeof(message->pool_buffer) >= 255 + 10 + 65535);
+    buffer = message_get_buffer(mesg);
+    record = pool; // no persistence required
 
     if(FAIL(return_value = input_stream_read_fully(is, buffer, tcplen)))
     {
+        pool_release(&xfr_pool, pool);
+        
         return return_value;
     }
+
+    message_set_size(mesg, return_value);
     
 #if DEBUG_XFR_INPUT_STREAM
-    log_memdump(g_system_logger, MSG_DEBUG1, &message->buffer[0], tcplen, 32);
+    LOGGER_EARLY_CULL_PREFIX(MSG_INFO) message_log(MODULE_MSG_HANDLE, MSG_INFO, mesg);
 #endif
     
-    message->received = return_value;
-    
     /* check the message makes sense */
 
-    const u64 *h64 = (u64*)buffer;
-    u64 m64 = AXFR_MESSAGE_HEADER_MASK;
-    u64 r64 = AXFR_MESSAGE_HEADER_RESULT;
+    bool axfr_strict_authority = (flags & XFR_LOOSE_AUTHORITY) == 0;
 
-    if(((*h64&m64) != r64) || (MESSAGE_NS(message->buffer) != 0))
+    const u64 *h64 = (u64*)buffer;
+    u64 m64 = axfr_strict_authority ? AXFR_MESSAGE_HEADER_MASK : AXFR_MESSAGE_LENIENT_HEADER_MASK;
+    u64 r64 = axfr_strict_authority ? AXFR_MESSAGE_HEADER_RESULT : AXFR_MESSAGE_LENIENT_HEADER_RESULT;
+
+    if(((*h64&m64) != r64) || (message_get_authority_count_ne(mesg) != 0))
     {
-        u8 code = MESSAGE_RCODE(message->buffer);
+        u8 code = message_get_rcode(mesg);
 
         if(code != 0)
         {
@@ -792,29 +910,32 @@ xfr_input_stream_init(input_stream* filtering_stream, const u8 *origin, input_st
             return_value = UNPROCESSABLE_MESSAGE;
         }
 
-         return return_value;
+        pool_release(&xfr_pool, pool);
+        
+        return return_value;
     }
 
-    //m64 = AXFR_NEXT_MESSAGE_HEADER_MASK;
-    //r64 = AXFR_NEXT_MESSAGE_HEADER_RESULT;
-
-    packet_reader_init(&reader, buffer, tcplen);
-    reader.offset = DNS_HEADER_LENGTH;
-
-    packet_reader_read_fqdn(&reader, record, RDATA_MAX_LENGTH + 1);
+    packet_reader_init_from_message(&pr, mesg);
+    packet_reader_read_fqdn(&pr, record, RDATA_MAX_LENGTH + 1);
 
     if(!dnsname_equals(record, origin))
     {
+        pool_release(&xfr_pool, pool);
+        
         return INVALID_PROTOCOL;
     }
 
-    if(FAIL(return_value = packet_reader_read_u16(&reader, &qtype)))
+    if(FAIL(return_value = packet_reader_read_u16(&pr, &qtype)))
     {
+        pool_release(&xfr_pool, pool);
+        
         return return_value;
     }
     
     if(return_value != 2)
     {
+        pool_release(&xfr_pool, pool);
+        
         return UNEXPECTED_EOF;
     }
 
@@ -829,6 +950,8 @@ xfr_input_stream_init(input_stream* filtering_stream, const u8 *origin, input_st
         {
             if((flags & XFR_ALLOW_AXFR) == 0)
             {
+                pool_release(&xfr_pool, pool);
+                
                 return INVALID_PROTOCOL;
             }
             break;
@@ -837,24 +960,33 @@ xfr_input_stream_init(input_stream* filtering_stream, const u8 *origin, input_st
         {
             if((flags & XFR_ALLOW_IXFR) == 0)
             {
+                pool_release(&xfr_pool, pool);
+                
                 return INVALID_PROTOCOL;
             }
             break;
         }
         default:
         {
+            pool_release(&xfr_pool, pool);
+            
             return INVALID_PROTOCOL;
         }
     }
 
-    if(FAIL(return_value = packet_reader_read_u16(&reader, &qclass)))
+    if(FAIL(return_value = packet_reader_read_u16(&pr, &qclass)))
     {
+        pool_release(&xfr_pool, pool);
+        
         return return_value;
     }
 
     if(qclass != CLASS_IN)
     {
-        /** wrong answer */
+        // wrong answer
+        
+        pool_release(&xfr_pool, pool);
+        
         return INVALID_PROTOCOL;
     }
     
@@ -863,7 +995,7 @@ xfr_input_stream_init(input_stream* filtering_stream, const u8 *origin, input_st
     u16 ancount = ntohs(MESSAGE_AN(buffer));
     
 #if DNSCORE_HAS_TSIG_SUPPORT
-    if((last_message_had_tsig = ((tsig = message->tsig.tsig) != NULL)))
+    if((last_message_had_tsig = ((tsig = message_tsig_get_key(mesg)) != NULL)))
     {
         /* verify the TSIG
          *
@@ -875,15 +1007,17 @@ xfr_input_stream_init(input_stream* filtering_stream, const u8 *origin, input_st
          *
          */
         
-        message->tsig.tsig = NULL;
+        message_tsig_clear_key(mesg);
 
-        old_mac_size = message->tsig.mac_size;
-        memcpy(old_mac, message->tsig.mac, old_mac_size);
+        old_mac_size = message_tsig_mac_get_size(mesg);
+        message_tsig_mac_copy(mesg, old_mac);
 
-        if(FAIL(return_value = tsig_message_extract(message)))
+        if(FAIL(return_value = tsig_message_extract(mesg)))
         {
             log_debug("xfr_input_stream: error extracting the signature");
 
+            pool_release(&xfr_pool, pool);
+            
             return return_value;
         }
 
@@ -891,26 +1025,32 @@ xfr_input_stream_init(input_stream* filtering_stream, const u8 *origin, input_st
         {
             log_debug("xfr_input_stream: no signature when one was requested");
 
+            pool_release(&xfr_pool, pool);
+            
             return TSIG_BADSIG; /* no signature, when one was requested, is a bad signature */
         }
 
-        if(message->tsig.tsig != tsig)
+        if(tsig != message_tsig_get_key(mesg))
         {
             /* This is not the one we started with */
 
             log_debug("xfr_input_stream: signature key does not match");
 
+            pool_release(&xfr_pool, pool);
+            
             return TSIG_BADSIG;
         }
 
-        /// check that the tsig in the message matches the one that was sent
+        /// check that the tsig in the message matches theh one that was sent
 
-        if(FAIL(return_value = tsig_verify_tcp_first_message(message, old_mac, old_mac_size)))
+        if(FAIL(return_value = tsig_verify_tcp_first_message(mesg, old_mac, old_mac_size)))
         {
+            pool_release(&xfr_pool, pool);
+            
             return return_value;
         }
 
-        reader.packet_size = message->received;
+        pr.packet_size = message_get_size(mesg);
         
         need_cleanup_tsig = TRUE;
     }
@@ -922,13 +1062,17 @@ xfr_input_stream_init(input_stream* filtering_stream, const u8 *origin, input_st
      * read the SOA (it MUST be an SOA)
      */
 
-    if(FAIL(record_len = packet_reader_read_record(&reader, record, RDATA_MAX_LENGTH + 1)))
+    if(FAIL(record_len = packet_reader_read_record(&pr, record, RDATA_MAX_LENGTH + 1)))
     {
+        pool_release(&xfr_pool, pool);
+        
         return record_len;
     }
 
     if(!dnsname_equals(record, origin))
     {
+        pool_release(&xfr_pool, pool);
+        
         return INVALID_PROTOCOL;
     }
 
@@ -936,6 +1080,8 @@ xfr_input_stream_init(input_stream* filtering_stream, const u8 *origin, input_st
 
     if(GET_U16_AT(*ptr) != TYPE_SOA)
     {
+        pool_release(&xfr_pool, pool);
+        
         return INVALID_PROTOCOL;
     }
 
@@ -945,6 +1091,8 @@ xfr_input_stream_init(input_stream* filtering_stream, const u8 *origin, input_st
     
     if(rdata_size < 22)
     {
+        pool_release(&xfr_pool, pool);
+        
         return INVALID_PROTOCOL;
     }
 
@@ -956,6 +1104,8 @@ xfr_input_stream_init(input_stream* filtering_stream, const u8 *origin, input_st
 
     if(len >= rdata_size)
     {
+        pool_release(&xfr_pool, pool);
+        
         return INVALID_PROTOCOL;
     }
     rdata_size -= len;
@@ -964,12 +1114,16 @@ xfr_input_stream_init(input_stream* filtering_stream, const u8 *origin, input_st
     len = dnsname_len(ptr);
     if(len >= rdata_size)
     {
+        pool_release(&xfr_pool, pool);
+        
         return INVALID_PROTOCOL;
     }
     rdata_size -= len;
 
     if(rdata_size != 4)
     {
+        pool_release(&xfr_pool, pool);
+        
         return INVALID_PROTOCOL;
     }
 
@@ -978,17 +1132,22 @@ xfr_input_stream_init(input_stream* filtering_stream, const u8 *origin, input_st
     // if the serial of the SOA is the same one as we know, then there is no
     // need to download the zone
     
-    last_serial = ntohl(GET_U32_AT(*ptr));
+    last_serial = ntohl(GET_U32_AT(ptr[0]));
     
     if(last_serial == current_serial)
     {
-        //args->out_loaded_serial = args->current_serial;
-                        
+        pool_release(&xfr_pool, pool);
+        
         return ZONE_ALREADY_UP_TO_DATE;
     }
 
+    u32 last_refresh = ntohl(GET_U32_AT(ptr[4]));
+    u32 last_retry = ntohl(GET_U32_AT(ptr[8]));
+    u32 last_expire = ntohl(GET_U32_AT(ptr[12]));
+    u32 last_nttl = ntohl(GET_U32_AT(ptr[16]));
+
     xfr_input_stream_data *data;    
-    ZALLOC_OR_DIE(xfr_input_stream_data*, data, xfr_input_stream_data, XFRISDTA_TAG);
+    ZALLOC_OBJECT_OR_DIE( data, xfr_input_stream_data, XFRISDTA_TAG);
     ZEROMEMORY(data, sizeof(xfr_input_stream_data));
     
     /*
@@ -1013,20 +1172,46 @@ xfr_input_stream_init(input_stream* filtering_stream, const u8 *origin, input_st
     filtering_stream->vtbl = &xfr_input_stream_vtbl;
     filtering_stream->data = data;
     
-    pipe_stream_init(&data->pipe_stream_output, &data->pipe_stream_input, 65536);
-    MEMCOPY(&data->reader, &reader, sizeof(packet_unpack_reader_data));
+    u32 pipe_buffer_size = 0x10000;
+    
+    pipe_stream_init(&data->pipe_stream_output, &data->pipe_stream_input, pipe_buffer_size);
+    MEMCOPY(&data->reader, &pr, sizeof(packet_unpack_reader_data));
     
     data->origin = origin;
-    data->message = message;
+    data->message = mesg;
+    
+    data->pool = pool;
     
     data->ancount = ancount - 1;
     data->record_index++;
     data->last_serial = last_serial;
+    data->last_refresh = last_refresh;
+    data->last_retry = last_retry;
+    data->last_expire = last_expire;
+    data->last_nttl = last_nttl;
+
+    if(axfr_strict_authority)
+    {
+        data->mesg_hdr_mask = AXFR_MESSAGE_HEADER_MASK;
+        data->mesg_hdr_result = AXFR_MESSAGE_HEADER_RESULT;
+        data->next_hdr_mask = AXFR_NEXT_MESSAGE_HEADER_MASK;
+        data->next_hdr_result = AXFR_NEXT_MESSAGE_HEADER_RESULT;
+    }
+    else
+    {
+        data->mesg_hdr_mask = AXFR_MESSAGE_LENIENT_HEADER_MASK;
+        data->mesg_hdr_result = AXFR_MESSAGE_LENIENT_HEADER_RESULT;
+        data->next_hdr_mask = AXFR_NEXT_MESSAGE_LENIENT_HEADER_MASK;
+        data->next_hdr_result = AXFR_NEXT_MESSAGE_LENIENT_HEADER_RESULT;
+    }
+
     data->xfr_mode = TYPE_ANY;
     data->ixfr_mark = FALSE;
     data->last_message_had_tsig = last_message_had_tsig;
     data->source_stream = *is;
     data->need_cleanup_tsig = need_cleanup_tsig;
+    data->owns_message = FALSE;
+    data->owns_input_stream = FALSE;
     
     /*
      * Then we read all records for all packets
@@ -1042,16 +1227,98 @@ xfr_input_stream_init(input_stream* filtering_stream, const u8 *origin, input_st
      * In order to know what the type is, read the first packet.
      */
     
-    return_value = xfr_input_stream_read_packet(data);
-    
-    if(FAIL(return_value))
+    if(ISOK(return_value = xfr_input_stream_fill(filtering_stream, pipe_buffer_size / 2)))
     {
-        xfr_input_stream_close(filtering_stream);
+        if(ISOK(return_value = xfr_input_stream_read_packet(data)))
+        {
+            return return_value;
+        }
     }
     
+    xfr_input_stream_close(filtering_stream);
+    
     return return_value;
 }
 
+ya_result
+xfr_input_stream_init_with_query(input_stream* filtering_stream, const host_address *server, const u8 *origin, s32 ttl, const u8 *soa_rdata, int soa_rdata_size, xfr_copy_flags flags)
+{
+    input_stream is;
+    output_stream os;
+    random_ctx rndctx = thread_pool_get_random_ctx();
+    message_data *mesg = message_new_instance();
+    ya_result ret;
+    u32 serial;
+    u16 id;
+    
+    if(FAIL(ret = rr_soa_get_serial(soa_rdata, soa_rdata_size, &serial)))
+    {
+        return ret;
+    }
+    
+     id = (u16)random_next(rndctx);
+             
+    message_make_ixfr_query(mesg, id, origin, ttl, soa_rdata_size, soa_rdata);
+    
+#if DNSCORE_HAS_TSIG_SUPPORT
+    if(server->tsig != NULL)
+    {
+        if(FAIL(ret = message_sign_query(mesg, server->tsig)))
+        {
+            message_free(mesg);
+            return ret;
+        }
+    }
+#endif
+
+    /*
+     * connect & send
+     */
+
+    while(FAIL(ret = tcp_input_output_stream_connect_host_address(server, &is, &os, 3)))
+    {
+        if(ret != MAKE_ERRNO_ERROR(EINTR))
+        {
+            message_free(mesg);
+            return ret;
+        }
+    }
+    
+    if(FAIL(ret = message_write_tcp(mesg, &os)))
+    {
+        input_stream_close(&is);
+        output_stream_close(&os);
+
+        message_free(mesg);
+
+        return ret;
+    }
+    
+    output_stream_flush(&os);
+
+    int fd = fd_input_stream_get_filedescriptor(&is);
+
+    tcp_set_sendtimeout(fd, 10, 0);
+    tcp_set_recvtimeout(fd, 10, 0);
+    
+    if(FAIL(xfr_input_stream_init(filtering_stream, origin, &is, mesg, serial, flags)))
+    {
+        input_stream_close(&is);
+        output_stream_close(&os);
+
+        message_free(mesg);
+
+        return ret;
+    }
+    
+    xfr_input_stream_data *data = (xfr_input_stream_data*)filtering_stream->data;
+    data->owns_message = TRUE;
+    data->owns_input_stream = TRUE;
+    data->source_output_stream = os;
+
+    return SUCCESS;
+}
+
 ya_result
 xfr_input_stream_get_type(input_stream *in_xfr_input_stream)
 {
@@ -1066,11 +1333,30 @@ xfr_input_stream_get_origin(input_stream *in_xfr_input_stream)
     return data->origin;
 }
 
-ya_result
+u32
 xfr_input_stream_get_serial(input_stream *in_xfr_input_stream)
 {
     xfr_input_stream_data *data = (xfr_input_stream_data*)in_xfr_input_stream->data;
     return data->last_serial;
 }
 
+u32
+xfr_input_stream_get_refresh(input_stream *in_xfr_input_stream)
+{
+    xfr_input_stream_data *data = (xfr_input_stream_data*)in_xfr_input_stream->data;
+    return data->last_refresh;
+}
+
+void
+xfr_input_stream_finalize()
+{
+    mutex_lock(&xfr_pool_init_mtx);
+    if(xfr_pool_initialised)
+    {
+        pool_finalize(&xfr_pool);
+        xfr_pool_initialised = FALSE;
+    }
+    mutex_unlock(&xfr_pool_init_mtx);
+}
+
 /** @} */
