@@ -1,36 +1,37 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
 /** @defgroup config Database configuration
  *  @ingroup dnsdb
  *  @brief Database configuration
@@ -53,12 +54,6 @@ extern "C"
 #endif
 
 #define DEFAULT_ASSUMED_CPU_COUNT       2
-    
-/**
- * Version of the database.
- */
-
-#define ZDB_VERSION "1.0"
 
 /**
  * Inlines the find operation of the AVLs/BTREEs
@@ -84,12 +79,6 @@ extern "C"
 
 #define ZDB_OPENSSL_SUPPORT 1
 
-/*
- * Required for building the IXFR streams
- */
-
-#define ZDB_CHANGE_FEEDBACK_SUPPORT 0
-
 /*
  * Use the threadpool system instead of the raw threads
  */
@@ -102,6 +91,20 @@ extern "C"
       #define ZDB_OPENSSL_SUPPORT 0
 #endif
 
+/**
+ *
+ * Enables or disables caching.
+ *
+ * 0 => global_resource_record field not needed
+ * 1 => global_resource_record field is the cache
+ *
+ */
+
+#ifndef ZDB_HAS_RRCACHE_ENABLED
+//#error "ZDB_HAS_RRCACHE_ENABLED has not been defined"
+#define ZDB_HAS_RRCACHE_ENABLED 0
+#endif
+
 /**
  * DEBUG: Ensures that the memory allocated by ZALLOC is trashed.
  * This is of course to avoid uninitialized memory issues.
@@ -111,7 +114,7 @@ extern "C"
  * Recommended value: 0
  */
 
-#define ZDB_DEBUG_ZALLOC_TRASHMEMORY 0 /*defined(DEBUG)*/
+#define DNSCORE_DEBUG_ZALLOC_TRASHMEMORY 0 /*DEBUG*/
 
 /**
  * If the number of items in a dictionnary goes beyond this number, the dictionnary
@@ -125,7 +128,7 @@ extern "C"
 
 /**
  * 
- * @todo 20141006 edf -- this is becoming obsolete
+ * @note 20141006 edf -- ZDB_RECORDS_MAX_CLASS set to 1 allows several optimisations
  *
  * Number of classes [1..n] to support.
  * Set to 1 to support only the IN class
@@ -133,7 +136,7 @@ extern "C"
  * Setting this to 1 also enables some code optimizations.
  *
  * Please note that the caller is responsible for checking that (qclass>0)&&(qclass<=ZDB_RECORDS_MAX_CLASS)
- * zdb_query_ex does the check but really should not.
+ * zdb_query_* does the check but really should not.
  * It's faster to check that a qclass is in that range or is not "CHaos" or "HeSiod" on the caller's side.
  * (ie: if ==IN -> query, else if ==CH answer chaos, else answer no match.)
  *
@@ -157,26 +160,6 @@ extern "C"
     
 
 #define ZDB_EXPLICIT_READER_ZONE_LOCK 1
-    
-/**
- *
- * DEBUG: Enables (1) or disable (0) stdout statistics output while loading a zone.
- *
- * Recommended value: 0
- *
- */
-
-#define ZDB_DEBUG_LOADZONE  0
-
-/**
- * DEBUG: try to read the zone file no matter what error occurs.
- *
- * Recommended value: 0
- *
- */
-
-#define ZDB_DEBUG_ZONEFILE_BESTEFFORT 0
-
 
 /**
  * The maximum number of loops allowed with a cname.
