@@ -1,36 +1,37 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
 /** @defgroup nsec3 NSEC3 functions
  *  @ingroup dnsdbdnssec
  *  @brief
@@ -127,9 +128,34 @@ nsec3_free_node(nsec3_zone_item* node)
      * This assert is wrong because this is actually the payload that has just overwritten our node
      * assert(node->rc == 0 && node->sc == 0 && node->label.owners == NULL && node->star_label.owners == NULL & node->type_bit_maps == NULL);
      */
-#ifdef DEBUG
-    yassert((node->rc == 0) && (node->sc == 0));
-#endif
+    if((node->rc | node->sc) != 0)
+    {
+        log_err("NSEC3 node %{digest32h} being deleted has RC,SC={%i,%i}, should be {0, 0}", node->digest, node->rc, node->sc);
+    }
+
+    if(node->rrsig != NULL)
+    {
+        log_warn("NSEC3 node %{digest32h} being deleted is still signed", node->digest, node->rc, node->sc);
+
+        zdb_packed_ttlrdata *rrsig = node->rrsig;
+
+        do
+        {
+            rdata_desc rrsig_record = {TYPE_RRSIG, ZDB_PACKEDRECORD_PTR_RDATASIZE(rrsig), ZDB_PACKEDRECORD_PTR_RDATAPTR(rrsig)};
+            log_debug("NSEC3 node %{digest32h} has %{typerdatadesc}", node->digest, &rrsig_record);
+
+            zdb_packed_ttlrdata *tmp = rrsig;
+            rrsig = rrsig->next;
+            ZDB_RECORD_ZFREE(tmp);
+        }
+        while(rrsig != NULL);
+    }
+
+    ZFREE_ARRAY(node->type_bit_maps, node->type_bit_maps_size);
+
+    node->type_bit_maps = NULL;
+    node->type_bit_maps_size = 0;
+
     u32 node_size = NSEC3_NODE_SIZE(node);
     ZFREE_ARRAY(node, node_size);
 }
@@ -144,6 +170,10 @@ nsec3_free_node(nsec3_zone_item* node)
  * It must be of type REFERENCE_TYPE
  */
 #define AVL_REFERENCE(node) (node)->digest
+
+#define AVL_TERNARYCMP 1
+
+#if !AVL_TERNARYCMP
 /*
  * A macro to compare two references
  * Returns TRUE if and only if the references are equal.
@@ -154,6 +184,9 @@ nsec3_free_node(nsec3_zone_item* node)
  * Returns TRUE if and only if the first one is bigger than the second one.
  */
 #define AVL_ISBIGGER(reference_a,reference_b) (memcmp(&(reference_a)[1],&(reference_b)[1],(reference_a)[0])>0)
+#else
+#define AVL_COMPARE(reference_a,reference_b) (memcmp(&(reference_a)[1],&(reference_b)[1],(reference_a)[0]))
+#endif
 /*
  * Copies the payload of a node
  * It MUST NOT copy the "proprietary" node fields : children, parent, balance
@@ -172,13 +205,13 @@ nsec3_free_node(nsec3_zone_item* node)
 #include <dnscore/avl.c.inc>
 
 AVL_NODE_TYPE*
-AVL_PREFIXED(avl_find_interval_start)(AVL_CONST_TREE_TYPE* root, const AVL_REFERENCE_TYPE obj_hash)
+AVL_PREFIXED(find_interval_start)(AVL_CONST_TREE_TYPE* root, const AVL_REFERENCE_TYPE obj_hash)
 {
     AVL_NODE_TYPE* node = *root;
     AVL_NODE_TYPE* lower_bound = NULL;
     AVL_REFERENCE_TYPE h;
     
-    yassert(node != NULL);
+    //yassert(node != NULL);
 
     /* This is one of the parts I could try to optimize
      * I've checked the assembly, and it sucks ...
@@ -203,7 +236,7 @@ AVL_PREFIXED(avl_find_interval_start)(AVL_CONST_TREE_TYPE* root, const AVL_REFER
          *
          */
 
-#ifdef DEBUG
+#if DEBUG
         if(h[0] != obj_hash[0])
         {
             DIE_MSG("NSEC3 corrupted NSEC3 node");
@@ -233,12 +266,13 @@ AVL_PREFIXED(avl_find_interval_start)(AVL_CONST_TREE_TYPE* root, const AVL_REFER
     if(lower_bound == NULL)
     {
         lower_bound = *root;
-        
-        yassert(lower_bound != NULL);
-        
-        while((node = AVL_CHILD(lower_bound, DIR_RIGHT)) != NULL)
+
+        if(lower_bound != NULL)
         {
-            lower_bound = node;
+            while((node = AVL_CHILD(lower_bound, DIR_RIGHT)) != NULL)
+            {
+                lower_bound = node;
+            }
         }
     }
     
@@ -246,7 +280,7 @@ AVL_PREFIXED(avl_find_interval_start)(AVL_CONST_TREE_TYPE* root, const AVL_REFER
 }
 
 AVL_NODE_TYPE*
-AVL_PREFIXED(avl_find_interval_prev_mod)(AVL_CONST_TREE_TYPE* root, const AVL_REFERENCE_TYPE obj_hash)
+AVL_PREFIXED(find_interval_prev_mod)(AVL_CONST_TREE_TYPE* root, const AVL_REFERENCE_TYPE obj_hash)
 {
     AVL_NODE_TYPE* node = *root;
     AVL_NODE_TYPE* lower_bound = NULL;
@@ -273,7 +307,7 @@ AVL_PREFIXED(avl_find_interval_prev_mod)(AVL_CONST_TREE_TYPE* root, const AVL_RE
          *
          */
 
-#ifdef DEBUG
+#if DEBUG
         if(h[0] != obj_hash[0])
         {
             DIE_MSG("NSEC3 corrupted NSEC3 node");
@@ -285,7 +319,7 @@ AVL_PREFIXED(avl_find_interval_prev_mod)(AVL_CONST_TREE_TYPE* root, const AVL_RE
         /* equals */
         if(cmp == 0)
         {
-            return nsec3_avl_node_mod_prev(node);
+            return nsec3_node_mod_prev(node);
         }
 
         /* bigger */
@@ -306,7 +340,7 @@ AVL_PREFIXED(avl_find_interval_prev_mod)(AVL_CONST_TREE_TYPE* root, const AVL_RE
         
         yassert(lower_bound != NULL);
         
-        while((node = AVL_CHILD(lower_bound, DIR_RIGHT)) != NULL)
+        while((node = AVL_CHILD(lower_bound, DIR_RIGHT)) != NULL) // VS false positive: an assert says this can't happen
         {
             lower_bound = node;
         }
@@ -316,7 +350,7 @@ AVL_PREFIXED(avl_find_interval_prev_mod)(AVL_CONST_TREE_TYPE* root, const AVL_RE
 }
 
 AVL_NODE_TYPE*
-AVL_PREFIXED(avl_find_prev_mod)(AVL_CONST_TREE_TYPE* root, const AVL_REFERENCE_TYPE obj_hash)
+AVL_PREFIXED(find_prev_mod)(AVL_CONST_TREE_TYPE* root, const AVL_REFERENCE_TYPE obj_hash)
 {
     AVL_NODE_TYPE* node = *root;
     AVL_NODE_TYPE* lower_bound = NULL;
@@ -347,7 +381,7 @@ AVL_PREFIXED(avl_find_prev_mod)(AVL_CONST_TREE_TYPE* root, const AVL_REFERENCE_T
          *
          */
 
-#ifdef DEBUG
+#if DEBUG
         if(h[0] != obj_hash[0])
         {
             DIE_MSG("NSEC3 corrupted NSEC3 node");
@@ -361,7 +395,7 @@ AVL_PREFIXED(avl_find_prev_mod)(AVL_CONST_TREE_TYPE* root, const AVL_REFERENCE_T
         {
             // want the prev mod
             
-            return nsec3_avl_node_mod_prev(node);
+            return nsec3_node_mod_prev(node);
         }
 
         /* bigger */
@@ -382,7 +416,7 @@ AVL_PREFIXED(avl_find_prev_mod)(AVL_CONST_TREE_TYPE* root, const AVL_REFERENCE_T
         
         yassert(lower_bound != NULL);
         
-        while((node = AVL_CHILD(lower_bound, DIR_RIGHT)) != NULL)
+        while((node = AVL_CHILD(lower_bound, DIR_RIGHT)) != NULL) // VS false positive: an assert says this can't happen
         {
             lower_bound = node;
         }
@@ -392,6 +426,3 @@ AVL_PREFIXED(avl_find_prev_mod)(AVL_CONST_TREE_TYPE* root, const AVL_REFERENCE_T
 }
 
 /** @} */
-
-/*----------------------------------------------------------------------------*/
-
