@@ -1,36 +1,37 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
 /** @defgroup dnsdbzone Zone related functions
  *  @ingroup dnsdb
  *  @brief Functions used to manipulate a zone
@@ -49,6 +50,7 @@
 #include <dnscore/dnscore.h>
 
 #include <dnscore/logger.h>
+#include <dnsdb/zdb-zone-lock-monitor.h>
 
 #include "dnsdb/zdb.h"
 
@@ -62,10 +64,10 @@
 
 #include "dnsdb/dnsrdata.h"
 
-#if ZDB_HAS_NSEC_SUPPORT != 0
+#if ZDB_HAS_NSEC_SUPPORT
 #include "dnsdb/nsec.h"
 #endif
-#if ZDB_HAS_NSEC3_SUPPORT != 0
+#if ZDB_HAS_NSEC3_SUPPORT
 #include "dnsdb/nsec3.h"
 #endif
 
@@ -74,7 +76,7 @@
 #include <dnsdb/zdb-zone-lock-monitor.h>
 #endif
 
-#ifdef DEBUG
+#if DEBUG
 #define ZONE_MUTEX_LOG 0    // set this to 0 to disable in DEBUG
 #else
 #define ZONE_MUTEX_LOG 0
@@ -88,30 +90,30 @@ extern logger_handle* g_database_logger;
 
 #if DNSCORE_HAS_MUTEX_DEBUG_SUPPORT
 
-static pthread_mutex_t zdb_zone_lock_set_mtx = PTHREAD_MUTEX_INITIALIZER;
+static mutex_t zdb_zone_lock_set_mtx = MUTEX_INITIALIZER;
 static ptr_set zdb_zone_lock_set = PTR_SET_PTR_EMPTY;
 
 void
 zdb_zone_lock_set_add(zdb_zone *zone)
 {
-    pthread_mutex_lock(&zdb_zone_lock_set_mtx);
-    ptr_node *node = ptr_set_avl_insert(&zdb_zone_lock_set, zone);
+    mutex_lock(&zdb_zone_lock_set_mtx);
+    ptr_node *node = ptr_set_insert(&zdb_zone_lock_set, zone);
     node->value = zone;
-    pthread_mutex_unlock(&zdb_zone_lock_set_mtx);
+    mutex_unlock(&zdb_zone_lock_set_mtx);
 }
 
 void
 zdb_zone_lock_set_del(zdb_zone *zone)
 {
-    pthread_mutex_lock(&zdb_zone_lock_set_mtx);
-    ptr_set_avl_delete(&zdb_zone_lock_set, zone);
-    pthread_mutex_unlock(&zdb_zone_lock_set_mtx);
+    mutex_lock(&zdb_zone_lock_set_mtx);
+    ptr_set_delete(&zdb_zone_lock_set, zone);
+    mutex_unlock(&zdb_zone_lock_set_mtx);
 }
 
 static s64 zdb_zone_lock_set_monitor_last_duration = 0;
 static s64 zdb_zone_lock_set_monitor_last_time = 0;
 
-#ifdef DEBUG
+#if DEBUG
 const char* zdb_zone_lock_names[11]=
 {
     "NOBODY",       // 0x00
@@ -133,7 +135,7 @@ const char* zdb_zone_lock_names[11]=
 void
 zdb_zone_lock_set_monitor()
 {
-    u64 now = timeus();
+    s64 now = timeus();
     
     if(now - zdb_zone_lock_set_monitor_last_time < zdb_zone_lock_set_monitor_last_duration)
     {
@@ -142,13 +144,13 @@ zdb_zone_lock_set_monitor()
     
     zdb_zone_lock_set_monitor_last_time = now;
     
-    pthread_mutex_lock(&zdb_zone_lock_set_mtx);
-    ptr_set_avl_iterator iter;
-    ptr_set_avl_iterator_init(&zdb_zone_lock_set, &iter);
+    mutex_lock(&zdb_zone_lock_set_mtx);
+    ptr_set_iterator iter;
+    ptr_set_iterator_init(&zdb_zone_lock_set, &iter);
 
-    while(ptr_set_avl_iterator_hasnext(&iter))
+    while(ptr_set_iterator_hasnext(&iter))
     {
-        ptr_node *node = ptr_set_avl_iterator_next_node(&iter);
+        ptr_node *node = ptr_set_iterator_next_node(&iter);
         zdb_zone *zone = (zdb_zone*)node->key;
         
         u8 owner = zone->lock_owner;
@@ -157,16 +159,16 @@ zdb_zone_lock_set_monitor()
             continue;
         }
         
-        u64 ts = zone->lock_timestamp;
+        s64 ts = zone->lock_timestamp;
         stacktrace trace = zone->lock_trace;
-        pthread_t id = zone->lock_id;
+        thread_t id = zone->lock_id;
         if(ts < now)
         {
             u64 dt = now - ts;
             if(dt > MUTEX_LOCKED_TOO_MUCH_TIME_US)
             {
                 // locked for 5 seconds ... trouble
-#ifndef DEBUG
+#if !DEBUG
                 log_warn("zdb_zone_lock@%p: %{dnsname}: locked by %x for %lluus by %p", zone, zone->origin, owner, dt, (intptr)id);
 #else
                 if(owner <= 10)
@@ -182,8 +184,8 @@ zdb_zone_lock_set_monitor()
             }
         }
     }
-    pthread_mutex_unlock(&zdb_zone_lock_set_mtx);
-    u64 after = timeus();
+    mutex_unlock(&zdb_zone_lock_set_mtx);
+    s64 after = timeus();
     if(after - now > zdb_zone_lock_set_monitor_last_duration)
     {
         zdb_zone_lock_set_monitor_last_duration = after - now;
@@ -231,22 +233,25 @@ zdb_zone_iswritelocked(zdb_zone *zone)
 void
 zdb_zone_lock(zdb_zone *zone, u8 owner)
 {
+
 #if ZONE_MUTEX_LOG
     log_debug7("acquiring lock for zone %{dnsname}@%p for %x", zone->origin, zone, owner);
 #endif
     
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
-    u64 start = timeus();
-#endif
+#if DNSCORE_HAS_MUTEX_DEBUG_SUPPORT
 #if ZDB_HAS_OLD_MUTEX_DEBUG_SUPPORT
     u64 start = timeus();
 #endif
+#endif
+
+#if MUTEX_CONTENTION_MONITOR
+    struct mutex_contention_monitor_s *mcm = mutex_contention_lock_begin(thread_self(), zone, debug_stacktrace_get(), "zdb_zone");
+#endif
 
     mutex_t *mutex = &zone->lock_mutex;
-    
     mutex_lock(mutex);
     
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
     struct zdb_zone_lock_monitor *holder = zdb_zone_lock_monitor_new(zone, owner, 0);
 #endif
     
@@ -270,7 +275,7 @@ zdb_zone_lock(zdb_zone *zone, u8 owner)
             break;
         }
         
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
         zdb_zone_lock_monitor_waits(holder);
 #endif
         
@@ -286,20 +291,28 @@ zdb_zone_lock(zdb_zone *zone, u8 owner)
         cond_wait(&zone->lock_cond, mutex);
 #endif
         
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
         zdb_zone_lock_monitor_resumes(holder);
 #endif
     }
+
+#if ZDB_ZONE_LOCK_HAS_OWNER_ID
+    zone->lock_last_owner_id = thread_self();
+#endif
     
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
     zdb_zone_lock_monitor_locks(holder);
 #endif
     
     mutex_unlock(mutex);
+
+#if MUTEX_CONTENTION_MONITOR
+    mutex_contention_lock_end(mcm);
+#endif
     
 #if ZDB_HAS_OLD_MUTEX_DEBUG_SUPPORT
     zone->lock_trace = debug_stacktrace_get();
-    zone->lock_id = pthread_self();
+    zone->lock_id = thread_self();
     zone->lock_timestamp = timeus();
 
     zdb_zone_lock_set_add(zone);
@@ -313,9 +326,13 @@ zdb_zone_trylock(zdb_zone *zone, u8 owner)
     log_debug7("trying to acquire lock for zone %{dnsname}@%p for %x", zone->origin, zone, owner);
 #endif
 
+#if MUTEX_CONTENTION_MONITOR
+    struct mutex_contention_monitor_s *mcm = mutex_contention_lock_begin(thread_self(), zone, debug_stacktrace_get(), "zdb_zone");
+#endif
+
     mutex_lock(&zone->lock_mutex);
     
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
     struct zdb_zone_lock_monitor *holder = zdb_zone_lock_monitor_new(zone, owner, 0);
 #endif
 
@@ -332,24 +349,31 @@ zdb_zone_trylock(zdb_zone *zone, u8 owner)
         log_debug7("acquired lock for zone %{dnsname}@%p for %x (#%i)", zone->origin, zone, owner, zone->lock_count);
 #endif
         
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
         zdb_zone_lock_monitor_locks(holder);
 #endif
 
+#if ZDB_ZONE_LOCK_HAS_OWNER_ID
+        zone->lock_last_owner_id = thread_self();
+#endif
+
         mutex_unlock(&zone->lock_mutex);
         
 #if ZDB_HAS_OLD_MUTEX_DEBUG_SUPPORT
         zone->lock_trace = debug_stacktrace_get();
-        zone->lock_id = pthread_self();
+        zone->lock_id = thread_self();
         zone->lock_timestamp = timeus();
 
         zdb_zone_lock_set_add(zone);
 #endif
 
+#if MUTEX_CONTENTION_MONITOR
+        mutex_contention_lock_end(mcm);
+#endif
         return TRUE;
     }
     
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
     zdb_zone_lock_monitor_cancels(holder);
 #endif
 
@@ -359,6 +383,10 @@ zdb_zone_trylock(zdb_zone *zone, u8 owner)
     log_debug7("failed to acquire lock for zone %{dnsname}@%p for %x", zone->origin, zone, owner);
 #endif
 
+#if MUTEX_CONTENTION_MONITOR
+    mutex_contention_lock_fail(mcm);
+#endif
+
     return FALSE;
 }
 
@@ -368,6 +396,10 @@ zdb_zone_trylock_wait(zdb_zone *zone, u64 usec, u8 owner)
 #if ZONE_MUTEX_LOG
     log_debug7("trying to acquire lock for %lluus for zone %{dnsname}@%p for %x", usec, zone->origin, zone, owner);
 #endif
+
+#if MUTEX_CONTENTION_MONITOR
+    struct mutex_contention_monitor_s *mcm = mutex_contention_lock_begin(thread_self(), zone, debug_stacktrace_get(), "zdb_zone");
+#endif
     
     u64 start = timeus();
     bool ret = FALSE;
@@ -415,16 +447,34 @@ zdb_zone_trylock_wait(zdb_zone *zone, u64 usec, u8 owner)
             break;
         }
     }
+
+#if ZDB_ZONE_LOCK_HAS_OWNER_ID
+    if(ret)
+    {
+        zone->lock_last_owner_id = thread_self();
+    }
+#endif
     
     mutex_unlock(mutex);
     
 #if DNSCORE_HAS_MUTEX_DEBUG_SUPPORT
     zone->lock_trace = debug_stacktrace_get();
-    zone->lock_id = pthread_self();
+    zone->lock_id = thread_self();
     zone->lock_timestamp = timeus();
 
     zdb_zone_lock_set_add(zone);
 #endif
+
+#if MUTEX_CONTENTION_MONITOR
+    if(ISOK(ret))
+    {
+        mutex_contention_lock_end(mcm);
+    }
+    else
+    {
+        mutex_contention_lock_fail(mcm);
+    }
+#endif
     
     return ret;
 }
@@ -434,20 +484,22 @@ zdb_zone_unlock(zdb_zone *zone, u8 owner)
 {
 #if ZONE_MUTEX_LOG
     log_debug7("releasing lock for zone %{dnsname}@%p by %x (owned by %x)", zone->origin, zone, owner, zone->lock_owner);
+#else
+    (void)owner;
 #endif
 
     mutex_lock(&zone->lock_mutex);
     
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
     struct zdb_zone_lock_monitor *holder = zdb_zone_lock_monitor_get(zone);
 #endif
 
-#ifdef DEBUG
+#if DEBUG
     if(((zone->lock_owner & ZDB_ZONE_MUTEX_UNLOCKMASK_FLAG) != (owner & ZDB_ZONE_MUTEX_UNLOCKMASK_FLAG)) || (zone->lock_count == 0))
     {
-        mutex_unlock(&zone->lock_mutex);
         yassert((zone->lock_owner == ZDB_ZONE_MUTEX_DESTROY) || (zone->lock_owner == (owner & ZDB_ZONE_MUTEX_UNLOCKMASK_FLAG)));
         yassert(zone->lock_count != 0);
+        abort(); // unreachable
     }
 #endif
 
@@ -463,9 +515,17 @@ zdb_zone_unlock(zdb_zone *zone, u8 owner)
         cond_notify(&zone->lock_cond);
     }
     
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
     zdb_zone_lock_monitor_unlocks(holder);
 #endif
+
+#if ZDB_ZONE_LOCK_HAS_OWNER_ID
+    thread_t tid = thread_self();
+    if(zone->lock_last_owner_id == tid)
+    {
+        zone->lock_last_owner_id = 0;
+    }
+#endif
     
 #if ZDB_HAS_OLD_MUTEX_DEBUG_SUPPORT
     zone->lock_trace = NULL;
@@ -476,22 +536,32 @@ zdb_zone_unlock(zdb_zone *zone, u8 owner)
 #endif
     
     mutex_unlock(&zone->lock_mutex);
+
+#if MUTEX_CONTENTION_MONITOR
+    mutex_contention_unlock(thread_self(), zone);
+#endif
 }
 
 void
 zdb_zone_double_lock(zdb_zone *zone, u8 owner, u8 secondary_owner)
 {
+#if MUTEX_CONTENTION_MONITOR
+    struct mutex_contention_monitor_s *mcm = mutex_contention_lock_begin(thread_self(), zone, debug_stacktrace_get(), "zdb_zone");
+#endif
+
 #if ZONE_MUTEX_LOG
     log_debug7("acquiring lock for zone %{dnsname}@%p for %x", zone->origin, zone, owner);
 #endif
 
 #if DNSCORE_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_OLD_MUTEX_DEBUG_SUPPORT
     u64 start = timeus();
+#endif
 #endif
     
     mutex_lock(&zone->lock_mutex);
     
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
     struct zdb_zone_lock_monitor *holder = zdb_zone_lock_monitor_new(zone, owner, secondary_owner);
 #endif
     
@@ -516,6 +586,12 @@ zdb_zone_double_lock(zdb_zone *zone, u8 owner, u8 secondary_owner)
                 zone->lock_owner = owner & ZDB_ZONE_MUTEX_LOCKMASK_FLAG;
                 zone->lock_count++;
                 zone->lock_reserved_owner = secondary_owner & ZDB_ZONE_MUTEX_LOCKMASK_FLAG;
+
+#if ZDB_ZONE_LOCK_HAS_OWNER_ID
+                thread_t tid = thread_self();
+                zone->lock_last_owner_id = tid;
+                zone->lock_last_reserved_owner_id = tid;
+#endif
             
 #if ZONE_MUTEX_LOG
                 log_debug7("acquired lock for zone %{dnsname}@%p for %x (#%i)", zone->origin, zone, owner, zone->lock_count);
@@ -529,7 +605,7 @@ zdb_zone_double_lock(zdb_zone *zone, u8 owner, u8 secondary_owner)
             // the secondary owner is already taken
         }
 
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
         zdb_zone_lock_monitor_waits(holder);
 #endif
         
@@ -546,7 +622,7 @@ zdb_zone_double_lock(zdb_zone *zone, u8 owner, u8 secondary_owner)
 #endif
     }
     
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
     zdb_zone_lock_monitor_locks(holder);
 #endif
     
@@ -554,28 +630,35 @@ zdb_zone_double_lock(zdb_zone *zone, u8 owner, u8 secondary_owner)
     
 #if DNSCORE_HAS_MUTEX_DEBUG_SUPPORT
     zone->lock_trace = debug_stacktrace_get();
-    zone->lock_id = pthread_self();
+    zone->lock_id = thread_self();
     zone->lock_timestamp = timeus();
 
     zdb_zone_lock_set_add(zone);
 #endif
+
+#if MUTEX_CONTENTION_MONITOR
+    mutex_contention_lock_end(mcm);
+#endif
 }
 
 bool
 zdb_zone_try_double_lock(zdb_zone *zone, u8 owner, u8 secondary_owner)
 {
+#if MUTEX_CONTENTION_MONITOR
+    struct mutex_contention_monitor_s *mcm = mutex_contention_lock_begin(thread_self(), zone, debug_stacktrace_get(), "zdb_zone");
+#endif
+
 #if ZONE_MUTEX_LOG
     log_debug7("trying to acquire lock for zone %{dnsname}@%p for %x", zone->origin, zone, owner);
 #endif
-
     mutex_lock(&zone->lock_mutex);
     
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
     struct zdb_zone_lock_monitor *holder = zdb_zone_lock_monitor_new(zone, owner, secondary_owner);
 #endif
 
     u8 so = zone->lock_reserved_owner & ZDB_ZONE_MUTEX_LOCKMASK_FLAG;
-        
+
     if(so == ZDB_ZONE_MUTEX_NOBODY || so == secondary_owner)
     {
         u8 co = zone->lock_owner & ZDB_ZONE_MUTEX_LOCKMASK_FLAG;
@@ -588,11 +671,17 @@ zdb_zone_try_double_lock(zdb_zone *zone, u8 owner, u8 secondary_owner)
             zone->lock_count++;
             zone->lock_reserved_owner = secondary_owner & ZDB_ZONE_MUTEX_LOCKMASK_FLAG;
 
+#if ZDB_ZONE_LOCK_HAS_OWNER_ID
+            thread_t tid = thread_self();
+            zone->lock_last_owner_id = tid;
+            zone->lock_last_reserved_owner_id = tid;
+#endif
+
 #if ZONE_MUTEX_LOG
             log_debug7("acquired lock for zone %{dnsname}@%p for %x (#%i)", zone->origin, zone, owner, zone->lock_count);
 #endif
             
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
             zdb_zone_lock_monitor_locks(holder);
 #endif
 
@@ -600,11 +689,14 @@ zdb_zone_try_double_lock(zdb_zone *zone, u8 owner, u8 secondary_owner)
 
 #if DNSCORE_HAS_MUTEX_DEBUG_SUPPORT
             zone->lock_trace = debug_stacktrace_get();
-            zone->lock_id = pthread_self();
+            zone->lock_id = thread_self();
             zone->lock_timestamp = timeus();
 
             zdb_zone_lock_set_add(zone);
 #endif
+#if MUTEX_CONTENTION_MONITOR
+            mutex_contention_lock_end(mcm);
+#endif
             
             return TRUE;
         }
@@ -616,7 +708,7 @@ zdb_zone_try_double_lock(zdb_zone *zone, u8 owner, u8 secondary_owner)
     }
     */
     
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
     zdb_zone_lock_monitor_cancels(holder);
 #endif
 
@@ -626,6 +718,97 @@ zdb_zone_try_double_lock(zdb_zone *zone, u8 owner, u8 secondary_owner)
     log_debug7("failed to acquire lock for zone %{dnsname}@%p for %x", zone->origin, zone, owner);
 #endif
 
+#if MUTEX_CONTENTION_MONITOR
+    mutex_contention_lock_fail(mcm);
+#endif
+
+    return FALSE;
+}
+
+bool
+zdb_zone_try_double_lock_ex(zdb_zone *zone, u8 owner, u8 secondary_owner, u8 *current_ownerp, u8 *current_reserved_ownerp)
+{
+#if MUTEX_CONTENTION_MONITOR
+    struct mutex_contention_monitor_s *mcm = mutex_contention_lock_begin(thread_self(), zone, debug_stacktrace_get(), "zdb_zone");
+#endif
+
+#if ZONE_MUTEX_LOG
+    log_debug7("trying to acquire lock for zone %{dnsname}@%p for %x", zone->origin, zone, owner);
+#endif
+    mutex_lock(&zone->lock_mutex);
+
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
+    struct zdb_zone_lock_monitor *holder = zdb_zone_lock_monitor_new(zone, owner, secondary_owner);
+#endif
+
+    *current_ownerp = zone->lock_owner;
+    *current_reserved_ownerp = zone->lock_reserved_owner;
+
+    u8 so = zone->lock_reserved_owner & ZDB_ZONE_MUTEX_LOCKMASK_FLAG;
+
+    if(so == ZDB_ZONE_MUTEX_NOBODY || so == secondary_owner)
+    {
+        u8 co = zone->lock_owner & ZDB_ZONE_MUTEX_LOCKMASK_FLAG;
+
+        if(co == ZDB_ZONE_MUTEX_NOBODY || co == owner)
+        {
+            yassert(!SIGNED_VAR_VALUE_IS_MAX(zone->lock_count));
+
+            zone->lock_owner = owner & ZDB_ZONE_MUTEX_LOCKMASK_FLAG;
+            zone->lock_count++;
+            zone->lock_reserved_owner = secondary_owner & ZDB_ZONE_MUTEX_LOCKMASK_FLAG;
+
+#if ZDB_ZONE_LOCK_HAS_OWNER_ID
+            thread_t tid = thread_self();
+            zone->lock_last_owner_id = tid;
+            zone->lock_last_reserved_owner_id = tid;
+#endif
+
+#if ZONE_MUTEX_LOG
+            log_debug7("acquired lock for zone %{dnsname}@%p for %x (#%i)", zone->origin, zone, owner, zone->lock_count);
+#endif
+
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
+            zdb_zone_lock_monitor_locks(holder);
+#endif
+
+            mutex_unlock(&zone->lock_mutex);
+
+#if DNSCORE_HAS_MUTEX_DEBUG_SUPPORT
+            zone->lock_trace = debug_stacktrace_get();
+            zone->lock_id = thread_self();
+            zone->lock_timestamp = timeus();
+
+            zdb_zone_lock_set_add(zone);
+#endif
+#if MUTEX_CONTENTION_MONITOR
+            mutex_contention_lock_end(mcm);
+#endif
+
+            return TRUE;
+        }
+    }
+    /*
+    else
+    {
+        // already double-owned
+    }
+    */
+
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
+    zdb_zone_lock_monitor_cancels(holder);
+#endif
+
+    mutex_unlock(&zone->lock_mutex);
+
+#if ZONE_MUTEX_LOG
+    log_debug7("failed to acquire lock for zone %{dnsname}@%p for %x", zone->origin, zone, owner);
+#endif
+
+#if MUTEX_CONTENTION_MONITOR
+    mutex_contention_lock_fail(mcm);
+#endif
+
     return FALSE;
 }
 
@@ -633,16 +816,32 @@ void
 zdb_zone_double_unlock(zdb_zone *zone, u8 owner, u8 secondary_owner)
 {
 #if ZONE_MUTEX_LOG
-    log_debug7("releasing lock for zone %{dnsname}@%p by %x (owned by %x)", zone->origin, zone, owner, zone->lock_owner);
+    log_debug7("releasing lock for zone %{dnsname}@%p by %x and %x (owned by %x and %x)", zone->origin, zone, owner, secondary_owner, zone->lock_owner, zone->lock_reserved_owner);
+#else
+    (void)owner;
+    (void)secondary_owner;
 #endif
 
     mutex_lock(&zone->lock_mutex);
 
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
     struct zdb_zone_lock_monitor *holder = zdb_zone_lock_monitor_get(zone);
 #endif
     
+#if ZDB_ZONE_LOCK_HAS_OWNER_ID
+    thread_t tid = thread_self();
+    if(zone->lock_last_owner_id == tid)
+    {
+        zone->lock_last_owner_id = 0;
+    }
+    if(zone->lock_last_reserved_owner_id == tid)
+    {
+        zone->lock_last_reserved_owner_id = 0;
+    }
+#endif
 
+    assert(zone->lock_reserved_owner == (secondary_owner & ZDB_ZONE_MUTEX_UNLOCKMASK_FLAG));
+    assert(zone->lock_owner == (owner & ZDB_ZONE_MUTEX_UNLOCKMASK_FLAG));
     
     zone->lock_reserved_owner = ZDB_ZONE_MUTEX_NOBODY;
 
@@ -661,7 +860,7 @@ zdb_zone_double_unlock(zdb_zone *zone, u8 owner, u8 secondary_owner)
         cond_notify(&zone->lock_cond);
     }
     
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
     zdb_zone_lock_monitor_unlocks(holder);
 #endif
     
@@ -674,145 +873,10 @@ zdb_zone_double_unlock(zdb_zone *zone, u8 owner, u8 secondary_owner)
 #endif
     
     mutex_unlock(&zone->lock_mutex);
-}
 
-/**
- * obsolete ?
- * 
- * @param zone
- * @param owner
- * @param secondary_owner
- */
-
-void
-zdb_zone_transfer_lock(zdb_zone *zone, u8 owner, u8 secondary_owner)
-{
-#if ZONE_MUTEX_LOG
-    log_debug7("transferring lock for zone %{dnsname}@%p from %x to %x (owned by %x:%x)", zone->origin, zone, owner, secondary_owner, zone->lock_owner, zone->lock_reserved_owner);
+#if MUTEX_CONTENTION_MONITOR
+    mutex_contention_unlock(thread_self(), zone);
 #endif
-    
-#if DNSCORE_HAS_MUTEX_DEBUG_SUPPORT
-    u64 start = timeus();
-#endif
-
-    mutex_lock(&zone->lock_mutex);
-    
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
-    struct zdb_zone_lock_monitor *holder = zdb_zone_lock_monitor_get(zone);
-#endif
-
-    // wait to be the last one
-    
-    while(zone->lock_count != 1)
-    {
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
-        zdb_zone_lock_monitor_waits(holder);
-#endif
-            
-#if ZDB_HAS_OLD_MUTEX_DEBUG_SUPPORT
-        s64 d = timeus() - start;
-        if(d > MUTEX_WAITED_TOO_MUCH_TIME_US)
-        {
-            log_warn("zdb_zone_transfer_lock(%{dnsname},%x,%x) : waited for %llius already ...", zone->origin, owner, secondary_owner, d);
-            debug_log_stacktrace(MODULE_MSG_HANDLE, MSG_WARNING, "zdb_zone_double_lock:");
-        }
-#endif
-        
-        cond_timedwait(&zone->lock_cond, &zone->lock_mutex, 100);
-        
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
-        zdb_zone_lock_monitor_resumes(holder);
-#endif
-    }
-    
-    zone->lock_owner = secondary_owner & ZDB_ZONE_MUTEX_LOCKMASK_FLAG;
-    zone->lock_reserved_owner = ZDB_ZONE_MUTEX_NOBODY;
-    
-
-#if ZONE_MUTEX_LOG
-    log_debug7("transferred lock for zone %{dnsname}@%p from %x to %x (#%i)", zone->origin, zone, owner, secondary_owner, zone->lock_count);
-#endif
-    
-    if((secondary_owner & ZDB_ZONE_MUTEX_EXCLUSIVE_FLAG) == 0)
-    {
-        cond_notify(&zone->lock_cond);
-    }
-    
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
-    zdb_zone_lock_monitor_exchanges(holder);
-    zdb_zone_lock_monitor_release(holder);
-#endif
-
-    mutex_unlock(&zone->lock_mutex);
-    
-#if ZDB_HAS_OLD_MUTEX_DEBUG_SUPPORT
-    zone->lock_trace = debug_stacktrace_get();
-    zone->lock_id = pthread_self();
-    zone->lock_timestamp = timeus();
-
-    zdb_zone_lock_set_add(zone);
-#endif
-}
-
-/**
- * Obsolete ?
- * 
- * @param zone
- * @param owner
- * @param secondary_owner
- * @return 
- */
-
-bool
-zdb_zone_try_transfer_lock(zdb_zone *zone, u8 owner, u8 secondary_owner)
-{
-#if ZONE_MUTEX_LOG
-    log_debug7("transferring lock for zone %{dnsname}@%p from %x to %x (owned by %x:%x)", zone->origin, zone, owner, secondary_owner, zone->lock_owner, zone->lock_reserved_owner);
-#endif
-    
-    mutex_lock(&zone->lock_mutex);
-    
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
-    struct zdb_zone_lock_monitor *holder = zdb_zone_lock_monitor_new(zone, owner, secondary_owner);
-#endif
-
-    // wait to be the last one
-    
-    if(zone->lock_count == 1)
-    {
-        zone->lock_owner = secondary_owner & ZDB_ZONE_MUTEX_LOCKMASK_FLAG;
-        zone->lock_reserved_owner = ZDB_ZONE_MUTEX_NOBODY;
-        
-        if((secondary_owner & ZDB_ZONE_MUTEX_EXCLUSIVE_FLAG) == 0)
-        {
-            cond_notify(&zone->lock_cond);
-        }
-        
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
-        zdb_zone_lock_monitor_exchanges(holder);
-        zdb_zone_lock_monitor_release(holder);
-#endif
-        
-        mutex_unlock(&zone->lock_mutex);
-        
-#if DNSCORE_HAS_MUTEX_DEBUG_SUPPORT
-        zone->lock_trace = debug_stacktrace_get();
-        zone->lock_id = pthread_self();
-        zone->lock_timestamp = timeus();
-
-        zdb_zone_lock_set_add(zone);
-#endif
-        
-        return TRUE;
-    }
-        
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
-    zdb_zone_lock_monitor_cancels(holder);
-#endif
-    
-    mutex_unlock(&zone->lock_mutex);
-    
-    return FALSE;
 }
 
 void
@@ -823,24 +887,29 @@ zdb_zone_exchange_locks(zdb_zone *zone, u8 owner, u8 secondary_owner)
 #endif
     
 #if DNSCORE_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_OLD_MUTEX_DEBUG_SUPPORT
     u64 start = timeus();
-    struct zdb_zone_lock_monitor *holder = zdb_zone_lock_monitor_new(zone, owner, secondary_owner);
+#endif
 #endif
 
     mutex_lock(&zone->lock_mutex);
+    
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
+    struct zdb_zone_lock_monitor *holder = zdb_zone_lock_monitor_new(zone, owner, secondary_owner);
+#endif
 
-#ifdef DEBUG
+#if DEBUG
     if((zone->lock_owner != (owner & ZDB_ZONE_MUTEX_LOCKMASK_FLAG)) || (zone->lock_count == 0))
     {
-        mutex_unlock(&zone->lock_mutex);
         yassert(zone->lock_owner == (owner & ZDB_ZONE_MUTEX_LOCKMASK_FLAG));
         yassert(zone->lock_count != 0);
+        abort(); // unreachable
     }
     
     if(zone->lock_reserved_owner != (secondary_owner & ZDB_ZONE_MUTEX_LOCKMASK_FLAG))
     {
-        mutex_unlock(&zone->lock_mutex);
         yassert(zone->lock_reserved_owner != (secondary_owner & ZDB_ZONE_MUTEX_LOCKMASK_FLAG));
+        abort(); // unreachable
     }
 #endif
     
@@ -848,7 +917,7 @@ zdb_zone_exchange_locks(zdb_zone *zone, u8 owner, u8 secondary_owner)
     
     while(zone->lock_count != 1)
     {
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
         zdb_zone_lock_monitor_waits(holder);
 #endif
         
@@ -860,10 +929,9 @@ zdb_zone_exchange_locks(zdb_zone *zone, u8 owner, u8 secondary_owner)
             debug_log_stacktrace(MODULE_MSG_HANDLE, MSG_WARNING, "zdb_zone_double_lock:");
         }
 #endif
-        
         cond_timedwait(&zone->lock_cond, &zone->lock_mutex, 100);
         
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
         zdb_zone_lock_monitor_resumes(holder);
 #endif
     }
@@ -880,20 +948,20 @@ zdb_zone_exchange_locks(zdb_zone *zone, u8 owner, u8 secondary_owner)
         cond_notify(&zone->lock_cond);
     }
     
-#if ZDB_HAS_MUTEX_DEBUG_SUPPORT
+#if ZDB_HAS_LOCK_DEBUG_SUPPORT
     zdb_zone_lock_monitor_exchanges(holder);
     zdb_zone_lock_monitor_release(holder);
 #endif
 
-    mutex_unlock(&zone->lock_mutex);
-    
 #if DNSCORE_HAS_MUTEX_DEBUG_SUPPORT
     zone->lock_trace = debug_stacktrace_get();
-    zone->lock_id = pthread_self();
+    zone->lock_id = thread_self();
     zone->lock_timestamp = timeus();
 
     zdb_zone_lock_set_add(zone);
 #endif
+
+    mutex_unlock(&zone->lock_mutex);
 }
 
 /** @} */
