@@ -1,36 +1,37 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
 /** @defgroup dnsdbzone Zone related functions
  *  @ingroup dnsdb
  *  @brief Internal functions for the database: zoned resource records label.
@@ -75,8 +76,7 @@ zdb_zone_label_create(const void *data)
 {
     zdb_zone_label* zone_label;
 
-    ZALLOC_OR_DIE(zdb_zone_label*, zone_label, zdb_zone_label,
-                  ZDB_ZONELABEL_TAG);
+    ZALLOC_OBJECT_OR_DIE(zone_label, zdb_zone_label, ZDB_ZONELABEL_TAG);
 
     zone_label->next = NULL;
     dictionary_init(&zone_label->sub);
@@ -91,15 +91,15 @@ zdb_zone_label_create(const void *data)
  * @brief INTERNAL callback, destroys a node instance and its collections.
  */
 
-void
-zdb_zone_label_destroy_callback(dictionary_node * zone_label_record)
+static void
+zdb_zone_label_destroy_callback(dictionary_node * zone_label_node)
 {
-    if(zone_label_record == NULL)
+    if(zone_label_node == NULL)
     {
         return;
     }
 
-    zdb_zone_label* zone_label = (zdb_zone_label*)zone_label_record;
+    zdb_zone_label *zone_label = (zdb_zone_label*)zone_label_node;
 
     /* detach is made by destroy */
 
@@ -113,7 +113,7 @@ zdb_zone_label_destroy_callback(dictionary_node * zone_label_record)
         zone_label->zone = NULL;
     }
 
-    ZFREE(zone_label_record, zdb_zone_label);
+    ZFREE_OBJECT(zone_label);
 }
 
 /**
@@ -170,13 +170,13 @@ zdb_zone_label_find(zdb * db, const dnsname_vector* origin) // mutex checked
  */
 
 zdb_zone_label*
-zdb_zone_label_find_nolock(zdb * db, dnsname_vector* origin)
+zdb_zone_label_find_nolock(zdb * db, const dnsname_vector* origin)
 {
     zdb_zone_label* zone_label;
     
     zone_label = db->root; /* the "." zone */
 
-    dnslabel_stack_reference sections = origin->labels;
+    const_dnslabel_stack_reference sections = origin->labels;
     s32 index = origin->size;
 
     /* look into the sub level */
@@ -257,7 +257,7 @@ zdb_zone_label_destroy(zdb_zone_label **zone_labelp)
 
     if(zone_label != NULL)
     {
-#ifdef DEBUG
+#if DEBUG
         log_debug5("zdb_zone_label_destroy: %{dnslabel}", zone_label->name);
 #endif
         
@@ -268,7 +268,7 @@ zdb_zone_label_destroy(zdb_zone_label **zone_labelp)
         
         if(zone != NULL)
         {
-#ifdef DEBUG
+#if DEBUG
             log_debug5("zdb_zone_label_destroy: %{dnsname}", zone->origin);
 #endif
             mutex_lock(&zone->lock_mutex);
@@ -283,7 +283,7 @@ zdb_zone_label_destroy(zdb_zone_label **zone_labelp)
         //zdb_zone_destroy(zone_label->zone);
         
         dnslabel_zfree(zone_label->name);
-        ZFREE(zone_label, zdb_zone_label);
+        ZFREE_OBJECT(zone_label);
         *zone_labelp = NULL;
     }
 }
@@ -340,7 +340,7 @@ zdb_zone_label_match(zdb * db, const dnsname_vector* origin,  // mutex checked
 }
 
 zdb_zone_label*
-zdb_zone_label_add_nolock(zdb * db, dnsname_vector* origin) // mutex checked
+zdb_zone_label_add_nolock(zdb * db, const dnsname_vector* origin) // mutex checked
 {
     zdb_zone_label* zone_label;
     
@@ -348,7 +348,7 @@ zdb_zone_label_add_nolock(zdb * db, dnsname_vector* origin) // mutex checked
 
     zone_label = db->root; /* the "." zone */
 
-    dnslabel_stack_reference sections = origin->labels;
+    const_dnslabel_stack_reference sections = origin->labels;
     s32 index = origin->size;
 
     /* look into the sub level */
@@ -441,7 +441,7 @@ zdb_zone_label_delete_process_callback(void *a, dictionary_node * node)
                 }
                 
                 dnslabel_zfree(zone_label->name);
-                ZFREE(zone_label, zdb_zone_label);
+                ZFREE_OBJECT(zone_label);
 
                 return COLLECTION_PROCESS_DELETENODE;
             }
@@ -471,7 +471,7 @@ zdb_zone_label_delete_process_callback(void *a, dictionary_node * node)
     }
     
     dnslabel_zfree(zone_label->name);
-    ZFREE(zone_label, zdb_zone_label);
+    ZFREE_OBJECT(zone_label);
 
     return COLLECTION_PROCESS_DELETENODE;
 }
@@ -519,7 +519,7 @@ zdb_zone_label_delete(zdb * db, dnsname_vector* name) // mutex checked
     return err;
 }
 
-#ifdef DEBUG
+#if DEBUG
 
 /**
  * DEBUG
