@@ -1,36 +1,37 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
 /** @defgroup config Configuration handling
  *  @ingroup yadifad
  *  @brief
@@ -51,16 +52,13 @@ extern "C" {
 #include <errno.h>
 #include <stddef.h>
 
-#include "config.h"
+#include "server-config.h"
 
 #include <dnscore/rfc.h>
 #include <dnscore/ptr_set.h>
-    
+#include <dnscore/acl.h>
+#include <dnscore/dnscore-release-date.h>
 #include <dnsdb/zdb_types.h>
-    
-#include "zone.h"
-#include "database.h"
-#include "acl.h"
 
     /*    ------------------------------------------------------------    */
 
@@ -81,19 +79,19 @@ extern "C" {
 #define     MAX_CONFIG_STRING           50
 #define     PRINTARGLEN                 10
 
-#define     PROGRAM_NAME                PACKAGE
+#define     PROGRAM_NAME                "yadifad"
 #define     PROGRAM_VERSION             PACKAGE_VERSION
-#define     RELEASEDATE                 "2019-02-11"
+#define     RELEASEDATE                 YADIFA_DNSCORE_RELEASE_DATE
 
     /* List of default values for the different configuration parameters */
 #define     S_CONFIGDIR                 SYSCONFDIR "/"
-#define     S_CONFIGFILE                PACKAGE ".conf"
-#define     S_CONFIGFILEDYNAMIC         PACKAGE ".conf.dyn"
+#define     S_CONFIGFILE                PROGRAM_NAME ".conf"
+#define     S_CONFIGFILEDYNAMIC         PROGRAM_NAME ".conf.dyn"
 #define     S_DATAPATH                  LOCALSTATEDIR "/zones/"
 #define     S_XFRPATH                   LOCALSTATEDIR "/zones/xfr/"
 #define     S_KEYSPATH                  LOCALSTATEDIR "/zones/keys/"        /** Keys should not be in "shared" */
 #define     S_LOGPATH                   LOGDIR
-#define     S_PIDFILE                   LOCALSTATEDIR "/run/" PACKAGE ".pid"
+#define     S_PIDFILE                   LOCALSTATEDIR "/run/" PROGRAM_NAME ".pid" /// @TODO 20200623 edf -- use RUNSTATEDIR instead
 
 #define     S_VERSION_CHAOS             PACKAGE_VERSION                  /* limit the size */ 
 #define     S_HOSTNAME_CHAOS            NULL
@@ -113,9 +111,10 @@ extern "C" {
      *       Run mode of the program */
 #define     S_RUNMODE                   RUNMODE_CONTINUE_CLEAN
     
-#define     S_NETWORK_MODEL             "0"
+#define     S_NETWORK_MODEL             "2"
 #define     S_INTERACTIVE               "0"
 #define     S_LOG_FROM_START            "0"
+#define     S_LOG_FILES_DISABLED        "0"
 
     /* */
 #define     S_CPU_COUNT_OVERRIDE        "0" /* max 256 */
@@ -123,7 +122,20 @@ extern "C" {
 #define     S_DNSSEC_THREAD_COUNT       "0" /* max 1024 */
 
 #define     S_ZONE_LOAD_THREAD_COUNT    "1"     // disk
+#define     ZONE_LOAD_THREAD_COUNT_MIN 1
+#define     ZONE_LOAD_THREAD_COUNT_MAX 4
+    
+#define     S_ZONE_SAVE_THREAD_COUNT    "1"     // disk
+#define     ZONE_SAVE_THREAD_COUNT_MIN 1
+#define     ZONE_SAVE_THREAD_COUNT_MAX 4
+
+#define     S_ZONE_UNLOAD_THREAD_COUNT    "1"     // cpu
+#define     ZONE_UNLOAD_THREAD_COUNT_MIN 1
+#define     ZONE_UNLOAD_THREAD_COUNT_MAX 4
+
 #define     S_ZONE_DOWNLOAD_THREAD_COUNT "4"    // network
+#define     ZONE_DOWNLOAD_THREAD_COUNT_MIN 1
+#define     ZONE_DOWNLOAD_THREAD_COUNT_MAX 16
     
     /* Chroot, uid and gid */
 #define     S_CHROOT                    "0"
@@ -134,10 +146,25 @@ extern "C" {
 
     /** \def S_LISTEN
      *       Listening to all interfaces */
-#define     S_LISTEN                    "0.0.0.0"
+#define     S_LISTEN                    "0.0.0.0;::0"
+
+#if HAS_SYSTEMD_RESOLVED_AVOIDANCE
+#define     S_DO_NOT_LISTEN             "127.0.0.53 port 53"
+#else
+#define     S_DO_NOT_LISTEN             ""
+#endif
+    
+#define     MAX_INTERFACES              256
+    
 #define     S_TOTALINTERFACES           1
-#define     S_MAX_TCP_QUERIES           "16"    /* max 512 */
+#define     S_MAX_TCP_QUERIES           "128"   /* max 255 */
 #define     S_TCP_QUERY_MIN_RATE        "512"   /* bytes per second minimum rate */
+    
+#define     S_TCP_QUEUE_SIZE            "1024"
+#define     S_TCP_QUEUE_SIZE_MIN        64
+#define     S_TCP_QUEUE_SIZE_MAX        65536
+
+#define     S_MAX_AXFR                  "10"
 
 #define     S_AXFR_MAX_RECORD_BY_PACKET "0"    /** No limit.  Old applications can only work with this set to 1 */
 #define     S_AXFR_PACKET_SIZE_MAX      "4096" /** plus TSIG */
@@ -145,6 +172,12 @@ extern "C" {
 #define     S_AXFR_RETRY_DELAY          "600"
 #define     S_AXFR_RETRY_JITTER         "180"
 
+#if HAS_NON_AA_AXFR_SUPPORT
+#define     S_AXFR_STRICT_AUTHORITY     "0"
+#else
+#define     S_AXFR_STRICT_AUTHORITY     "1"
+#endif
+
 #define     S_AXFR_RETRY_FAILURE_DELAY_MULTIPLIER "5"
 #define     AXFR_RETRY_FAILURE_DELAY_MULTIPLIER_MIN 0
 #define     AXFR_RETRY_FAILURE_DELAY_MULTIPLIER_MAX 86400
@@ -153,7 +186,10 @@ extern "C" {
 #define     AXFR_RETRY_FAILURE_DELAY_MULTIPLIER_MAX_MIN 0
 #define     AXFR_RETRY_FAILURE_DELAY_MULTIPLIER_MAX_MAX 604800
 
-    
+#define     S_SERVER_RW_BACKLOG_QUEUE_SIZE   "16384"
+#define     SERVER_RW_BACKLOG_QUEUE_SIZE_MIN 0x001000
+#define     SERVER_RW_BACKLOG_QUEUE_SIZE_MAX 0x100000
+
 #define     S_XFR_CONNECT_TIMEOUT       "5"    /* seconds */
     
 #define     S_QUERIES_LOG_TYPE          "1"    /* 0: none, 1: YADIFA, 2: bind 3:both */
@@ -199,14 +235,15 @@ extern "C" {
 #define     S_ZONE_FLAG_MAINTAIN_DNSSEC  "1"
 #define     S_ZONE_FLAG_TRUE_MULTIMASTER "0"
 #define     S_ZONE_FLAG_RRSIG_NSUPDATE_ALLOWED "0"
+    
 #define     S_MULTIMASTER_RETRIES       "0"             // in a multimaster setup, how many retries before changing master
                                                         // 0 is perfectly fine except in true-multimaster mode where the resource cost
                                                         // asks for some caution.  In that case 60 would be a good choice. Maximum is 255
 #define     S_ZONE_DNSSEC_DNSSEC        "off"
     
-#define     S_JOURNAL_SIZE_KB_DEFAULT   "0"
-#define     S_JOURNAL_SIZE_KB_MIN       0               // less than this is asking for trouble
-#define     S_JOURNAL_SIZE_KB_MAX       8388608         // 8*2^20 => 8GB
+#define     S_JOURNAL_SIZE_KB_DEFAULT   "0"             // 0 means "automatic"
+#define     S_JOURNAL_SIZE_KB_MIN       0               // less than 64KB is asking for trouble
+#define     S_JOURNAL_SIZE_KB_MAX       3698688         // 3GB
 
     /*    ------------------------------------------------------------    */
 
@@ -247,16 +284,23 @@ extern "C" {
 #define     SERVER_FL_INTERACTIVE       0x20   
 #define     SERVER_FL_DYNAMIC_PROVISIONING 0x40
 #define     SERVER_FL_LOG_FROM_START    0x8000
+#define     SERVER_FL_LOG_FILE_DISABLED 0x4000
 
     /* IP flags */
 #define     IP_FLAGS_IPV4               0x01
 #define     IP_FLAGS_IPV6               0x02
 
+#if 0 /* fix */
+#else
 #define     SIGNATURE_VALIDITY_INTERVAL_MIN     7       /* 7  days */
+#endif
 #define     SIGNATURE_VALIDITY_INTERVAL_MAX     366     /* 366 days */
 #define     SIGNATURE_VALIDITY_INTERVAL_S       86400   /* seconds for that unit */
-    
+
+#if 0 /* fix */
+#else
 #define     SIGNATURE_VALIDITY_REGENERATION_MIN 24      /* 1 day  */
+#endif
 #define     SIGNATURE_VALIDITY_REGENERATION_MAX 168     /* 7 days */
 #define     SIGNATURE_VALIDITY_REGENERATION_S   3600    /* seconds for that unit */
     
@@ -289,6 +333,12 @@ struct tcp
     int sockfd;
 };
 
+#ifdef WIN32
+#ifdef interface
+#undef interface
+#endif
+#endif // WIN32
+
 typedef struct interface interface;
 struct interface
 {
@@ -323,15 +373,21 @@ typedef struct config_data config_data;
 
 #define YGCONFIG_TAG 0x4749464e4f434759
 
+#define CONFIG_SIGNATURE_TYPE_CONFIGURABLE 0
+
 struct config_data
 {
-    /* Which are the interfaces to listen at */
+    // Which are the interfaces to listen to
     host_address                                                *listen;
+    // Which are the interfaces to not listen to
+    host_address                                         *do_not_listen;
 
     /* General variables */
     char                                                     *data_path; /* zones */
     char                                                      *xfr_path; /* full and incremental images base ... */
+#ifndef WIN32
     char                                                   *chroot_path; /* chroot point */
+#endif
     char                                                      *log_path; /* log files */
     char                                                     *keys_path; /* keys */
     char                                                   *config_file; /* config */
@@ -342,6 +398,10 @@ struct config_data
     char                                                *hostname_chaos;
     char                                                *serverid_chaos;
     char                                                   *server_port;
+    
+#if HAS_EVENT_DYNAMIC_MODULE
+    ptr_vector                                          dynamic_modules;
+#endif
 
     pid_t                                                           pid;
 
@@ -351,14 +411,19 @@ struct config_data
 
     int                                                total_interfaces;
     int                                              cpu_count_override;
-    int                                         thread_count_by_address;
+    s32                                         thread_count_by_address;
     int                                            thread_affinity_base;
     int                                      thread_affinity_multiplier;
+#if DATABASE_ZONE_RRSIG_THREAD_POOL
     int                                             dnssec_thread_count;
+#endif
     int                                          zone_load_thread_count;
+    int                                         zone_store_thread_count;
+    int                                        zone_unload_thread_count;
     int                                      zone_download_thread_count;
     int                                                 max_tcp_queries;
     int                                              tcp_query_min_rate;
+    int                                                  tcp_queue_size;
     int                                       axfr_max_record_by_packet;
     int                                            axfr_max_packet_size;
     int                                                axfr_retry_delay;
@@ -366,14 +431,16 @@ struct config_data
     u32                             axfr_retry_failure_delay_multiplier;
     u32                                    axfr_retry_failure_delay_max;
     int                                             xfr_connect_timeout;
-    int                                           statistics_max_period;
+    u32                                           statistics_max_period;
     int                                                  edns0_max_size;
-    int                                                   network_model; // 0: default MT, 1: experimental RqW 
+    int                                                   network_model; // 0: default MT, 1: experimental RqW
+    u32                                       worker_backlog_queue_size;
     bool                                          axfr_compress_packets;
+    bool                                          axfr_strict_authority; // if the AA bit isn't set, AXFR is rejected
 
     /**/
 
-    access_control                                                   ac;
+    access_control                                                  *ac;
 
     /**/
 
@@ -381,6 +448,7 @@ struct config_data
     uid_t                                                           uid;
 
     u16                                                   process_flags;
+    u16                                               server_port_value;
 
     //u8                                                               ip;
 
@@ -396,13 +464,19 @@ struct config_data
     u32                                             sig_validity_interval;
     u32                                         sig_validity_regeneration;
     u32                                               sig_validity_jitter;
+#if CONFIG_SIGNATURE_TYPE_CONFIGURABLE
     u16                                                  sig_signing_type;
+#endif
 #endif
 
     double                                          tcp_query_min_rate_us;
 
     bool                                                         chrooted;
     bool                                                       reloadable;
+
+    bool                                                    hidden_master;
+
+    bool                                                   check_policies;
 };
 
 /**
@@ -413,6 +487,9 @@ struct config_data
  * 
  */
 
+struct zone_desc_s;
+typedef struct zone_desc_s zone_desc_s;
+
 typedef ya_result config_section_zone_filter_callback(zone_desc_s *, void *);
 
 #ifndef CONFS_MAIN_C_
@@ -421,7 +498,6 @@ extern config_data                         *g_config;
 
 #endif
 
-#include    "zone.h"
 
 /**
  * @brief Tool function printing all the known names in a table.
@@ -437,12 +513,14 @@ void config_logger_cleardefault();
 ya_result yadifad_config_init();
 ya_result yadifad_config_cmdline(int argc, char **argv);
 ya_result yadifad_config_read(const char *config_file);
-ya_result yadifad_config_finalise();
+ya_result yadifad_config_finalize();
 
 ya_result yadifad_config_update(const char *config_file);
 
 ya_result yadifad_config_update_zone(const char *config_file, const ptr_set *fqdn);
 
+void yadifad_print_usage(const char *name);
+
 /*    ------------------------------------------------------------    */
 
 ya_result confs_set_dnssec(const char *value, u32 *dest, anytype notused);
