@@ -1,36 +1,37 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
 /** @defgroup 
  *  @ingroup 
  *  @brief 
@@ -42,22 +43,21 @@
  *----------------------------------------------------------------------------*/
 
 #include "server-config.h"
-#include "server-config.h"
 
 #if HAS_PTHREAD_SETNAME_NP
-#ifdef DEBUG
+#if DEBUG
 #define _GNU_SOURCE 1
 #endif
 #endif
 
-#include <pthread.h>
-
-#include "config.h"
+#include <dnscore/thread.h>
 
 #include <dnscore/dnscore.h>
 #include <dnscore/logger.h>
 #include <dnscore/threaded_queue.h>
+#include <dnscore/thread.h>
 #include <dnscore/thread_pool.h>
+#include <dnscore/message.h>
 #include <dnsdb/zdb_types.h>
 
 #include "database.h"
@@ -85,14 +85,12 @@
  */
 
 /**
- * 
- * Move this into YADIFAD
  * 
  * The dynupdate service loads the next update from the queue and runs it.
  */
 
-static threaded_queue dynupdate_query_service_queue = THREADED_QUEUE_NULL;
-static volatile pthread_t dynupdate_query_service_thread_id = 0;
+static threaded_queue dynupdate_query_service_queue = THREADED_QUEUE_EMPTY;
+static volatile thread_t dynupdate_query_service_thread_id = 0;
 static volatile bool dynupdate_query_service_thread_run = FALSE;
 
 typedef struct dynupdate_query_service_args dynupdate_query_service_args;
@@ -104,34 +102,28 @@ struct dynupdate_query_service_args
     zdb            *db;
     message_data   *mesg;
     u32             timestamp;
+    int             sockfd;
 };
 
-static void*
+static const u32 dynupdate_query_timeout_seconds = 2;
+
+static noreturn void*
 dynupdate_query_service_thread(void *args)
 {
+    (void)args;
     thread_pool_setup_random_ctx();
     
     log_debug("dynupdate_query_service_thread: service started");
 
-#if HAS_PTHREAD_SETNAME_NP
-#ifdef DEBUG
-#if __APPLE__
-    pthread_setname_np("dynupdate-query");
-#else
-    pthread_setname_np(pthread_self(), "dynupdate-query");
-#endif // __APPLE__
-#endif
-#endif
+    thread_set_name("dynupdate-query", 0, 0);
     
-#if DNSCORE_HAS_LOG_THREAD_TAG_ALWAYS_ON
-    thread_set_tag(pthread_self(), "dynupdte");
+#if DNSCORE_HAS_LOG_THREAD_TAG
+    logger_handle_set_thread_tag("dynupdte");
 #endif
     
     for(;;)
     {
-#if UDP_USE_MESSAGES
 dynupdate_query_service_thread_main_loop:
-#endif
 
         if(dnscore_shuttingdown())
         {
@@ -163,7 +155,7 @@ dynupdate_query_service_thread_main_loop:
         
         u32 now = time(NULL);
         
-        if((now - parms->timestamp) <= 3) /** @todo 20121106 edf -- set this as a configuration parameter (dynupdate-processing-timeout or something) */
+        if((now - parms->timestamp) <= dynupdate_query_timeout_seconds)
         {
             /* process */
 
@@ -173,68 +165,67 @@ dynupdate_query_service_thread_main_loop:
             /* use the same scheduling mechanism as for TCP */
 
             log_info("update (%04hx) %{dnsname} %{dnstype} (%{sockaddr})",
-                                        ntohs(MESSAGE_ID(mesg->buffer)),
-                                        mesg->qname,
-                                        &mesg->qtype,
-                                        &mesg->other.sa);
+                                        ntohs(message_get_id(mesg)),
+                                        message_get_canonised_fqdn(mesg),
+                                        message_get_query_type_ptr(mesg),
+                                        message_get_sender_sa(mesg));
 
-            finger_print return_code = database_update(database, mesg);
+            ya_result ret = database_update(database, mesg);
 
-            if(FAIL(return_code))
+            if(FAIL(ret))
             {
-                log_err("update (%04hx) %{dnsname} %{dnstype} failed: %r",
-                        ntohs(MESSAGE_ID(mesg->buffer)),
-                        mesg->qname,
-                        &mesg->qtype,
-                        return_code);
+                if(message_get_query_type(mesg) == TYPE_SOA)
+                {
+                    if(ret == ZDB_JOURNAL_MUST_SAFEGUARD_CONTINUITY)
+                    {
+                        log_info("update (%04hx) %{dnsname} temporary failure: zone file must be stored: %r",
+                                 ntohs(message_get_id(mesg)),
+                                 message_get_canonised_fqdn(mesg),
+                                 ret);
+                    }
+                    else
+                    {
+                        log_warn("update (%04hx) %{dnsname} failed: %r",
+                                 ntohs(message_get_id(mesg)),
+                                 message_get_canonised_fqdn(mesg),
+                                 ret);
+                    }
+                }
+                else
+                {
+                    if(ret == ZDB_JOURNAL_MUST_SAFEGUARD_CONTINUITY)
+                    {
+                        log_info("update (%04hx) %{dnsname} %{dnstype} temporary failure: zone file must be stored: %r",
+                                 ntohs(message_get_id(mesg)),
+                                 message_get_canonised_fqdn(mesg),
+                                 message_get_query_type_ptr(mesg),
+                                 ret);
+                    }
+                    else
+                    {
+                        log_warn("update (%04hx) %{dnsname} %{dnstype} failed: %r",
+                                ntohs(message_get_id(mesg)),
+                                message_get_canonised_fqdn(mesg),
+                                message_get_query_type_ptr(mesg),
+                                ret);
+                    }
+                }
             }
 
-            //local_statistics->udp_fp[mesg->status]++;
+            //local_statistics->udp_fp[message_get_status(mesg)]++;
 
 #if !HAS_DROPALL_SUPPORT
 
-            s32 sent;
+            ssize_t sent;
 
-#ifdef DEBUG
-            log_debug("dynupdate_query_service_thread: sendto(%d, %p, %d, %d, %{sockaddr}, %d)", mesg->sockfd, mesg->buffer, mesg->send_length, 0, (struct sockaddr*)&mesg->other.sa, mesg->addr_len);
-            log_memdump_ex(g_server_logger, MSG_DEBUG5, mesg->buffer, mesg->send_length, 16, OSPRINT_DUMP_HEXTEXT);
+#if DEBUG
+            log_debug("dynupdate_query_service_thread: sendto(%d, %p, %d, %d, %{sockaddr}, %d)",
+                    parms->sockfd, message_get_buffer_const(mesg), message_get_size(mesg), 0,
+                    message_get_sender_sa(mesg), message_get_sender_size(mesg));
+            log_memdump_ex(g_server_logger, MSG_DEBUG5, message_get_buffer_const(mesg), message_get_size(mesg), 16, OSPRINT_DUMP_HEXTEXT);
 #endif
             
-#if !UDP_USE_MESSAGES
-            while((sent = sendto(mesg->sockfd, mesg->buffer, mesg->send_length, 0, (struct sockaddr*)&mesg->other.sa, mesg->addr_len)) < 0)
-            {
-                int error_code = errno;
-
-                if(error_code != EINTR)
-                {
-                    /** @warning server_st_process_udp needs to be modified */
-                    //log_err("sendto: %r", MAKE_ERRNO_ERROR(error_code));
-
-                    free(parms);
-                    free(mesg);
-
-                    return NULL/*ERROR*/;
-                }
-            }
-#else
-            struct iovec    udp_iovec;
-            struct msghdr   udp_msghdr;
-            
-            udp_iovec.iov_base = &mesg->buffer;
-            udp_iovec.iov_len = mesg->send_length;
-
-            udp_msghdr.msg_name = &mesg->other.sa;
-            udp_msghdr.msg_namelen = mesg->addr_len;
-            udp_msghdr.msg_iov = &udp_iovec;
-            udp_msghdr.msg_iovlen = 1;
-            udp_msghdr.msg_control = NULL;
-            udp_msghdr.msg_controllen = 0;
-            udp_msghdr.msg_flags = 0;
-
-#ifdef DEBUG
-            log_debug("sendmsg(%d, %p, %d", mesg->sockfd, &udp_msghdr, 0);
-#endif
-            while( (sent = sendmsg(mesg->sockfd, &udp_msghdr, 0)) < 0)
+            while( (sent = message_send_udp(mesg, parms->sockfd)) < 0)
             {
                 int error_code = errno;
 
@@ -243,15 +234,13 @@ dynupdate_query_service_thread_main_loop:
                     /** @warning server_st_process_udp needs to be modified */
                     
                     log_err("update (%04hx) %{dnsname} %{dnstype} send failed: %r",
-                        ntohs(MESSAGE_ID(mesg->buffer)),
-                        mesg->qname,
-                        &mesg->qtype,
+                        ntohs(message_get_id(mesg)),
+                        message_get_canonised_fqdn(mesg),
+                        message_get_query_type_ptr(mesg),
                         MAKE_ERRNO_ERROR(error_code));
-                    
-
 
                     free(parms);
-                    free(mesg);
+                    message_free(mesg);
 
                     /**********************************************************
                      * GOTO !
@@ -268,13 +257,13 @@ dynupdate_query_service_thread_main_loop:
                      *********************************************************/
                 }
             }
-#endif
+            
             //local_statistics->udp_output_size_total += sent;
 
-            if(sent != mesg->send_length)
+            if((size_t)sent != message_get_size(mesg))
             {
                 /** @warning server_st_process_udp needs to be modified */
-                log_err("short byte count sent (%i instead of %i)", sent, mesg->send_length);
+                log_err("short byte count sent (%lli instead of %i)", sent, message_get_size(mesg));
             }
 #else
             log_debug("dynupdate_query_service_thread: drop all");
@@ -283,20 +272,21 @@ dynupdate_query_service_thread_main_loop:
         }
 
         free(parms);
-        free(mesg);
+        message_free(mesg);
     }
     
     log_debug("dynupdate_query_service_thread: service stopped");
     
     thread_pool_destroy_random_ctx();
     
-#if DNSCORE_HAS_LOG_THREAD_TAG_ALWAYS_ON
-    thread_clear_tag(pthread_self());
+#if DNSCORE_HAS_LOG_THREAD_TAG
+    logger_handle_clear_thread_tag();
 #endif
     
-    pthread_exit(NULL); /* not from the pool, so it's the way */
-    
-    return NULL;
+    thread_exit(NULL); /* not from the pool, so it's the way */
+
+    // unreachable
+    // return NULL;
 }
 
 ya_result
@@ -315,8 +305,8 @@ dynupdate_query_service_start()
     
     threaded_queue_init(&dynupdate_query_service_queue, 256);
     
-    pthread_t id;
-    if(pthread_create(&id, NULL, dynupdate_query_service_thread, NULL) != 0)
+    thread_t id;
+    if(thread_create(&id, dynupdate_query_service_thread, NULL) != 0)
     {
         log_crit("failed to start dynamic query service thread");
         
@@ -344,7 +334,7 @@ dynupdate_query_service_stop()
     
     threaded_queue_enqueue(&dynupdate_query_service_queue, NULL);
     
-    pthread_join(dynupdate_query_service_thread_id, NULL);
+    thread_join(dynupdate_query_service_thread_id, NULL);
     
     log_debug("emptying dynamic update queue");
     
@@ -369,39 +359,28 @@ dynupdate_query_service_stop()
 }
 
 ya_result
-dynupdate_query_service_enqueue(zdb *db, message_data *msg)
+dynupdate_query_service_enqueue(zdb *db, message_data *mesg, int sockfd)
 {
     if(dynupdate_query_service_thread_id == 0)
     {
         return SERVICE_NOT_RUNNING;
     }
-    
-    message_data *mesg_clone;
-    MALLOC_OR_DIE(message_data*, mesg_clone, sizeof(message_data), MESGDATA_TAG);
-    memcpy(mesg_clone, msg, sizeof(message_data));
     // ensure the original message cannot be used anymore
-    
-#if HAS_TSIG_SUPPORT
-    msg->tsig.tsig = NULL;
-#endif
-    msg->received = 0;
-    msg->send_length = 0;
-    
     struct dynupdate_query_service_args *parms;
-    MALLOC_OR_DIE(struct dynupdate_query_service_args *, parms, sizeof(dynupdate_query_service_args), DYNUPQSA_TAG);
+    MALLOC_OBJECT_OR_DIE(parms, dynupdate_query_service_args, DYNUPQSA_TAG);
     parms->db = db;
-    parms->mesg = mesg_clone;
-        
+    parms->mesg = message_dup(mesg);
     parms->timestamp = time(NULL);
+    parms->sockfd = sockfd;
     
     threaded_queue_enqueue(&dynupdate_query_service_queue, parms);
-    
+
+#if DNSCORE_HAS_TSIG_SUPPORT
+    message_tsig_clear_key(mesg);
+#endif
+    message_set_size(mesg, 0);  // resets the message size
+
     return SUCCESS;
 }
 
-/*    ------------------------------------------------------------    */
-
 /** @} */
-
-/*----------------------------------------------------------------------------*/
-
