@@ -1,36 +1,37 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
 /**
  *  @defgroup server Server
  *  @ingroup yadifad
@@ -38,7 +39,6 @@
  *
  * @{
  */
-/*----------------------------------------------------------------------------*/
 
 #pragma once
 
@@ -48,6 +48,7 @@
 
 #include <dnscore/message.h>
 #include <dnscore/logger.h>
+#include <dnscore/service.h>
 
 #include "confs.h"
 
@@ -65,8 +66,6 @@ extern volatile int program_mode;
 #define SOA_MIN_RETRY   60
 #define SOA_MIN_EXPIRE  60
 
-#define ANCILIARY_BUFFER_SIZE 64
-
 #define TPROCPRM_TAG 0x4d5250434f525054
 #define POLLFDBF_TAG 0x464244464c4c4f50
 
@@ -83,12 +82,14 @@ extern volatile int program_mode;
 struct server_desc_s
 {
     ya_result (* const context_init)(int workers_per_interface);
-    ya_result (* const loop)();
+    ya_result (* const loop)(struct service_worker_s *worker);
     const char *name;
 };
 
 typedef struct server_statistics_t server_statistics_t;
 
+#define SUCCESS_DROPPED ((int)0x80ff0001)
+
 /**
  * volatile is only needed for variables changed by another thread
  */
@@ -106,9 +107,13 @@ struct server_statistics_t
     volatile u64 loop_rate_elapsed;
     
     /* udp */
-
+#ifndef WIN32
     volatile u64 udp_input_count __attribute__ ((aligned (64)));
     volatile u64 udp_queries_count __attribute__ ((aligned (64)));
+#else
+    volatile u64 udp_input_count;
+    volatile u64 udp_queries_count;
+#endif
     volatile u64 udp_notify_input_count;
     volatile u64 udp_updates_count;
     volatile u64 udp_dropped_count;
@@ -144,20 +149,31 @@ struct server_statistics_t
     volatile u64 tcp_fp[SERVER_STATISTICS_ERROR_CODES_COUNT];
 };
 
+struct network_thread_context_base_s
+{
+    struct service_worker_s *worker;
+    server_statistics_t *statisticsp;
+    thread_t idr;
+    int sockfd;
+    u16 idx;
+    volatile bool must_stop;
+};
+
+typedef struct network_thread_context_base_s network_thread_context_base_t;
+
+void server_process_message_udp_set_database(zdb *db);
+
+int server_process_message_udp(network_thread_context_base_t *ctx, message_data *mesg);
+
 #define TCPSTATS(__field__) mutex_lock(&server_statistics.mtx);server_statistics. __field__ ;mutex_unlock(&server_statistics.mtx)
 
 #ifndef SERVER_C_
 extern server_statistics_t server_statistics;
-
 #endif
 
 
 
-void tcp_send_message_data(message_data* mesg);
-
-void server_process_tcp(zdb *database, int sockfd);
-
-void log_msghdr(logger_handle* hndl, u32 level, struct msghdr *hdr);
+void server_process_tcp(int sockfd);
 
 /**
  * Initialises the DNS service.
@@ -200,6 +216,28 @@ ya_result server_service_start_and_wait();
 
 ya_result server_service_wait();
 
+/**
+ * Tells the DNS service to reconfigure at the earliest convenience
+ * 
+ * @return an error code
+ */
+
+ya_result server_service_reconfigure();
+
+/**
+ * Tell the DNS service to stop, do not wait for completion.
+ *
+ * @return an error code
+ */
+
+
+ya_result server_service_stop_nowait();
+
+/**
+ * Closes all server sockets (UDP/TCP)
+ */
+
+void server_context_close();
 /**
  * Stops the DNS service.
  * 
@@ -214,6 +252,6 @@ ya_result server_service_stop();
  * @return an error code
  */
 
-ya_result server_service_finalise();
+ya_result server_service_finalize();
 
 /** @} */
