@@ -1,36 +1,37 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2019, EURid vzw. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2021, EURid vzw. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright
+ *          notice, this list of conditions and the following disclaimer in the
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be
+ *          used to endorse or promote products derived from this software
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
+
 /** @defgroup ### #######
  *  @ingroup yadifad
  *  @brief
@@ -44,12 +45,17 @@
 /*------------------------------------------------------------------------------
  *
  * USE INCLUDES */
-#include	"config.h"
+#include "server-config.h"
 
 #include <dnscore/dnsname.h>
 #include <dnscore/rfc.h>
 #include <dnscore/mutex.h>
 #include <dnscore/ptr_set.h>
+#include <dnscore/logger.h>
+
+
+
+#define MAINTAIN_ONLY_AT_DIFF_AND_REPLAY 0
 
 typedef struct zone_data_set zone_data_set;
 
@@ -198,11 +204,12 @@ zone_desc_s *zone_acquirebydnsname(const u8 *name);
 
 
 
+
+
 /*
  * functions used for removing a zone_desc
  */
 
-void zone_setmodified(zone_desc_s *zone_desc, bool v);
 void zone_setloading(zone_desc_s *zone_desc, bool v);
 void zone_setmustsavefile(zone_desc_s *zone_desc, bool v);
 void zone_setmustsaveaxfr(zone_desc_s *zone_desc, bool v);
@@ -213,7 +220,7 @@ void zone_setdynamicupdating(zone_desc_s *zone_desc, bool v);
 
 bool zone_isidle(zone_desc_s *zone_desc);
 bool zone_isfrozen(zone_desc_s *zone_desc);
-bool zone_ismodified(zone_desc_s *zone_desc);
+
 bool zone_isloading(zone_desc_s *zone_desc);
 bool zone_mustsavefile(zone_desc_s *zone_desc);
 bool zone_mustsaveaxfr(zone_desc_s *zone_desc);
@@ -288,12 +295,15 @@ ya_result zone_setwithzone(zone_desc_s *zone_desc, zone_desc_s *src);
  * @return TRUE iff zone has ZONE_FLAG_MAINTAIN_DNSSEC.
  */
 
+#if HAS_MASTER_SUPPORT
 static inline bool
 zone_maintains_dnssec(zone_desc_s *zone_desc)
 {
     return (zone_desc->flags & ZONE_FLAG_MAINTAIN_DNSSEC) != 0;
 }
+#endif
 
+#if HAS_MASTER_SUPPORT
 static inline void
 zone_maintains_dnssec_set(zone_desc_s *zone_desc, bool enable)
 {
@@ -306,6 +316,7 @@ zone_maintains_dnssec_set(zone_desc_s *zone_desc, bool enable)
         zone_desc->flags &= ~ZONE_FLAG_MAINTAIN_DNSSEC;
     }
 }
+#endif
 
 static inline bool
 zone_is_auto_notify(zone_desc_s *zone_desc)
@@ -332,11 +343,13 @@ zone_is_drop_before_load(zone_desc_s *zone_desc)
     return (zone_desc->flags & ZONE_FLAG_DROP_BEFORE_LOAD) != 0;
 }
 
+#if HAS_MASTER_SUPPORT
 static inline bool
 zone_rrsig_nsupdate_allowed(zone_desc_s *zone_desc)
 {
     return (zone_desc->flags & ZONE_FLAG_RRSIG_NSUPDATE_ALLOWED) != 0;
 }
+#endif
 
 void zone_enqueue_command(zone_desc_s *zone_desc, u32 id, void* parm, bool has_priority);
 zone_command_s* zone_dequeue_command(zone_desc_s *zone_desc);
@@ -386,6 +399,7 @@ static inline bool zone_is_true_multimaster(const zone_desc_s *zone_desc)
 }
 
 void zone_set_status(zone_desc_s *zone_desc, u32 flags);
+u32 zone_get_set_status(zone_desc_s *zone_desc, u32 flags);
 void zone_clear_status(zone_desc_s *zone_desc, u32 flags);
 u32 zone_get_status(const zone_desc_s *zone_desc);
 
@@ -393,5 +407,14 @@ void zone_dnssec_status_update(zdb_zone *zone);
 
 u8 zone_policy_guess_dnssec_type(zdb_zone *zone);
 
-/** @} */
+u32 zone_policy_get_earliest_queued_key_generation(zone_desc_s *zone_desc);
+u32 zone_policy_set_earliest_queued_key_generation(zone_desc_s *zone_desc, u32 epoch);
+u32 zone_policy_clear_earliest_queued_key_generation(zone_desc_s *zone_desc, u32 epoch);
 
+struct dnssec_policy_key_suite;
+
+bool zone_policy_key_suite_is_marked_processed(zone_desc_s *zone_desc, const struct dnssec_policy_key_suite *kr);
+bool zone_policy_key_suite_mark_processed(zone_desc_s *zone_desc, const struct dnssec_policy_key_suite *kr);
+void zone_policy_key_suite_unmark_processed(zone_desc_s *zone_desc, const struct dnssec_policy_key_suite *kr);
+
+/** @} */
